﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Collections;
using NBarcodeApi;
using NSysSvcApi;
using _Native;

namespace Barcode
{
    public partial class Form1 : Form
    {
        public BarcodeApi barcode = null;
        public static SysSvcApi sysSvc = null;
        public const string title = "Barcode Example";
        public ArrayList specifiedKeys = null;
        public bool is2DScanner = false;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;
            this.KeyPreview = true;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                listView1.Location = new Point(0, 0);
            }

            // create system service object
            sysSvc = new SysSvcApi();
            HW_OPTIONS hwOptions = new HW_OPTIONS();
            // Check what type of scanner.
            sysSvc.GetHardwareOptions(ref hwOptions);
            if (hwOptions.Barcode == HW_TYPE_BARCODE.NONE)
            {
                sysSvc.Dispose();
                sysSvc = null;
                MessageBox.Show("Barcode is not found!");
                return;
            }
            if (hwOptions.Barcode == HW_TYPE_BARCODE._2D_SWD_H || hwOptions.Barcode == HW_TYPE_BARCODE._2D_SWD_M)
                is2DScanner = true;
            else
                is2DScanner = false;

            // If another application is already using the barcode, you must shut down the application.
            IntPtr hBarcodeEmulator = _Native.Utils.FindWindow(BarcodeEmulatorInfo.WindowClass, BarcodeEmulatorInfo.WindowTitle);
            if (hBarcodeEmulator != IntPtr.Zero)
                Utils.SendMessage(hBarcodeEmulator, Utils.WM_COMMAND, BarcodeEmulatorInfo.CloseID, 0);

            // create barcode object
            barcode = new BarcodeApi();
            BARCODE_RESULT result = barcode.Open();
            if (result != BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
            {
                MessageBox.Show(result.ToString());
                return;
            }

            // When decoding is successful, register a callback function to be executed.
            barcode.SetCallback(new BarcodeApi.BARCODECALLBACK(barcodeCallbackProc));

            // Setting for receiving the H/W keys.
            // For more information, please refer to SystemService_CheckKeyPresses example.
            specifiedKeys = new ArrayList();
            specifiedKeys.Add(Convert.ToInt32(Keys.F7));
            specifiedKeys.Add(Convert.ToInt32(Keys.F8));
            specifiedKeys.Add(Convert.ToInt32(Keys.F9));
            specifiedKeys.Add(Convert.ToInt32(Keys.F10));
            specifiedKeys.Add(Convert.ToInt32(Keys.F11));
            specifiedKeys.Add(Convert.ToInt32(Keys.F19));
            specifiedKeys.Add(Convert.ToInt32(0xEF));
            specifiedKeys.Add(Convert.ToInt32(0xF0));
            specifiedKeys.Add(Convert.ToInt32(0xF2));
            specifiedKeys.Add(Convert.ToInt32(0xF3));
            specifiedKeys.Add(Convert.ToInt32(0xD3));
            specifiedKeys.Add(Convert.ToInt32(0xD2));

            if (is2DScanner)
            {
                cbxLeaveLightOn.Items.Add("Be flashed on and off");
                cbxLeaveLightOn.Items.Add("Always on");
                bool alwaysOn = false;
                barcode.GetLeaveLightsOn(ref alwaysOn);
                cbxLeaveLightOn.SelectedIndex = alwaysOn ? 1 : 0;

                cbxLightMode.Items.Add("Neither aimers nor illumination");
                cbxLightMode.Items.Add("Illumination only");
                cbxLightMode.Items.Add("Aimers only");
                cbxLightMode.Items.Add("Both aimers and illumination");
                cbxLightMode.SelectedIndex = 3;
            }
            else
            {
                cbxLeaveLightOn.Items.Add("LeaveLightOn not supported");
                cbxLeaveLightOn.SelectedIndex = 0;
                cbxLightMode.Items.Add("LightMode not supported");
                cbxLightMode.SelectedIndex = 0;

                chkRaw.Enabled = false;
                cbxLeaveLightOn.Enabled = false;
                cbxLightMode.Enabled = false;
            }

            this.cbxLightMode.SelectedIndexChanged += new System.EventHandler(this.cbxLightMode_SelectedIndexChanged);
            this.cbxLeaveLightOn.SelectedIndexChanged += new System.EventHandler(this.cbxLeaveLightOn_SelectedIndexChanged);
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the barcode object
            if (barcode != null)
            {
                if (barcode.IsOpened())
                {
                    barcode.Stop();
                    barcode.Close();
                }
                barcode = null;
            }

            // destruction of the system service object
            if (sysSvc != null)
            {
                sysSvc.Dispose();
                sysSvc = null;
            }
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (specifiedKeys.Contains(e.KeyValue))
            {
                if (barcode != null)
                {
                    if(chkRaw.Checked)
                        barcode.StartScanRawData();
                    else
                        barcode.Start();
                }
            }
            base.OnKeyDown(e);
        }
        protected override void OnKeyUp(KeyEventArgs e)
        {
            if (specifiedKeys.Contains(e.KeyValue))
            {
                if (barcode != null)
                {
                    if(chkRaw.Checked)
                        barcode.StopScanRawData();
                    else
                        barcode.Stop();
                }
            }
            base.OnKeyUp(e);
        }

        public void vibratorCallbackProc(Object obj)
        {
            if (sysSvc == null)
                return;

            sysSvc.SetVibratorEnable(true);
            Thread.Sleep(250);
            sysSvc.SetVibratorEnable(false);
        }

        /// <summary>
        /// If the bar code decoding is successful, this function is executed.
        /// When this function returns, which is a barcode decoding process is complete.
        /// Therefore, within this function, you should not act as a destruction of the barcode object.
        /// </summary>
        public void barcodeCallbackProc()
        {
            byte[] byteValue = new byte[4096];
            int lenOfValue = 0;
            string Value = new string(new char[4096]);
            string TypeName = new string(new char[128]);
            string TypeID = new string(new char[10]);

            if (barcode == null)
                return;

            if (chkRaw.Checked)
            {
                BARCODE_RESULT result = barcode.GetBarcodeRawData(ref byteValue, ref lenOfValue, ref TypeName, ref TypeID);
                Value = String.Empty;
                for (int i = 0; i < lenOfValue; i++)
                    Value += String.Format("{0:X2} ", byteValue[i]);
            }
            else
            {
                barcode.GetBarcodeData(ref Value, ref TypeName, ref TypeID);
                Value = Value.Substring(0, Value.IndexOf(char.MinValue));
                TypeName = TypeName.Substring(0, TypeName.IndexOf(char.MinValue));
                TypeID = TypeID.Substring(0, TypeID.IndexOf(char.MinValue));
            }

            ListViewItem item = new ListViewItem(Convert.ToString(listView1.Items.Count + 1));
            item.SubItems.Add(Value);
            item.SubItems.Add(TypeName);
            listView1.Items.Add(item);
            listView1.EnsureVisible(listView1.Items.Count - 1);

            if (chkVibrator.Checked)
                ThreadPool.QueueUserWorkItem(new WaitCallback(vibratorCallbackProc));

            if (chkSound.Checked)
                Utils.PlaySound(@"\Windows\Success.wav");
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (barcode == null)
                return;

            if (chkRaw.Checked)
                barcode.StartScanRawData();
            else
                barcode.Start();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (barcode == null)
                return;

            if (chkRaw.Checked)
                barcode.StopScanRawData();
            else
                barcode.Stop();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            listView1.Items.Clear();
        }

        private void cbxLightMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (barcode == null)
                return;

            // 0 : Neither aimers nor illumination
            // 1 : Illumination only
            // 2 : Aimers only
            // 3 : Both aimers and illumination
            barcode.SetScanningLightsMode(cbxLightMode.SelectedIndex);
        }

        private void cbxLeaveLightOn_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (barcode == null)
                return;

            // true : Always on
            // false : Be flashed on and off
            barcode.SetLeaveLightsOn(cbxLeaveLightOn.SelectedIndex == 1 ? true : false);
        }
    }
}