﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NBarcodeApi;
using NSysSvcApi;
using _Native;

namespace Barcode_ImageCapturing
{
    public partial class Form1 : Form
    {
        public BarcodeApi barcode = null;
        public static SysSvcApi sysSvc = null;
        public const string title = "BarcodeImageCapturing Example";
        public bool bDoPreview = false;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
            }

            // create system service object
            sysSvc = new SysSvcApi();
            // When the PDA is entering suspend mode, or when waking up from suspend mode, 
            // register a callback function to be executed.
            sysSvc.SleepWakeupNotificationSet(new SysSvcApi.SleepWakeupNotifyCALLBACK(notifyCallbackProc));
            // Check what type of scanner.
            HW_OPTIONS hwOptions = new HW_OPTIONS();
            sysSvc.GetHardwareOptions(ref hwOptions);
            if (hwOptions.Barcode == HW_TYPE_BARCODE.NONE)
            {
                sysSvc.Dispose();
                sysSvc = null;
                MessageBox.Show("Barcode is not found!");
                return;
            }

            // Only 2D scanner supports an image capture.
            if (hwOptions.Barcode != HW_TYPE_BARCODE._2D_SWD_H && hwOptions.Barcode != HW_TYPE_BARCODE._2D_SWD_M)
            {
                MessageBox.Show("1D scanner is not supporting  image capturing ");
                btnStart.Enabled = false;
                btnStop.Enabled = false;
                btnCapture.Enabled = false;
                lblPreview.Text = "1D scanner is not supporting  image capturing";
                return;
            }

            // If another application is already using the barcode, you must shut down the application.
            IntPtr hBarcodeEmulator = _Native.Utils.FindWindow(BarcodeEmulatorInfo.WindowClass, BarcodeEmulatorInfo.WindowTitle);
            if (hBarcodeEmulator != IntPtr.Zero)
                Utils.SendMessage(hBarcodeEmulator, Utils.WM_COMMAND, BarcodeEmulatorInfo.CloseID, 0);

            // create barcode object
            barcode = new BarcodeApi();
            BARCODE_RESULT result = barcode.Open();
            if (result != BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
            {
                MessageBox.Show("Open : " + result.ToString());
                return;
            }

            result = barcode.InitCapture();
            if (result != BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
            {
                MessageBox.Show("InitCapture : " + result.ToString());
                barcode.Close();
                return;
            }

            result = barcode.SetPreviewHwnd(lblPreview.Handle);
            if (result != BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
            {
                MessageBox.Show("SetPreviewHWnd : " + result.ToString());
                barcode.DeinitCapture();
                barcode.Close();
                return;
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the barcode object
            if (barcode != null)
            {
                if (barcode.IsOpened())
                {
                    barcode.StopPreview();
                    barcode.Close();
                }
                barcode = null;
            }

            // destruction of the system service object
            if (sysSvc != null)
            {
                sysSvc.SleepWakeupNotificationReset();
                sysSvc.Dispose();
                sysSvc = null;
            }
        }

        /// <summary>
        /// When the PDA is entering suspend mode, or when waking up from suspend mode, this function is executed.
        /// </summary>
        /// <param name="enterSleep"></param>
        public void notifyCallbackProc(bool enterSleep)
        {
            // PDA is entering suspend mode
            if (enterSleep)
            {
                // Preview Stop
                if (bDoPreview)
                    barcode.StopPreview();
            }
            // waking up from suspend mode
            else
            {
                // Preview Start
                if (bDoPreview)
                    barcode.StartPreview();
            }
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            if (barcode == null)
                return;

            if (bDoPreview)
                return;

            BARCODE_RESULT result = barcode.StartPreview();
            if (result != BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
                MessageBox.Show("StartPreview : " + result.ToString());
            else
                bDoPreview = true;
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            if (barcode == null)
                return;

            if (!bDoPreview)
                return;

            BARCODE_RESULT result = barcode.StopPreview();
            if (result != BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
                MessageBox.Show("StopPreview : " + result.ToString());
            else
            {
                bDoPreview = false;
                lblPreview.Refresh();
            }
        }

        private void btnCapture_Click(object sender, EventArgs e)
        {
            if (barcode == null)
                return;

            BARCODECAPTUREPARAMS para = new BARCODECAPTUREPARAMS();
            para.ImgFormat = (byte)IMAGE_FORMAT.IMG_FORMAT_JPEG;
            para.ImgSize = (byte)IMAGE_SIZE.CAP_IMG_VGA;
            para.sFilePath = @"\Flash Disk\Test.jpg";

            BARCODE_RESULT result = barcode.DoCapture(ref para);
            if (result == BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Success.wav");
                if (bDoPreview)
                    barcode.StartPreview();
            }
            else
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("DoCapture : " + result.ToString());
            }
        }
    }
}