﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using NBarcodeApi;
using NSysSvcApi;
using _Native;

namespace Barcode_MultipleDecoding
{
    public partial class Form1 : Form
    {
        BarcodeApi barcode = null;
        SysSvcApi sysSvc = null;
        public ArrayList specifiedKeys = null;
        public const string title = "Multiple Decode Example";

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;
            this.KeyPreview = true;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                lvResult.Location = new Point(0, 0);
            }

            // create system service object
            sysSvc = new SysSvcApi();
            HW_OPTIONS hwOptions = new HW_OPTIONS();
            // Check what type of scanner.
            sysSvc.GetHardwareOptions(ref hwOptions);
            if (hwOptions.Barcode == HW_TYPE_BARCODE.NONE)
            {
                sysSvc.Dispose();
                sysSvc = null;
                MessageBox.Show("Barcode is not found!");
                return;
            }

            // Multiple decoding is supported only with 2D sw decoder
            if (hwOptions.Barcode != HW_TYPE_BARCODE._2D_SWD_H)
            {
                sysSvc.Dispose();
                sysSvc = null;
                MessageBox.Show("Not supported!");
                return;
            }

            barcode = new BarcodeApi();
            barcode.SetCallback(new BarcodeApi.BARCODECALLBACK(barcodeCallbackProc));
            BARCODE_RESULT result = barcode.Open();
            if (result != BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
            {
                sysSvc.Dispose();
                sysSvc = null;
                barcode.Close();
                barcode = null;
                MessageBox.Show(result.ToString());
                return;
            }

            // Setting for receiving the H/W keys.
            // For more information, please refer to SystemService_CheckKeyPresses example.
            specifiedKeys = new ArrayList();
            specifiedKeys.Add(Convert.ToInt32(Keys.F7));
            specifiedKeys.Add(Convert.ToInt32(Keys.F8));
            specifiedKeys.Add(Convert.ToInt32(Keys.F9));
            specifiedKeys.Add(Convert.ToInt32(Keys.F10));
            specifiedKeys.Add(Convert.ToInt32(Keys.F11));
            specifiedKeys.Add(Convert.ToInt32(Keys.F19));
            specifiedKeys.Add(Convert.ToInt32(0xEF));
            specifiedKeys.Add(Convert.ToInt32(0xF0));
            specifiedKeys.Add(Convert.ToInt32(0xF2));
            specifiedKeys.Add(Convert.ToInt32(0xF3));
            specifiedKeys.Add(Convert.ToInt32(0xD3));
            specifiedKeys.Add(Convert.ToInt32(0xD2));
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (barcode != null)
            {
                barcode.Close();
                barcode = null;
            }

            if (sysSvc != null)
            {
                sysSvc.Dispose();
                sysSvc = null;
            }
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (specifiedKeys.Contains(e.KeyValue))
            {
                if (barcode != null)
                {
                    barcode.StartScanMultiple(1000 * 60);
                }
            }
            base.OnKeyDown(e);
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            if (specifiedKeys.Contains(e.KeyValue))
            {
                if (barcode != null)
                {
                    barcode.StopScanMultiple();
                }
            }
            base.OnKeyUp(e);
        }

        public void barcodeCallbackProc()
        {
            string barcodeData = new string(new char[4096]);
            string typeName = new string(new char[128]);
            string typeID = new string(new char[4]);

            barcode.GetBarcodeData(ref barcodeData, ref typeName, ref typeID);

            AddItem(barcodeData.Substring(0, barcodeData.IndexOf(char.MinValue)),
                    typeName.Substring(0, typeName.IndexOf(char.MinValue)));
        }

        public void AddItem(string data, string type)
        {
            ListViewItem item = new ListViewItem(new string[] { data, type });
            lvResult.Items.Add(item);
            lvResult.EnsureVisible(lvResult.Items.Count - 1);
            Utils.PlaySound(@"\Windows\Success.wav");
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            BARCODE_RESULT result = BARCODE_RESULT.BARCODE_RESULT_FAILURE;

            if (barcode == null)
                return;

            // Look for and report all barcodes in captured frame.
            // You can set the scanning time in millisecond units.
            result = barcode.StartScanMultiple(1000 * 5);
            if (result != BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
                MessageBox.Show(result.ToString());
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            BARCODE_RESULT result = BARCODE_RESULT.BARCODE_RESULT_FAILURE;

            if (barcode == null)
                return;

            result = barcode.StopScanMultiple();
            if (result != BARCODE_RESULT.BARCODE_RESULT_SUCCESS)
                MessageBox.Show(result.ToString());
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            lvResult.Items.Clear();
        }
    }
}