﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NCamApi;
using NSysSvcApi;
using _Native;

namespace Camera
{
    public partial class Form1 : Form
    {
        public SysSvcApi sysSvc = null;
        public CamApi cam = null;
        public const string title = "Camera Example";
        public bool flashEnabled = false;
        public SENSOR_SETUP_INFO sensor;
        public SENSOR_CAPA sensorCapabilities;
        public bool doPreview = false;
        
        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
            }
            else
            {
                MessageBox.Show("in Windows Mobile, you can not use this sample. Please refer to the Windows Mobile 6 SDK");
                return;
            }

            // create system service object
            sysSvc = new SysSvcApi();
            HW_OPTIONS hwOptions = new HW_OPTIONS();
            // Check what type of camera.
            sysSvc.GetHardwareOptions(ref hwOptions);
            if (hwOptions.Camera == HW_TYPE_CAM.NONE)
            {
                sysSvc.Dispose();
                sysSvc = null;
                MessageBox.Show("Camera is not found!");
                return;
            }

            // 2D Scanner and Camera shall use same physical lines(CIF, I2C).
            // Therefore, 2D scanner and camera can not run at the same time.
            if (hwOptions.Barcode == HW_TYPE_BARCODE._2D_SWD_H || hwOptions.Barcode == HW_TYPE_BARCODE._2D_SWD_M)
            {
                // If the barcode application is running, you must shut down.
                IntPtr hBarcodeEmulator = _Native.Utils.FindWindow(BarcodeEmulatorInfo.WindowClass, BarcodeEmulatorInfo.WindowTitle);
                if (hBarcodeEmulator != IntPtr.Zero)
                    Utils.SendMessage(hBarcodeEmulator, Utils.WM_COMMAND, BarcodeEmulatorInfo.CloseID, 0);
            }

            // Create camera object
            cam = new CamApi(pbxPreview.Handle);
            // When capturing is successful, register a callback function to be executed.
            cam.SetCallBack(new CamApi.CAMERACALLBACK(camCallbackProc));
            CAM_RESULT result = cam.Open();
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                sysSvc.Dispose();
                sysSvc = null;
                MessageBox.Show("Open : " + result.ToString());
                return;
            }

            sensor = new SENSOR_SETUP_INFO();
            cam.GetInfo(ref sensor);
            cam.GetSensorCapa(ref sensorCapabilities);

            cbxWhiteBalance.Items.Add("Auto");
            cbxWhiteBalance.Items.Add("Cloudy");
            cbxWhiteBalance.Items.Add("Fluorescent");
            cbxWhiteBalance.Items.Add("Incandescent");
            cbxWhiteBalance.Items.Add("Sunny");
            cbxWhiteBalance.SelectedIndex = 0;

            cbxFlipMirror.Items.Add("No Effect");
            cbxFlipMirror.Items.Add("Flip");
            cbxFlipMirror.Items.Add("Mirror");
            cbxFlipMirror.Items.Add("Flip & Mirror");
            cbxFlipMirror.SelectedIndex = 0;

            cbxDigitalEffect.Items.Add("No Effect");
            cbxDigitalEffect.Items.Add("Blue");
            cbxDigitalEffect.Items.Add("Green");
            cbxDigitalEffect.Items.Add("Mono");
            cbxDigitalEffect.Items.Add("Negative");
            cbxDigitalEffect.Items.Add("Red");
            cbxDigitalEffect.Items.Add("Sepia");
            cbxDigitalEffect.Items.Add("Violet");
            cbxDigitalEffect.SelectedIndex = 0;

            cbxFlashLight.Items.Add("Off");
            cbxFlashLight.Items.Add("On");
            cbxFlashLight.SelectedIndex = 0;


            cbxResolution.Items.Add("QQVGA");   // 160 x 120
            cbxResolution.Items.Add("QVGA");    // 320 x 240
            cbxResolution.Items.Add("CIF");     // 352 x 288
            cbxResolution.Items.Add("VGA");     // 640 x 480
            cbxResolution.Items.Add("SVGA");    // 800 x 600
            cbxResolution.Items.Add("XGA");     // 1024 x 768
            cbxResolution.Items.Add("SXGA");    // 1280 x 960
            cbxResolution.Items.Add("UXGA");    // 1600 x 1200
            cbxResolution.Items.Add("QXGA");    // 2048 x 1536
            cbxResolution.SelectedIndex = (int)sensor.Resol;

            nudContrast.Minimum = sensorCapabilities.Contrast.ContrastMin;
            nudContrast.Maximum = sensorCapabilities.Contrast.ContrastMax;
            nudContrast.Value = sensorCapabilities.Contrast.ContrastDefault;

            nudSaturation.Minimum = sensorCapabilities.Saturation.SaturationMin;
            nudSaturation.Maximum = sensorCapabilities.Saturation.SaturationMax;
            nudSaturation.Value = sensorCapabilities.Saturation.SaturationDefault;

            nudBrightness.Minimum = sensorCapabilities.Brightness.BrightMin;
            nudBrightness.Maximum = sensorCapabilities.Brightness.BrightMax;
            nudBrightness.Value = sensorCapabilities.Brightness.BrightDefault;

            this.cbxWhiteBalance.SelectedIndexChanged += new System.EventHandler(this.cbxWhiteBalance_SelectedIndexChanged);
            this.cbxFlipMirror.SelectedIndexChanged += new System.EventHandler(this.cbxFlipMirror_SelectedIndexChanged);
            this.cbxDigitalEffect.SelectedIndexChanged += new System.EventHandler(this.cbxDigitalEffect_SelectedIndexChanged);
            this.cbxFlashLight.SelectedIndexChanged += new System.EventHandler(this.cbxFlashLight_SelectedIndexChanged);
            this.nudContrast.ValueChanged += new System.EventHandler(this.nudContrast_ValueChanged);
            this.nudSaturation.ValueChanged += new System.EventHandler(this.nudSaturation_ValueChanged);
            this.nudBrightness.ValueChanged += new System.EventHandler(this.nudBrightness_ValueChanged);
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the camera object
            if (cam != null)
            {
                if (cam.IsOpen())
                {
                    if (flashEnabled)
                        cam.EnableFlash(false);

                    cam.StopPreview();
                    cam.Close();
                }

                cam = null;
            }

            // destruction of the system service object
            if (sysSvc != null)
            {
                sysSvc.Dispose();
                sysSvc = null;
            }
        }

        /// <summary>
        /// If image capturing is successful, this function is executed.
        /// </summary>
        /// <param name="camMsg"></param>
        public void camCallbackProc(CamApi.CAMERAMSG camMsg)
        {
            switch (camMsg)
            {
                case CamApi.CAMERAMSG.CAPTURE:
                    break;
                    // 
                case CamApi.CAMERAMSG.COMPLETE:
                    Utils.PlaySound(@"\Windows\Success.wav");
                    Cursor.Current = Cursors.Default;

                    if(doPreview)
                        cam.StartPreview();

                    break;
                default:
                    break;
            }
        }

        private void btnPreview_Click(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            CAM_RESULT result = cam.StartPreview();
            if (result == CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                doPreview = true;
            }
            else
            {
                doPreview = false;
                MessageBox.Show("StartPreview : " + result.ToString());
            }
                
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            CAM_RESULT result = cam.StopPreview();
            if (result == CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                doPreview = false;
                pbxPreview.Refresh();
            }
            else
            {
                MessageBox.Show("StopPreview : " + result.ToString());
            }
        }

        private void btnCapture_Click(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            CAPTURE_SETUP_INFO captureInfo = new CAPTURE_SETUP_INFO();

            captureInfo.SensorSetup = sensor;
            captureInfo.SensorSetup.ColorSpace = CAM_COL_SP.YCbCr;
            captureInfo.SensorSetup.bpp = CAM_BPP.YCbCr422;
            captureInfo.SensorSetup.Resol = (CAM_RESOL)cbxResolution.SelectedIndex;
            captureInfo.imgFormat = IMG_ENCODE_FORMAT.IMG_ENCODE_JPG;
            captureInfo.nJpegQuality = 100;
            captureInfo.sSavePath = "\\Flash Disk\\" + DateTime.Now.Year.ToString() +
                                                    DateTime.Now.Month.ToString() +
                                                    DateTime.Now.Day.ToString() +
                                                    DateTime.Now.Hour.ToString() +
                                                    DateTime.Now.Minute.ToString() +
                                                    DateTime.Now.Second.ToString() + ".jpg";

            Cursor.Current = Cursors.WaitCursor;
            CAM_RESULT result = cam.Capture(ref captureInfo);
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                Cursor.Current = Cursors.Default;
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("Capture : " + result.ToString());
            }
        }

        private void cbxWhiteBalance_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            switch (cbxWhiteBalance.SelectedIndex)
            {
                case 0:
                    sensor.WhiteBalance = WHITE_BAL.WB_AUTO;
                    break;
                case 1:
                    sensor.WhiteBalance = WHITE_BAL.WB_CLOUDY;
                    break;
                case 2:
                    sensor.WhiteBalance = WHITE_BAL.WB_FLUORESCENT;
                    break;
                case 3:
                    sensor.WhiteBalance = WHITE_BAL.WB_INCANDESCENT;
                    break;
                case 4:
                    sensor.WhiteBalance = WHITE_BAL.WB_SUNNY;
                    break;
            }

            CAM_RESULT result = cam.SetWhiteBalance(ref sensor);
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetWhiteBalance : " + result.ToString());
            }
        }

        private void cbxFlipMirror_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            System.Diagnostics.Debug.WriteLine("cbxFlipMirror : " + cbxFlipMirror.SelectedIndex.ToString());
            switch (cbxFlipMirror.SelectedIndex)
            {
                case 0:
                    sensor.FlipMirror = FLIP_MIRROR.NO_MIRROR_FLIP;
                    break;
                case 1:
                    sensor.FlipMirror = FLIP_MIRROR.FLIP;
                    break;
                case 2:
                    sensor.FlipMirror = FLIP_MIRROR.MIRR;
                    break;
                case 3:
                    sensor.FlipMirror = FLIP_MIRROR.FLIP_AND_MIRROR;
                    break;
            }

            CAM_RESULT result = cam.SetFlipMirror(ref sensor);
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetFlipMirror : " + result.ToString());
            }
        }

        private void cbxDigitalEffect_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            switch (cbxDigitalEffect.SelectedIndex)
            {
                case 0:
                    sensor.Effect = EFFECT.NO_EFFECT;
                    break;
                case 1:
                    sensor.Effect = EFFECT.EF_BLUE;
                    break;
                case 2:
                    sensor.Effect = EFFECT.EF_GREEN;
                    break;
                case 3:
                    sensor.Effect = EFFECT.EF_MONO;
                    break;
                case 4:
                    sensor.Effect = EFFECT.EF_NEGATIVE;
                    break;
                case 5:
                    sensor.Effect = EFFECT.EF_RED;
                    break;
                case 6:
                    sensor.Effect = EFFECT.EF_SEPIA;
                    break;
                case 7:
                    sensor.Effect = EFFECT.EF_VIOLET;
                    break;
            }

            CAM_RESULT result = cam.SetEffect(ref sensor);
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetEffect : " + result.ToString());
            }
        }

        private void nudContrast_ValueChanged(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            sensor.Contrast = (CONT_OPTIONS)nudContrast.Value;
            System.Diagnostics.Debug.WriteLine("sensor.Contrast: " + sensor.Contrast.ToString());
            CAM_RESULT result = cam.SetContrast(ref sensor);
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetContrast : " + result.ToString());
            }
        }

        private void nudSaturation_ValueChanged(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            sensor.Saturation = (SAT_OPTIONS)nudSaturation.Value;
            System.Diagnostics.Debug.WriteLine("sensor.Saturation: " + sensor.Saturation.ToString());
            CAM_RESULT result = cam.SetSaturation(ref sensor);
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetSaturation : " + result.ToString());
            }
        }

        private void nudBrightness_ValueChanged(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            sensor.Brightness = (int)nudBrightness.Value;
            System.Diagnostics.Debug.WriteLine("sensor.Brightness: " + sensor.Brightness.ToString());
            CAM_RESULT result = cam.SetBrightness(ref sensor);
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetBrightness : " + result.ToString());
            }
        }

        private void cbxFlashLight_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            CAM_RESULT result = cam.EnableFlash(cbxFlashLight.SelectedIndex == 0 ? false : true);
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("EnableFlash : " + result.ToString());
            }
            else
            {
                flashEnabled = cbxFlashLight.SelectedIndex == 0 ? false : true;
            }
        }

        private void btnAutoFocus_Click(object sender, EventArgs e)
        {
            if (cam == null)
                return;

            sensor.bAutoFocus = true;
            CAM_RESULT result = cam.SetAutoFocus(ref sensor);
            if (result != CAM_RESULT.CAM_RESULT_SUCCESS)
            {
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetAutoFocus : " + result.ToString());
            }
        }
    }
}