﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NFingerPrintApi;
using NSysSvcApi;
using _Native;

namespace FingerPrint
{
    public partial class Form1 : Form
    {
        public SysSvcApi sysService = null;
        public FingerPrintApi fingerPrint = null;
        FPR_RESULT result = FPR_RESULT.FPR_RESULT_FAILURE;
        public const string title = "FingerPrint Example";

        Timer timer_Registrion = null;
        Timer timer_Authentication = null;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            comboBoxID.Items.Add("0");
            comboBoxID.Items.Add("1");
            comboBoxID.Items.Add("2");
            comboBoxID.Items.Add("3");
            comboBoxID.Items.Add("4");
            comboBoxID.Items.Add("5");
            comboBoxID.Items.Add("6");
            comboBoxID.Items.Add("7");
            comboBoxID.Items.Add("8");
            comboBoxID.Items.Add("9");

            comboBoxID.SelectedIndex = 0;

            byte Data = 0;

            // create system service object
            sysService = new SysSvcApi();
            HW_OPTIONS hwOptions = new HW_OPTIONS();
            // Check what type of fingerPrint.
            sysService.GetHardwareOptions(ref hwOptions);
            if (hwOptions.FingerPrint == HW_TYPE_FINGERPRINT.NONE)
            {
                sysService.Dispose();
                sysService = null;
                MessageBox.Show("FingerPrint is not found!");
                return;
            }

            // create finger print object
            fingerPrint = new FingerPrintApi();
            result = fingerPrint.Open();
            if (result != FPR_RESULT.FPR_RESULT_SUCCESS)
            {
                MessageBox.Show("Open : " + result.ToString());
                return;
            }

            result = fingerPrint.GetFirmwareVersion(ref Data);
            if (result != FPR_RESULT.FPR_RESULT_SUCCESS)
            {
                fingerPrint.Close();
                MessageBox.Show("GetFirmwareVersion : " + result.ToString());
                return;
            }
            textBoxVersion.Text = String.Format("{0:X}", Data);

            // get the count of fingerprints that are registered in the module.
            result = fingerPrint.GetIdCount(ref Data);
            if (result != FPR_RESULT.FPR_RESULT_SUCCESS)
            {
                fingerPrint.Close();
                MessageBox.Show("GetIdCount : " + result.ToString());
                return;
            }
            textBoxIdCount.Text = String.Format("{0:D}", Data);

            timer_Authentication = new Timer();
            timer_Registrion = new Timer();
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            timer_Authentication.Enabled = false;
            timer_Registrion.Enabled = false;

            // destruction of the finger print object
            if (fingerPrint != null)
            {
                fingerPrint.Cancel();
                fingerPrint.Close();
                fingerPrint = null;
            }

            // destruction of the system service object
            if (sysService != null)
            {
                sysService.Dispose();
                sysService = null;
            }
        }

        public void SetStatusText(String Msg)
        {
            labelStatus.Text = Msg;
        }

        public void RefreshIDCount()
        {
            if (fingerPrint == null)
                return;

            byte Data = 0;

            fingerPrint.GetIdCount(ref Data);
            textBoxIdCount.Text = String.Format("{0:D}", Data);
        }

        void timer_Registrion_Tick(object sender, EventArgs e)
        {
            if (fingerPrint == null)
                return;

            string strtemp = "";
            byte Data = 0;

            result = fingerPrint.ReadResult(Convert.ToByte(FPR_CMD.FPR_CMD_REGISTRATION), ref Data);

            if (result != FPR_RESULT.FPR_RESULT_SUCCESS)
                return;

            timer_Registrion.Tick -= new EventHandler(timer_Registrion_Tick);
            timer_Registrion.Enabled = false;

            if (Data >= 0x00 && Data <= 0x09)
            {
                strtemp = string.Format("Registration Success ID = {0:D}", Data);
                SetStatusText(strtemp);
                Utils.PlaySound(@"\Windows\Success.wav");
                RefreshIDCount();
            }
            else
            {
                SetStatusText("Registration Failed");
                Utils.PlaySound(@"\Windows\Fail.wav");
            }

        }

        private void buttonRegist_Click(object sender, EventArgs e)
        {
            if (fingerPrint == null)
                return;

            byte ID = Convert.ToByte(comboBoxID.SelectedIndex);

            // You can register up to 10 fingerprints.(0 ~ 9)
            // Can be registered using the vacant ID.(0 ~ 9)
            result = fingerPrint.StartRegistration(ID);
            if (result != FPR_RESULT.FPR_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                SetStatusText("StartRegistration failed");
                return;
            }

            SetStatusText("Wait...");

            timer_Registrion.Interval = 250;
            timer_Registrion.Tick += new EventHandler(timer_Registrion_Tick);
            timer_Registrion.Enabled = true;
        }

        private void buttonErase_Click(object sender, EventArgs e)
        {
            if (fingerPrint == null)
                return;

            String strTemp = "";
            byte ID = Convert.ToByte(comboBoxID.SelectedIndex);

            // Delete one of the ID registered in the module.(0 ~ 9)
            result = fingerPrint.DeRegister(ID);
            if (result == FPR_RESULT.FPR_RESULT_SUCCESS)
            {
                strTemp = string.Format("Erase ID = {0:D} Success", ID);
                SetStatusText(strTemp);
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                strTemp = string.Format("Erase ID = {0:D} Failed", ID);
                SetStatusText(strTemp);
                Utils.PlaySound(@"\Windows\Fail.wav");
            }

            RefreshIDCount();
        }

        private void buttonEraseAll_Click(object sender, EventArgs e)
        {
            if (fingerPrint == null)
                return;

            // Deletes all the registered ID 
            result = fingerPrint.DeRegister(0x3a);
            if (result == FPR_RESULT.FPR_RESULT_SUCCESS)
            {
                SetStatusText("Erase All ID Success");
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                SetStatusText("Erase All ID Fail");
                Utils.PlaySound(@"\Windows\Fail.wav");
            }

            RefreshIDCount();
        }

        void timer_Authentication_Tick(object sender, EventArgs e)
        {
            if (fingerPrint == null)
                return;

            string strtemp = "";
            byte Data = 0;

            result = fingerPrint.ReadResult(Convert.ToByte(FPR_CMD.FPR_CMD_AUTHENTICATION), ref Data);
            if (result != FPR_RESULT.FPR_RESULT_SUCCESS)
                return;

            timer_Authentication.Tick -= new EventHandler(timer_Authentication_Tick);
            timer_Authentication.Enabled = false;

            if (Data >= 0x00 && Data <= 0x09)
            {
                strtemp = string.Format("Authentication Success ID = {0:D}", Data);
                SetStatusText(strtemp);
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                SetStatusText("Authentication Failed");
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
        }

        private void buttonAuth_Click(object sender, EventArgs e)
        {
            if (fingerPrint == null)
                return;

            // By scanning a fingerprint from the user, and attempts to authenticate.
            result = fingerPrint.StartAuthentication();
            if (result != FPR_RESULT.FPR_RESULT_SUCCESS)
            {
                SetStatusText("Authentication Failed");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            SetStatusText("Wait...");

            timer_Authentication.Interval = 250;
            timer_Authentication.Tick += new EventHandler(timer_Authentication_Tick);
            timer_Authentication.Enabled = true;  
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            if (fingerPrint == null)
                return;

            timer_Authentication.Enabled = false;
            timer_Registrion.Enabled = false;

            // Stops the current task being attempted.
            result = fingerPrint.Cancel();
            SetStatusText("");
        }
    }
}