﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Globalization;
using System.Collections;
using Microsoft.Win32;
using NGpsApi;
using NSysSvcApi;
using _Native;

namespace GPS
{
    public partial class Form1 : Form
    {
        public const string title = "GPS Example";
        public GpsApi gps = null;
        public SysSvcApi sysSvc = null;
        GPS_RESULT result = GPS_RESULT.GPS_RESULT_FAILURE;
        HW_TYPE_GPS gpsType = HW_TYPE_GPS.NONE;
        public Queue qRecvBuff = null;
        public Thread threadDequeue = null;
        public static bool exitThread = false;
        public NmeaParser parser = null;
        //public static AutoResetEvent arEventReceived = null;

        public delegate void DisplayPlatformModelEventHandler(NavigationEngineSettings.PlatformModel platformModel);

        public enum DEVICENAME : int
        {
            UNKNOWN = 0,
            AT880,
            AT980,
            AT280,
            AT380,
            AB700
        }

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            cbxPlatformModel.Items.Add("Portable");
            cbxPlatformModel.Items.Add("Stationary");
            cbxPlatformModel.Items.Add("Pedestrian");
            cbxPlatformModel.Items.Add("Automotive");
            cbxPlatformModel.Items.Add("Sea");
            cbxPlatformModel.Items.Add("Airborne <1g");
            cbxPlatformModel.Items.Add("Airborne <2g");
            cbxPlatformModel.Items.Add("Airborne <4g");

            // create the Queue to store the NMEA data
            qRecvBuff = new Queue();
            threadDequeue = new Thread(new ThreadStart(threadDequeueProc));
            threadDequeue.IsBackground = true;
            threadDequeue.Start();

            // create the system service object
            sysSvc = new SysSvcApi();

            // Reads the hardware option information.
            HW_OPTIONS hwoptions = new HW_OPTIONS();
            // Check what type of GPS.
            sysSvc.GetHardwareOptions(ref hwoptions);
            if (hwoptions.GPS == HW_TYPE_GPS.NONE)
            {
                exitThread = true;
                threadDequeue.Join();
                threadDequeue = null;

                MessageBox.Show("GPS is not found!");
                return;
            }
            else
            {
                gpsType = hwoptions.GPS;
            }

            // create the nmea parser object
            parser = new NmeaParser(this);

            // create the gps object
            gps = new GpsApi();

            // Register a callback for receiving a byte array.
            gps.SetRawDataCallback(new GpsApi.GPSRAWDATACALLBACK(gpsCallbackProc));
            // If you wish to receive a string form, you can proceed as follows.
            // gps.SetCallBack(new GpsApi.GPSCALLBACK(gpsCallbackProc));

            // The default port name is COM6:
            PORT port = PORT.COM6;
            // The defualt baudrate is 9600
            BAUDRATE baudrate = BAUDRATE.BR_9600;

            // If the GPS module is IT503M, you must set the baudrate to 115200.
            if (gpsType == HW_TYPE_GPS.IT530M)
                baudrate = BAUDRATE.BR_115200;

            // If you are using a AT280, you must control the GPS power and use port COM9:
            if (GetDeviceName() == DEVICENAME.AT280)
            {
                // Turn on the GPS power
                Utils.setGpsPower(true);
                // use port COM9:
                port = PORT.COM9;
            }

            result = gps.Open(port, baudrate);
            if (result != GPS_RESULT.GPS_RESULT_SUCCESS)
            {
                sysSvc.Dispose();
                sysSvc = null;
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("Open : " + result.ToString());
                return;
            }

            // If the GPS module is UBlox series, you can use a pedestrian mode.
            if (gpsType == HW_TYPE_GPS.UBLUX || gpsType == HW_TYPE_GPS.UBLOX_MAX7)
            {
                // Reads the current platform mode.
                byte[] buff = new byte[128];
                int len = 0;
                NavigationEngineSettings.PollConfiguration(ref buff, ref len);
                gps.TxData(buff, len);
            }
            else
            {
                // If the GPS module is not UBlox series, hides the Controls associated with platform mode settings.
                label10.Visible = false;
                cbxPlatformModel.Visible = false;
                btnApply.Visible = false;
                lblDescription.Visible = false;
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the gps object
            if (gps != null)
            {
                if (gps.IsOpen())
                    gps.Close();

                gps = null;
            }

            // destruction of the system serivce object
            if (sysSvc != null)
            {
                sysSvc.Dispose();
                sysSvc = null;
            }

            // destruction of the dequeue
            if (threadDequeue != null)
            {
                exitThread = true;
                threadDequeue.Join();
                threadDequeue = null;
            }

            // If you are using a AT280, you must control the GPS power
            if (GetDeviceName() == DEVICENAME.AT280)
                Utils.setGpsPower(false); // Turn off the GPS power
        }

        public DEVICENAME GetDeviceName()
        {
            DEVICENAME devName = DEVICENAME.UNKNOWN;

            try
            {
                if (!Utils.IsWinCE) // WM
                {
                    string regValue = (string)Registry.GetValue(@"HKEY_LOCAL_MACHINE\Ident", "Name", string.Empty);
                    if (string.IsNullOrEmpty(regValue))
                        return devName;

                    regValue = regValue.Substring(0, 5);
                    switch (regValue)
                    {
                        case "SI203":
                        case "AT880":
                            devName = DEVICENAME.AT880;
                            break;
                        case "SI303":
                        case "AT980":
                            devName = DEVICENAME.AT980;
                            break;
                        case "SI890":
                        case "AB700":
                            devName = DEVICENAME.AB700;
                            break;
                        case "AT380":
                            devName = DEVICENAME.AT380;
                            break;
                        case "AT280":
                            devName = DEVICENAME.AT280;
                            break;
                    }
                }
                else // CE
                {
                    int regValue = (int)Registry.GetValue(@"HKEY_CURRENT_USER\ControlPanel\KeyTrans", "ModelNumber", 0);
                    devName = (DEVICENAME)regValue;
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }

            return devName;
        }

        public void SetTextColor(Color color)
        {
            lblLatitude.ForeColor = color;
            lblLongitude.ForeColor = color;
            lblSpeed.ForeColor = color;
            lblSatellitesInUse.ForeColor = color;
            label7.ForeColor = color;
            lblSatellitesInView.ForeColor = color;
            lblDateTime.ForeColor = color;
        }

        public void StatusReceived(bool FixedObtained)
        {
            if (this.InvokeRequired)
            {
                NmeaParser.StautsReceivedEventHandler d = new NmeaParser.StautsReceivedEventHandler(StatusReceived);
                this.BeginInvoke(d, new object[] { FixedObtained });
            }
            else
            {
                if (FixedObtained)
                {
                    lblStatus.ForeColor = Color.Blue;
                    SetTextColor(Color.Black);
                    lblStatus.Text = "Active";
                }
                else
                {
                    lblStatus.ForeColor = Color.Red;
                    SetTextColor(Color.DarkGray);
                    lblStatus.Text = "inValid";
                }
            }
        }

        public void DateTimeChanged(DateTime dt)
        {
            if (this.InvokeRequired)
            {
                NmeaParser.DateTimeChangedEventHandler d = new NmeaParser.DateTimeChangedEventHandler(DateTimeChanged);
                this.BeginInvoke(d, new object[] { dt });
            }
            else
            {
                lblDateTime.Text = string.Format("{0:d4}-{1:d2}-{2:d2} {3:d2}:{4:d2}:{5:d2}", dt.Year, dt.Month, dt.Day, dt.Hour, dt.Minute, dt.Second);
            }
        }

        public void SpeedReceived(double speed)
        {
            if (this.InvokeRequired)
            {
                NmeaParser.SpeedReceivedEventHandler d = new NmeaParser.SpeedReceivedEventHandler(SpeedReceived);
                this.BeginInvoke(d, new object[] { speed });
            }
            else
            {
                lblSpeed.Text = speed.ToString();
            }
        }

        public void SatellitesInViewReceived(int value)
        {
            if (this.InvokeRequired)
            {
                NmeaParser.SatellitesInViewReceivedEventHandler d = new NmeaParser.SatellitesInViewReceivedEventHandler(SatellitesInViewReceived);
                this.BeginInvoke(d, new object[] { value });
            }
            else
            {
                lblSatellitesInView.Text = value.ToString();
            }
        }

        public void SatellitesInUseReceived(int value)
        {
            if (this.InvokeRequired)
            {
                NmeaParser.SatellitesInUseReceivedEventHandler d = new NmeaParser.SatellitesInUseReceivedEventHandler(SatellitesInUseReceived);
                this.BeginInvoke(d, new object[] { value });
            }
            else
            {
                lblSatellitesInUse.Text = value.ToString();
            }
        }

        public void PositionReceived(string latitude, string longitude)
        {
            if (this.InvokeRequired)
            {
                NmeaParser.PositionReceivedEventHandler d = new NmeaParser.PositionReceivedEventHandler(PositionReceived);
                this.BeginInvoke(d, new object[] { latitude, longitude });
            }
            else
            {
                lblLatitude.Text = latitude;
                lblLongitude.Text = longitude;
            }
        }

        public void DisplayPlatformModel(NavigationEngineSettings.PlatformModel platformModel)
        {
            if (this.InvokeRequired)
            {
                DisplayPlatformModelEventHandler d = new DisplayPlatformModelEventHandler(DisplayPlatformModel);
                this.BeginInvoke(d, new object[] { platformModel });
            }
            else
            {
                switch (platformModel)
                {
                    case NavigationEngineSettings.PlatformModel.Portable:
                        cbxPlatformModel.SelectedIndex = 0;
                        break;
                    case NavigationEngineSettings.PlatformModel.Stationary:
                        cbxPlatformModel.SelectedIndex = 1;
                        break;
                    case NavigationEngineSettings.PlatformModel.Pedestrian:
                        cbxPlatformModel.SelectedIndex = 2;
                        break;
                    case NavigationEngineSettings.PlatformModel.Automotive:
                        cbxPlatformModel.SelectedIndex = 3;
                        break;
                    case NavigationEngineSettings.PlatformModel.Sea:
                        cbxPlatformModel.SelectedIndex = 4;
                        break;
                    case NavigationEngineSettings.PlatformModel.Airbone_1g:
                        cbxPlatformModel.SelectedIndex = 5;
                        break;
                    case NavigationEngineSettings.PlatformModel.Airbone_2g:
                        cbxPlatformModel.SelectedIndex = 6;
                        break;
                    case NavigationEngineSettings.PlatformModel.Airbone_4g:
                        cbxPlatformModel.SelectedIndex = 7;
                        break;
                }
            }
        }

        // Callback function for receiving a string
        //public void gpsCallbackProc(string NMEASentence)
        //{
        //    qRecvBuff.Enqueue(NMEASentence);
        //}

        // Callback function for receiving a byte array
        public void gpsCallbackProc(byte[] pBuff, int len)
        {
            byte[] recv = new byte[len];
            Array.Copy(pBuff, recv, len);
            qRecvBuff.Enqueue(recv);
        }

        public void threadDequeueProc()
        {
            byte[] recv = null;

            while (!exitThread)
            {
                if (qRecvBuff.Count > 0)
                {
                    recv = (byte[])qRecvBuff.Dequeue();
                    if (recv != null)
                    {
                        // If the received data is UBLOX packet, you don't need to parse.
                        // You need to parse NMEA data.
                        if (recv[0] != 0xB5 && recv[1] != 0x62)
                        {
                            // Since you've received a byte array, you need a process of converting to a string.
                            string str = Encoding.Default.GetString(recv, 0, recv.Length);
                            parser.Parse(str);
                            //System.Diagnostics.Debug.Write(str);
                        }

                        if (gpsType == HW_TYPE_GPS.UBLUX || gpsType == HW_TYPE_GPS.UBLOX_MAX7)
                        {
                            // If the UBLOX packet is received 
                            if (recv[0] == 0xB5 && recv[1] == 0x62)
                            {
                                //foreach (byte b in recv)
                                //    System.Diagnostics.Debug.Write(string.Format("{0:X2} ", b));
                                //System.Diagnostics.Debug.WriteLine("");

                                if (recv[2] == 0x06 && recv[3] == 0x24)
                                {
                                    // Now that you have received information about the platform mode, 
                                    // displays this information on the screen.
                                    NavigationEngineSettings.Parse(recv, recv.Length);
                                    DisplayPlatformModel(NavigationEngineSettings.getPlatformModel());
                                }

                                if (recv[2] == 0x05)
                                {
                                    if (recv[3] == 0x01) // ACK
                                    {
                                        Utils.PlaySound(@"\Windows\Success.wav");
                                        System.Diagnostics.Debug.WriteLine("==>> received ACK");
                                    }
                                    else     // 0x00 NAK
                                    {
                                        Utils.PlaySound(@"\Windows\Fail.wav");
                                        System.Diagnostics.Debug.WriteLine("==>> received NAK");
                                    }
                                }
                            }
                        }
                    }
                }
                else
                {
                    Thread.Sleep(10);
                }
            }

            System.Diagnostics.Debug.WriteLine("exit threadDequeueProc...");
        }

        private void btnColdStart_Click(object sender, EventArgs e)
        {
            // In some cases, you may need to erase the memory of the GPS module.
            // Depending on the type of GPS module, you can send the following commands.

            byte[] coldStart = new byte[128];
            int count = 0;

            if (gps == null)
                return;

            if (!gps.IsOpen())
                return;

            switch (gpsType)
            {
                case HW_TYPE_GPS.SIRF3:
                case HW_TYPE_GPS.SIRF4:
                    coldStart[count++] = Convert.ToByte('$');
                    coldStart[count++] = Convert.ToByte('P');
                    coldStart[count++] = Convert.ToByte('S');
                    coldStart[count++] = Convert.ToByte('R');
                    coldStart[count++] = Convert.ToByte('F');
                    coldStart[count++] = Convert.ToByte('1');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = Convert.ToByte('1');
                    coldStart[count++] = Convert.ToByte(',');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = Convert.ToByte(',');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = Convert.ToByte(',');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = Convert.ToByte(',');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = Convert.ToByte(',');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = Convert.ToByte(',');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = Convert.ToByte(',');
                    coldStart[count++] = Convert.ToByte('1');
                    coldStart[count++] = Convert.ToByte('2');
                    coldStart[count++] = Convert.ToByte(',');
                    coldStart[count++] = Convert.ToByte('4');
                    coldStart[count++] = Convert.ToByte('*');
                    coldStart[count++] = Convert.ToByte('1');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = 0x0D;
                    coldStart[count++] = 0x0A;
                    break;
                case HW_TYPE_GPS.UBLUX:
                case HW_TYPE_GPS.UBLOX_MAX7:
                    coldStart[count++] = 0xB5;
                    coldStart[count++] = 0x62;
                    coldStart[count++] = 0x06;
                    coldStart[count++] = 0x04;
                    coldStart[count++] = 0x04;
                    coldStart[count++] = 0x00;
                    coldStart[count++] = 0xFF;
                    coldStart[count++] = 0xFF;
                    coldStart[count++] = 0x00;
                    coldStart[count++] = 0x00;
                    coldStart[count++] = 0x0C;
                    coldStart[count++] = 0x5D;
                    break;
                case HW_TYPE_GPS.IT530M:
                    coldStart[count++] = Convert.ToByte('$');
                    coldStart[count++] = Convert.ToByte('P');
                    coldStart[count++] = Convert.ToByte('M');
                    coldStart[count++] = Convert.ToByte('T');
                    coldStart[count++] = Convert.ToByte('K');
                    coldStart[count++] = Convert.ToByte('1');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = Convert.ToByte('3');
                    coldStart[count++] = Convert.ToByte('*');
                    coldStart[count++] = Convert.ToByte('3');
                    coldStart[count++] = Convert.ToByte('0');
                    coldStart[count++] = 0x0D;
                    coldStart[count++] = 0x0A;
                    break;
                default:
                    MessageBox.Show("not supported");
                    return;
            }

            GPS_RESULT result = gps.TxData(coldStart, count);
            if (result != GPS_RESULT.GPS_RESULT_SUCCESS)
            {
                _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("TxData : " + result.ToString());
            }

            // If the GPS module is UBlox series, read the platform mode again.
            if (gpsType == HW_TYPE_GPS.UBLUX || gpsType == HW_TYPE_GPS.UBLOX_MAX7)
            {
                Thread.Sleep(500);
                byte[] buff = new byte[128];
                int len = 0;
                NavigationEngineSettings.PollConfiguration(ref buff, ref len);
                gps.TxData(buff, len);
            }
        }

        private void cbxPlatformModel_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cbxPlatformModel.SelectedIndex)
            {
                case 0: // Portable
                    lblDescription.Text = "Applications with low acceleration, e.g. portable devices. Suitable for most situations.";
                    break;
                case 1: // Stationary
                    lblDescription.Text = "Used in timing applications (antenna must be stationary) or other stationary applications. Velocity restricted to 0 m/s. Zero dynamics assumed.";
                    break;
                case 2: // Pedestrian
                    lblDescription.Text = "Applications with low acceleration and speed, e.g. how a pedestrian would move. Low acceleration assumed.";
                    break;
                case 3: // Automotive
                    lblDescription.Text = "Used for applications with equivalent dynamics to those of a passenger car. Low vertical acceleration assumed.";
                    break;
                case 4: // At sea
                    lblDescription.Text = "Recommended for applications at sea, with zero vertical velocity. Zero vertical velocity assumed. Sea level assumed.";
                    break;
                case 5: // Airborne <1g
                    lblDescription.Text = "Used for applications with a higher dynamic range and vertical acceleration than a passenger car. No 2D position fixes supported.";
                    break;
                case 6: // Airborne <2g
                    lblDescription.Text = "Recommended for typical airborne environment. No 2D position fixes supported.";
                    break;
                case 7: // Airborne <4g
                    lblDescription.Text = "Only recommended for extremely dynamic environments. No 2D position fixes supported.";
                    break;
            }
            btnApply.Focus();
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            if (gps == null)
                return;

            if (gpsType != HW_TYPE_GPS.UBLUX && gpsType != HW_TYPE_GPS.UBLOX_MAX7)
            {
                MessageBox.Show("Not supported");
                return;
            }

            // You can change the platform mode.

            byte[] buff = new byte[128];
            int len = 0;
            NavigationEngineSettings.PlatformModel platformModel = NavigationEngineSettings.PlatformModel.Portable;
            switch (cbxPlatformModel.SelectedIndex)
            {
                case 0:
                    platformModel = NavigationEngineSettings.PlatformModel.Portable;
                    break;
                case 1:
                    platformModel = NavigationEngineSettings.PlatformModel.Stationary;
                    break;
                case 2:
                    platformModel = NavigationEngineSettings.PlatformModel.Pedestrian;
                    break;
                case 3:
                    platformModel = NavigationEngineSettings.PlatformModel.Automotive;
                    break;
                case 4:
                    platformModel = NavigationEngineSettings.PlatformModel.Sea;
                    break;
                case 5:
                    platformModel = NavigationEngineSettings.PlatformModel.Airbone_1g;
                    break;
                case 6:
                    platformModel = NavigationEngineSettings.PlatformModel.Airbone_2g;
                    break;
                case 7:
                    platformModel = NavigationEngineSettings.PlatformModel.Airbone_4g;
                    break;
            }

            NavigationEngineSettings.setPlatformModel(platformModel, ref buff, ref len);
            gps.TxData(buff, len);
        }

    }
}