﻿using System;

using System.Collections.Generic;
using System.Text;


namespace GPS
{
    /// <summary>
    /// This class is for UBLOX engine settings.
    /// </summary>
    public class NavigationEngineSettings
    {
        private const byte SYNC_CHAR1 = 0xB5;
        private const byte SYNC_CHAR2 = 0x62;
        private static byte[] EngineSettings = new byte[512];

        public enum PlatformModel : int
        {
            Portable = 0,
            Stationary = 2,
            Pedestrian = 3,
            Automotive = 4,
            Sea = 5,
            Airbone_1g = 6,
            Airbone_2g = 7,
            Airbone_4g = 8
        }

        private static bool mkCheckSum(byte[] Buffer, int Len, ref byte CK_A, ref byte CK_B)
        {
            byte ck_a = 0x00;
            byte ck_b = 0x00;

            for (int i = 2; i < Len; i++)
            {
                ck_a = (byte)(ck_a + Buffer[i]);
                ck_b = (byte)(ck_b + ck_a);
            }

            CK_A = ck_a;
            CK_B = ck_b;

            return true;
        }

        public static bool PollConfiguration(ref byte[] buff, ref int len)
        {
            bool bRet = false;
            int count = 0;
            byte CK_A = 0x00;
            byte CK_B = 0x00;
            byte[] command = new byte[30];

            command[count++] = 0xB5;
            command[count++] = 0x62;
            command[count++] = 0x06;
            command[count++] = 0x24;
            command[count++] = 0x00;
            command[count++] = 0x00;

            mkCheckSum(command, count, ref CK_A, ref CK_B);

            command[count++] = CK_A;
            command[count++] = CK_B;

            if (buff.Length < count)
                return false;

            try
            {
                Array.Copy(command, buff, count);
                len = count;
                bRet = true;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                bRet = false;
            }

            return bRet;
        }

        public static bool Parse(byte[] buff, int len)
        {
            /*
            0 Portable
            2 Stationary
            3 Pedestrian
            4 Automotive
            5 Sea
            6 Airborne with <1g Acceleration
            7 Airborne with <2g Acceleration
            8 Airborne with <4g Acceleration
            */

            bool bRet = false;
            PlatformModel nRet = PlatformModel.Portable;

            if (buff.Length < len)
            {
                System.Diagnostics.Debug.WriteLine("invalid args");
                return bRet;
            }

            byte CLASS = 0x06;
            byte ID = 0x24;
            byte CK_A = 0;
            byte CK_B = 0;
            int sizeOfContents = 0;

            if ((buff[0] != SYNC_CHAR1) || (buff[1] != SYNC_CHAR2))
            {
                System.Diagnostics.Debug.WriteLine("invalid args");
                return bRet;
            }

            if ((buff[2] != CLASS) || (buff[3] != ID))
            {
                System.Diagnostics.Debug.WriteLine("invalid args");
                return bRet;
            }

            sizeOfContents = (buff[5] << 5) | buff[4];
            if ((sizeOfContents + 8) != len)
            {
                System.Diagnostics.Debug.WriteLine("invalid args");
                return bRet;
            }

            if (!mkCheckSum(buff, len - 2, ref CK_A, ref CK_B))
            {
                System.Diagnostics.Debug.WriteLine("invalid args");
                return bRet;
            }

            if ((CK_A != buff[len - 2]) || (CK_B != buff[len - 1]))
            {
                System.Diagnostics.Debug.WriteLine("invalid args");
                return bRet;
            }

            Array.Copy(buff, EngineSettings, len);
            bRet = true;

            return bRet;
        }
        public static PlatformModel getPlatformModel()
        {
            return (PlatformModel)EngineSettings[8];
        }
        public static bool setPlatformModel(PlatformModel platformModel, ref byte[] buff, ref int len)
        {
            byte CK_A = 0x00;
            byte CK_B = 0x00;
            
            switch (platformModel)
            {
                case PlatformModel.Portable:
                    EngineSettings[8] = 0x00;
                    break;
                case PlatformModel.Stationary:
                    EngineSettings[8] = 0x02;
                    break;
                case PlatformModel.Pedestrian:
                    EngineSettings[8] = 0x03;
                    break;
                case PlatformModel.Automotive:
                    EngineSettings[8] = 0x04;
                    break;
                case PlatformModel.Sea:
                    EngineSettings[8] = 0x05;
                    break;
                case PlatformModel.Airbone_1g:
                    EngineSettings[8] = 0x06;
                    break;
                case PlatformModel.Airbone_2g:
                    EngineSettings[8] = 0x07;
                    break;
                case PlatformModel.Airbone_4g:
                    EngineSettings[8] = 0x08;
                    break;
            }

            mkCheckSum(EngineSettings, 42, ref CK_A, ref CK_B);
            EngineSettings[42] = CK_A;
            EngineSettings[43] = CK_B;

            len = 44;

            Array.Copy(EngineSettings, buff, len);

            return true;
        }
    }
}
