﻿using System;

using System.Collections.Generic;
using System.Text;
using System.Globalization;
using System.Collections;

namespace GPS
{
    /// <summary>
    /// This class has been implemented with reference to the following information.
    /// http://www.codeproject.com/Articles/9115/Writing-Your-Own-GPS-Applications-Part
    /// </summary>
    public class NmeaParser
    {
        
        public static CultureInfo NmeaCultureInfo = CultureInfo.CurrentCulture;
        // Used to convert knots into miles per hour
        public static double MPHPerKnot = double.Parse("1.150779", NmeaCultureInfo);
        public delegate void PositionReceivedEventHandler(string latitude, string longitude);
        public delegate void DateTimeChangedEventHandler(System.DateTime dateTime);
        public delegate void SpeedReceivedEventHandler(double speed);
        public delegate void SatellitesInViewReceivedEventHandler(int value);
        public delegate void SatellitesInUseReceivedEventHandler(int value);
        public delegate void StautsReceivedEventHandler(bool FixedObtained);
        

        public Form1 frm = null;

        public NmeaParser(Form1 _frm)
        {
            this.frm = _frm;
        }

        private string[] GetWords(string sentence)
        {
            string[] temp = sentence.Split(',');
            int idx = -1;
            for (int i = 0; i < temp.Length; i++)
            {
                idx = temp[i].IndexOf("*");
                if (idx > 0)
                    temp[i] = temp[i].Substring(0, idx);
            }

            return temp;
        }

        private bool ParseGPRMC(string sentence)
        {
            string[] Words = GetWords(sentence);
            if (Words[3] != "" && Words[4] != "" &&
              Words[5] != "" && Words[6] != "")
            {
                // Yes. Extract latitude and longitude
                // Append hours
                string Latitude = Words[3].Substring(0, 2) + "°";
                // Append minutes
                Latitude = Latitude + Words[3].Substring(2) + "\"";
                // Append hours
                Latitude = Latitude + Words[4]; // Append the hemisphere
                string Longitude = Words[5].Substring(0, 3) + "°";
                // Append minutes
                Longitude = Longitude + Words[5].Substring(3) + "\"";
                // Append the hemisphere
                Longitude = Longitude + Words[6];
                // Notify the calling application of the change
                if (frm != null)
                    frm.PositionReceived(Latitude, Longitude);
            }
            // Do we have enough values to parse satellite-derived time?
            if (Words[1] != "" && Words[9] != "")
            {
                // Yes. Extract hours, minutes, seconds and milliseconds
                int UtcHours = Convert.ToInt32(Words[1].Substring(0, 2));
                int UtcMinutes = Convert.ToInt32(Words[1].Substring(2, 2));
                int UtcSeconds = Convert.ToInt32(Words[1].Substring(4, 2));
                int UtcMilliseconds = 0;

                int Day = Convert.ToInt32(Words[9].Substring(0, 2));
                int Month = Convert.ToInt32(Words[9].Substring(2, 2));
                int Year = 2000 + Convert.ToInt32(Words[9].Substring(4, 2));

                // Extract milliseconds if it is available
                if (Words[1].Length > 7)
                {
                    UtcMilliseconds = Convert.ToInt32(
                        float.Parse(Words[1].Substring(6), NmeaCultureInfo) * 1000);
                }
                // Now build a DateTime object with all values
                //System.DateTime Today = System.DateTime.Now.ToUniversalTime();
                System.DateTime SatelliteTime = new System.DateTime(Year, Month, Day, UtcHours, UtcMinutes, UtcSeconds, UtcMilliseconds);
                // Notify of the new time, adjusted to the local time zone
                if (frm != null)
                    frm.DateTimeChanged(SatelliteTime.ToLocalTime());// //DateTimeChanged(SatelliteTime.ToLocalTime());
            }
            // Do we have enough information to extract the current speed?
            if (Words[7] != "")
            {
                // Yes.  Parse the speed and convert it to MPH
                double Speed = double.Parse(Words[7], NmeaCultureInfo) *
                  MPHPerKnot;
                // Notify of the new speed
                if (frm != null)
                    frm.SpeedReceived(Speed);
            }
            
            // Does the device currently have a satellite fix?
            if (Words[2] != "")
            {
                switch (Words[2])
                {
                    case "A":
                        if (frm != null)
                            frm.StatusReceived(true);
                        break;
                    case "V":
                        if (frm != null)
                            frm.StatusReceived(false);
                        break;
                }
            }
            // Indicate that the sentence was recognized
            return true;
        }

        private bool ParseGPGSV(string sentence)
        {
            string[] Words = GetWords(sentence);

            if (Words[3] != "")
            {
                if (frm != null)
                    frm.SatellitesInViewReceived(Convert.ToInt32(Words[3]));
            }
            
            return true;
        }

        private bool ParseGPGSA(string sentence)
        {
            string temp = string.Empty;
            int SatellitesInUse = 0;

            // Divide the sentence into words
            string[] Words = GetWords(sentence);

            for (int i = 3; i < 15; i++)
            {
                if (Words[i] != "" && Words[i] != String.Empty && Words[i] != null)
                    SatellitesInUse++;
            }

            if (frm != null)
                frm.SatellitesInUseReceived(SatellitesInUse);

            
            return true;
        }

        public void Parse(string sentence)
        {
            switch (GetWords(sentence)[0])
            {
                case "$GPRMC":
                    ParseGPRMC(sentence);
                    break;
                case "$GPGSV":
                    ParseGPGSV(sentence);
                    break;
                case "$GPGSA":
                    ParseGPGSA(sentence);
                    break;
            }
        }
    }
}
