﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NMsrApi;
using NSysSvcApi;
using _Native;

namespace MSR
{
    public partial class Form1 : Form
    {
        public MsrApi msr = null;
        public SysSvcApi sysSvc = null;
        public const string title = "MSR Example";

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                tbxData.Location = new Point(0, 0);
            }

            // create system service object
            sysSvc = new SysSvcApi();
            HW_OPTIONS hwoptions = new HW_OPTIONS();
            // Check what type of MSR
            sysSvc.GetHardwareOptions(ref hwoptions);
            if (hwoptions.MSR == HW_TYPE_MSR.NONE)
            {
                MessageBox.Show("MSR is not found!");
                return;
            }

            // create MSR object
            msr = new MsrApi();
            msr.SetCallback(new MsrApi.MSRCALLBACK(msrCallbackProc));
            MSR_RESULT result = msr.Open();
            if (result != MSR_RESULT.MSR_RESULT_SUCCESS)
            {
                MessageBox.Show("Open : " + result.ToString());
                return;
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the MSR object
            if (msr != null)
            {
                msr.CancelReadCard();
                msr.Close();
                msr = null;
            }

            // destruction of the system service object
            if (sysSvc != null)
            {
                sysSvc.Dispose();
                sysSvc = null;
            }
        }

        /// <summary>
        /// When received result from MSR, this function is executed.
        /// </summary>
        /// <param name="MsrResult"></param>
        public void msrCallbackProc(MSR_RESULT MsrResult)
        {
            System.Diagnostics.Debug.WriteLine("msrCallbackProc, " + MsrResult.ToString());
            switch (MsrResult)
            {
                // Succeeded the card data read.
                case MSR_RESULT.MSR_RESULT_SUCCESS:
                    {
                        string strMsrData = string.Empty;
                        string strTemp = string.Empty;

                        string strTrack1 = string.Empty;
                        string strTrack2 = string.Empty;
                        string strTrack3 = string.Empty;
                        string strJIS = string.Empty;

                        if (msr.GetData(ref strTrack1, ref strTrack2, ref strTrack3, ref strJIS) == MSR_RESULT.MSR_RESULT_SUCCESS)
                        {
                            strMsrData = "[Track1] \r\n";

                            if (!string.IsNullOrEmpty(strTrack1))
                            {
                                strMsrData += strTrack1;
                                strMsrData += "\r\n\r\n";
                            }

                            strMsrData += "[Track2] \r\n";
                            if (!string.IsNullOrEmpty(strTrack2))
                            {
                                strMsrData += strTrack2;
                                strMsrData += "\r\n\r\n";
                            }

                            strMsrData += "[Track3] \r\n";
                            if (!string.IsNullOrEmpty(strTrack3))
                            {
                                strMsrData += strTrack3;
                                strMsrData += "\r\n\r\n";
                            }
                            if (!string.IsNullOrEmpty(strTrack1) || !string.IsNullOrEmpty(strTrack2) || !string.IsNullOrEmpty(strTrack3))
                            {
                                _Native.Utils.PlaySound(@"\Windows\Success.wav");
                                tbxData.Text = strMsrData;
                            }
                            
                            btnStartCancel.Text = "StartRead";
                        }
                    }
                    break;

                // Because of the use of the printer function, the function of the MSR is paused.
                case MSR_RESULT.MSR_RESULT_WAITING_END_OF_PRINTING:
                    tbxData.Text = "\r\n\r\n\r\n\r\n\r\n        Waiting for end of printing job...";
                    break;

                // The print function is complete and being able to use the MSR
                case MSR_RESULT.MSR_RESULT_RECEIVED_END_OF_PRINTING:
                    tbxData.Text = String.Empty;
                    break;

                default:
                    tbxData.Text = MsrResult.ToString();
                    _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                    break;
            }
        }

        private void btnStartCancel_Click(object sender, EventArgs e)
        {
            if (msr == null)
                return;

            if (!msr.IsOpened())
            {
                MessageBox.Show("MSR is not opened");
                return;
            }

            MSR_RESULT result = MSR_RESULT.MSR_RESULT_FAILURE;
            if (btnStartCancel.Text.Equals("StartRead"))
            {
                // Asynchronously to start the card reading.
                result = msr.StartReadCard();
                if (result == MSR_RESULT.MSR_RESULT_SUCCESS)
                {
                    tbxData.Text = string.Empty;
                    btnStartCancel.Text = "CancelRead";
                }
                else
                {
                    MessageBox.Show("StartReadCard : " + result.ToString());
                    _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                }
            }
            else
            {
                // Cancel the card read.
                result = msr.CancelReadCard();
                if (result == MSR_RESULT.MSR_RESULT_SUCCESS)
                {
                    btnStartCancel.Text = "StartRead";
                }
                else
                {
                    MessageBox.Show("CancelReadCard : " + result.ToString());
                    _Native.Utils.PlaySound(@"\Windows\Fail.wav");
                }
            }
        }
    }
}