﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using Microsoft.Win32;
using NModemApi;
using NSysSvcApi;
using _Native;

namespace Modem
{
    public partial class Form1 : Form
    {
        public const string title = "Modem Example";
        public ModemApi modem = null;
        public SysSvcApi sysService = null;
        public MODEM_RESULT result = MODEM_RESULT.MODEM_RESULT_FAILURE;
        public RAS.RASError error = RAS.RASError.SUCCESS;
        public HW_TYPE_MODEM modemType = HW_TYPE_MODEM.NONE;
        public string szUserName = string.Empty;
        public string szPassword = string.Empty;
        public IntPtr pRasConn = new IntPtr();
        public NotiMsgWindow msgWnd = null;
        public bool isConnected = false;
        public bool isSynchronously = false;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }
            else
            {
                MessageBox.Show("in Windows Mobile, you can not use this sample. Please refer to the Windows Mobile 6 SDK");
                return;
            }

            // create system service object
            sysService = new SysSvcApi();
            sysService.SleepWakeupNotificationSet(new SysSvcApi.SleepWakeupNotifyCALLBACK(notifyCallbackProc));

            HW_OPTIONS hwOptions = new HW_OPTIONS();
            // Check what type of Modem
            sysService.GetHardwareOptions(ref hwOptions);
            modemType = hwOptions.Modem;
            if (modemType == HW_TYPE_MODEM.NONE)
            {
                sysService.SleepWakeupNotificationReset();
                sysService.Dispose();
                MessageBox.Show("Modem is not found!");
                return;
            }

            // create modem object
            modem = new ModemApi();
            modem.AllocContext();
            modem.SetCallback(new ModemCallbackProc(modemCallbackProc));

            // If RasDial is called asynchronously, you need to create a message windows for receiving RasConnState.
            if (!isSynchronously)
            {
                msgWnd = new NotiMsgWindow();
                msgWnd.OnWndMsg += new NotiMsgWindow.OnWndMsgEventHandler(msgWnd_OnWndMsg);
            }

            tbxEntryName.Text = "MyRasEntry";
            switch (modemType)
            {
                case HW_TYPE_MODEM.AM200:
                    tbxAPN.Text = "do not use this value";
                    tbxUserName.Text = "019";
                    break;
                case HW_TYPE_MODEM.AME200:
                    tbxAPN.Text = "do not use this value";
                    tbxUserName.Text = "phone number";
                    break;
                case HW_TYPE_MODEM.BPLM210:
                    tbxAPN.Text = "do not use this value";
                    break;
                case HW_TYPE_MODEM.HC25:
                case HW_TYPE_MODEM.HC28:
                case HW_TYPE_MODEM.PH8:
                case HW_TYPE_MODEM.MC55I:
                    tbxAPN.Text = "www.aitd1.com";
                    tbxUserName.Text = "ai@vodafone";
                    tbxPassword.Text = "vodafone";
                    break;
            }

            // CDMA, EVDO modem used in Korea
            if (modemType == HW_TYPE_MODEM.AM200 || modemType == HW_TYPE_MODEM.AME200)
            {
                tbxAPN.Enabled = false;
                btnCardStatus.Enabled = false;
                btnAuthStatus.Enabled = false;
                btnGetRssi.Enabled = false;
                btnGetNetStatus.Enabled = false;
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (modemType == HW_TYPE_MODEM.AM200 || modemType == HW_TYPE_MODEM.AME200)
            {
                if((modem != null) && (modem.GetPowerStatus() != MODEM_POWER_STATUS.MODEM_POWER_STATUS_OFF))
                {
                    MessageBox.Show("Turn off the modem power");
                    e.Cancel = true;
                    return;
                }
            }

            // destruction of the system service object
            if (sysService != null)
            {
                sysService.SleepWakeupNotificationReset();
                sysService.Dispose();
                sysService = null;
            }

            // If the state of the modem is connected to a data service, you must close the connection.
            if (isConnected)
            {
                RAS.RasHangUp(pRasConn);
                pRasConn = IntPtr.Zero;
            }

            if (modem != null)
            {
                if (modemType != HW_TYPE_MODEM.AM200 && modemType != HW_TYPE_MODEM.AME200)
                {
                    // Close the modem port.
                    if (modem.IsOpen())
                        modem.Close();
                }

                // Turn off the modem power synchronously.
                if (modem.GetPowerStatus() == MODEM_POWER_STATUS.MODEM_POWER_STATUS_ON)
                    modem.PowerEnable(false);

                // destruction of the modem object
                modem.DeallocContext();
                modem = null;
            }

            //destruction of the message window object
            if (msgWnd != null)
            {
                msgWnd.Dispose();
                msgWnd = null;
            }
        }

        // When RasDial have called asynchronously, function receives a RasConnState.
        void msgWnd_OnWndMsg(int nMessage, IntPtr wParam, IntPtr lParam)
        {
            if (nMessage == RAS.WM_RASDIALEVENT)
            {
                RAS.RasConnState rasConnState = (RAS.RasConnState)wParam.ToInt32();

                // Successfully connected to a data service
                if (rasConnState == RAS.RasConnState.RASCS_Connected)
                {
                    isConnected = true;
                    EnableControls(true);
                }
                // disconnected data service connection
                else if (rasConnState == RAS.RasConnState.RASCS_Disconnected)
                {
                    isConnected = false;
                    EnableControls(true);
                    RAS.RasHangUp(pRasConn);
                    pRasConn = IntPtr.Zero;
                }

                AddString("RAS : " + rasConnState.ToString());
            }
        }

        public void AddString(string str)
        {
            if (lbxLog.Items.Count > 1024)
                lbxLog.Items.Clear();

            lbxLog.Items.Add(str);
            lbxLog.SelectedIndex = lbxLog.Items.Count - 1;
        }

        /// <summary>
        /// When the PDA is entering suspend mode, or when waking up from suspend mode, this function is executed.
        /// </summary>
        /// <param name="enterSleep"></param>
        public void notifyCallbackProc(bool enterSleep)
        {
            // When the PDA is entering suspend mode
            if (enterSleep)
            {
                // If the state of the modem is connected to a data service, you must close the connection.
                if (isConnected)
                {
                    RAS.RasHangUp(pRasConn);
                    pRasConn = IntPtr.Zero;
                    isConnected = false;
                }

                if (modemType != HW_TYPE_MODEM.AM200 && modemType != HW_TYPE_MODEM.AME200)
                {
                    // If the port is opened, you must close port
                    if (modem.IsOpen())
                        modem.Close();

                    // If the modem is powered on, you must turn off the power.
                    if (modem.GetPowerStatus() == MODEM_POWER_STATUS.MODEM_POWER_STATUS_ON)
                        modem.PowerEnable(false);
                }
            }
        }

        /// <summary>
        /// When modem state is changed, this function is executed.
        /// </summary>
        /// <param name="cbData"></param>
        public void modemCallbackProc(MODEM_CALLBACK_DATA cbData)
        {
            MODEM_SYS_NOTI msg = (MODEM_SYS_NOTI)cbData.m_UserWnMsg;
            switch (msg)
            {
                case MODEM_SYS_NOTI.PWR_UP_INITIALIZE:
                    AddString("PowerUp Initialize...");
                    break;
                case MODEM_SYS_NOTI.PWR_UP_SUCCESS: // Complete Modem Power On
                    AddString("PowerUp : SUCCESS");

                    // Only for the case of EVDO modem.(LGU+)
                    // You need a phone number for the user name of RASDIAL.
                    // You should read the phone numbers are stored in the registry.
                    if (modemType == HW_TYPE_MODEM.AME200)
                    {
                        string UserName = ((string[])Registry.GetValue(@"HKEY_CURRENT_USER\Software\ATID", "PhoneNumber", "01000000000"))[0];
                        tbxUserName.Text = UserName;
                    }

                    EnableControls(true);
                    break;
                case MODEM_SYS_NOTI.PWR_DOWN_SUCCESS: // Complete Modem Power Off
                    AddString("PowerDown : SUCCESS");
                    EnableControls(true);
                    break;
                case MODEM_SYS_NOTI.PWR_UP_FAIL:
                    AddString("PowerUp : FAIL");
                    EnableControls(true);
                    break;
                case MODEM_SYS_NOTI.PWR_DOWN_FAIL:
                    AddString("PowerDown : FAIL");
                    EnableControls(true);
                    break;
            }
        }

        /// <summary>
        /// In order to run the RasDial, we must necessarily have RasEntryName.
        /// </summary>
        /// <param name="APN">Access Point Name</param>
        /// <returns></returns>
        public RAS.RASError CreateRasEntryName(string APN)
        {
            string apnString = string.Empty;
            RAS.RASError error = RAS.RASError.ERROR_UNKNOWN;
            RAS.RASENTRY RasEntry = new RAS.RASENTRY();
            string phoneNumber = string.Empty;
            string entryName = tbxEntryName.Text.Trim();
           
            switch (modemType)
            {
                // LTE(Korea LGU+)
                case HW_TYPE_MODEM.BPLM210:
                    RasEntry.szDeviceName = RAS.DEVICE_NAME_BPLM210;
                    phoneNumber = "*98#";
                    break;
                // WCDMA 3G
                case HW_TYPE_MODEM.HC25:
                case HW_TYPE_MODEM.HC28:
                case HW_TYPE_MODEM.PH8:
                    RasEntry.szDeviceName = RAS.DEVICE_NAME_HC25_HC28_PH8;
                    phoneNumber = "*99#";
                    break;
                // CDMA(Korea LGU+) 
                case HW_TYPE_MODEM.AM200:
                    RasEntry.szDeviceName = RAS.DEVICE_NAME_UART;
                    phoneNumber = "019";
                    break;
                // GSM
                case HW_TYPE_MODEM.MC55I:
                    RasEntry.szDeviceName = RAS.DEVICE_NAME_UART;
                    phoneNumber = "*99***1#";
                    break;
                // EVDO(Korea LGU+)
                case HW_TYPE_MODEM.AME200:
                    RasEntry.szDeviceName = RAS.DEVICE_NAME_AME200;
                    phoneNumber = "#777";
                    break;
                default:
                    System.Diagnostics.Debug.WriteLine(string.Format("ModemType:{0}, Not supported!", modemType.ToString()));
                    return RAS.RASError.ERROR_UNKNOWN;
            }

            RasEntry.dwSize = 3472;
            RasEntry.dwfOptions = RAS.RASEO_ProhibitEAP | RAS.RASEO_SwCompression | RAS.RASEO_IpHeaderCompression;
            RasEntry.dwCountryCode = 82;
            RasEntry.szAreaCode = "2";
            RasEntry.szLocalPhoneNumber = phoneNumber;
            RasEntry.dwfNetProtocols = 4;
            RasEntry.dwFramingProtocol = 1;
            RasEntry.szDeviceType = RAS.RASDT_Modem;

            byte[] pDevConfig = new byte[104];
            for (int n = 0; n < 104; n++)
                pDevConfig[n] = 0x00;

            pDevConfig[0] = 0x30;
            pDevConfig[4] = 0x05;
            pDevConfig[8] = 0x10;
            pDevConfig[9] = 0x01;
            pDevConfig[13] = 0xC2;
            pDevConfig[14] = 0x01;
            pDevConfig[18] = 0x08;

            if (modemType != HW_TYPE_MODEM.AM200 && modemType != HW_TYPE_MODEM.AME200 && modemType != HW_TYPE_MODEM.BPLM210)
            {
                apnString = String.Format("+CGDCONT=1,\"IP\",\"{0}\"", APN);
                int idx = 22;
                if (APN.Length != 0)
                {
                    foreach (char ch in apnString)
                    {
                        pDevConfig[idx] = Convert.ToByte(ch);
                        idx += 2;
                    }
                }
            }

            error = RAS.RasSetEntryProperties(null, entryName, ref RasEntry, RasEntry.dwSize, pDevConfig, pDevConfig.Length);
            if (error != RAS.RASError.SUCCESS)
                return error;

            RAS.SetDialingParameters("0", "Work", "G", "G", "G", "", "", "", "1");
            RAS.SetDialingParameters("1", "Home", "G", "G", "G", "", "", "", "1");
            RAS.SetDialingParameters("2", "Data", "G", "G", "G", "", "", "", "1");

            return error;
        }

        public bool modifyAPN(string entryName, string APN)
        {
            bool bRet = false;
            RAS.RASENTRY rasEntry = new RAS.RASENTRY();
            string apnString = string.Empty;
            rasEntry.dwSize = Marshal.SizeOf(rasEntry);
            int sizeOfRasEntry = Marshal.SizeOf(rasEntry);
            byte[] pDevConfig = new byte[104];
            int sizeOfDevConfig = pDevConfig.Length;

            if (string.IsNullOrEmpty(entryName))
                return bRet;

            RAS.RASError error = RAS.RasGetEntryProperties(null, entryName, ref rasEntry, ref sizeOfRasEntry, pDevConfig, ref sizeOfDevConfig);
            if (error != RAS.RASError.SUCCESS)
            {
                AddString("RasGetEntryProperties : " + error.ToString());
                return bRet;
            }
            
            apnString = String.Format("+CGDCONT=1,\"IP\",\"{0}\"", APN);
            int idx = 22;
            for (int i = idx; i < pDevConfig.Length; i++)
                pDevConfig[i] = 0x00;

            if (APN.Length != 0)
            {
                foreach (char ch in apnString)
                {
                    pDevConfig[idx] = Convert.ToByte(ch);
                    idx += 2;
                }
            }

            error = RAS.RasSetEntryProperties(null, entryName, ref rasEntry, rasEntry.dwSize, pDevConfig, pDevConfig.Length);
            if (error != RAS.RASError.SUCCESS)
            {
                bRet = false;
                AddString("RasSetEntryProperties : " + error.ToString());
            }
            else
                bRet = true;

            return bRet;
        }

        public bool DoRasDial()
        {
            bool bRet = false;
            RAS.RASDIALPARAMS rasDialParam = new RAS.RASDIALPARAMS();
            string rasEntryName = tbxEntryName.Text.Trim();
            string APN = tbxAPN.Text.Trim();

            // The following modems do not have to enter the APN to RasEntryName.
            if (modemType == HW_TYPE_MODEM.AM200 || modemType == HW_TYPE_MODEM.AME200 || 
                modemType == HW_TYPE_MODEM.BPLM210)
                APN = string.Empty;

            // Make sure that you have already made RasEntryName
            error = RAS.RasValidateEntryName(null, rasEntryName);
            if (error == RAS.RASError.SUCCESS)
            {
                // If you do not have RasEntryName, create a new one.
                error = CreateRasEntryName(APN);
                if (error != RAS.RASError.SUCCESS)
                {
                    Utils.PlaySound(@"\Windows\Fail.wav");
                    AddString("CreateRasEntryName : " + error.ToString());
                    return bRet;
                }
            }
            else
            {
                // If it has already been created, is updated with the new information.
                modifyAPN(rasEntryName, APN);
            }

            rasDialParam.dwSize = Marshal.SizeOf(rasDialParam);
            rasDialParam.szEntryName = rasEntryName;
            rasDialParam.szUserName = tbxUserName.Text.Trim();
            rasDialParam.szPassword = tbxPassword.Text.Trim();

            // If 5th parameter is set to IntPtr.Zero, rasdial operates synchronously.
            // Please refer to RasDial on MSDN for more information.
            error = RAS.RasDial(IntPtr.Zero, 
                                IntPtr.Zero, 
                                ref rasDialParam, 
                                0xffffffff,
                                isSynchronously ? IntPtr.Zero : msgWnd.Hwnd, 
                                ref pRasConn);

            AddString("RasDial : " + error.ToString());
            if (error != RAS.RASError.SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");

                // Even if RasDial fails, you must invoke the RasHangUp to prevent memory leaks.
                RAS.RasHangUp(pRasConn);
                pRasConn = IntPtr.Zero;
            }
            else
            {
                bRet = true;
            }

            if (isSynchronously)
                isConnected = (error == RAS.RASError.SUCCESS ? true : false);

            return bRet;
        }

        public void EnableControls(bool enable)
        {
            btnPowerUp.Enabled = enable;
            btnPowerDown.Enabled = enable;
            tbxEntryName.Enabled = enable;
            tbxUserName.Enabled = enable;
            tbxPassword.Enabled = enable;
            btnRasDial.Enabled = enable;
            btnRasHangUp.Enabled = enable;

            switch (modemType)
            {
                case HW_TYPE_MODEM.BPLM210:
                    btnGetRssi.Enabled = enable;
                    btnGetNetStatus.Enabled = enable;
                    btnCardStatus.Enabled = enable;
                    btnAuthStatus.Enabled = enable;
                    tbxAPN.Enabled = enable;
                    break;
                case HW_TYPE_MODEM.AM200:
                case HW_TYPE_MODEM.AME200:
                    break;
                default:
                    btnGetRssi.Enabled = enable;
                    btnGetNetStatus.Enabled = enable;
                    btnCardStatus.Enabled = enable;
                    btnAuthStatus.Enabled = enable;
                    tbxAPN.Enabled = enable;
                    break;
            }
        }

        private void btnPowerUp_Click(object sender, EventArgs e)
        {
            if (modem == null)
            {
                AddString("Modem is null");
                return;
            }

            EnableControls(false);

            // Turn on the modem power asynchronously.
            // If PWR_UP_SUCCESS is received by modemCallbackProc function, and the power is completely turned on.
            result = modem.PowerUp();
            if (result != MODEM_RESULT.MODEM_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("PowerUp : " + result.ToString());
                EnableControls(true);
            }
        }

        private void btnPowerDown_Click(object sender, EventArgs e)
        {
            if (modem == null)
            {
                AddString("Modem is null");
                return;
            }

            // If the state of the modem is connected to a data service, you must close the connection.
            if (isConnected)
            {
                EnableControls(false);
                RAS.RasHangUp(pRasConn);
                pRasConn = IntPtr.Zero;
                isConnected = false;
            }

            if (modemType != HW_TYPE_MODEM.AM200 && modemType != HW_TYPE_MODEM.AME200)
            {
                // If the port is opened, you must close port
                if (modem.IsOpen())
                {
                    EnableControls(false);
                    modem.Close();
                }
            }

            EnableControls(false);

            // Turn off the modem power asynchronously.
            // If PWR_DOWN_SUCCESS is received by modemCallbackProc function, and the power is completely turned off.
            result = modem.PowerDown();
            if (result != MODEM_RESULT.MODEM_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("PowerDown : " + result.ToString());
                EnableControls(true);
            }
        }

        private void btnRasDial_Click(object sender, EventArgs e)
        {
            if (modem == null)
            {
                AddString("Modem is null");
                return;
            }

            if (modem.GetPowerStatus() != MODEM_POWER_STATUS.MODEM_POWER_STATUS_ON)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("Not powered on");
                return;
            }

            if (isConnected)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("Already connectd");
                return;
            }

            EnableControls(false);

            DoRasDial();

            if(isSynchronously)
                EnableControls(true);
        }

        private void btnRasHangUp_Click(object sender, EventArgs e)
        {
            if (modem == null)
            {
                AddString("Modem is null");
                return;
            }

            if (modem.GetPowerStatus() != MODEM_POWER_STATUS.MODEM_POWER_STATUS_ON)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("Not powered on");
                return;
            }

            if (!isConnected)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("Not connectd");
                return;
            }

            EnableControls(false);
            error = RAS.RasHangUp(pRasConn);
            pRasConn = IntPtr.Zero;
            if (error == RAS.RASError.SUCCESS)
                isConnected = false;
            EnableControls(true);
            AddString("RasHangUp : " + error.ToString());
        }

        private void btnGetRssi_Click(object sender, EventArgs e)
        {
            if (modem == null)
            {
                AddString("Modem is null");
                return;
            }

            int rssi = 0;

            if (modem.GetPowerStatus() != MODEM_POWER_STATUS.MODEM_POWER_STATUS_ON)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("Not powered on");
                return;
            }

            if (!modem.IsOpen())
            {
                EnableControls(false);
                modem.Open();
            }

            EnableControls(false);
            result = modem.GetRSSI(ref rssi);
            EnableControls(true);
            if (result != MODEM_RESULT.MODEM_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("GetRSSI : " + result.ToString());
            }
            else
            {
                AddString("RSSI : " + rssi.ToString());
            }
        }

        private void btnGetNetStatus_Click(object sender, EventArgs e)
        {
            if (modem == null)
            {
                AddString("Modem is null");
                return;
            }

            MODEM_NETWORK_REGISTRATION_STATUS status = MODEM_NETWORK_REGISTRATION_STATUS.UNKNOWN;

            if (modem.GetPowerStatus() != MODEM_POWER_STATUS.MODEM_POWER_STATUS_ON)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("Not powered on");
                return;
            }

            if (!modem.IsOpen())
            {
                EnableControls(false);
                modem.Open();
            }

            EnableControls(false);
            status = modem.GetNetworkRegistrationStatus();
            AddString(status.ToString());
            EnableControls(true);
        }

        private void btnCardStatus_Click(object sender, EventArgs e)
        {
            if (modem == null)
            {
                AddString("Modem is null");
                return;
            }

            MODEM_SIM_PIN_AUTH_STATUS status = MODEM_SIM_PIN_AUTH_STATUS.SIM_FAILURE;

            if (modem.GetPowerStatus() != MODEM_POWER_STATUS.MODEM_POWER_STATUS_ON)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("Not powered on");
                return;
            }

            if (!modem.IsOpen())
            {
                EnableControls(false);
                modem.Open();
            }

            EnableControls(false);
            status = modem.SimQueryCardHolderStatus();
            AddString(status.ToString());
            EnableControls(true);
        }

        private void btnAuthStatus_Click(object sender, EventArgs e)
        {
            if (modem == null)
            {
                AddString("Modem is null");
                return;
            }

            MODEM_SIM_PIN_AUTH_STATUS status = MODEM_SIM_PIN_AUTH_STATUS.SIM_FAILURE;
            Form2 frmAuthWnd;
            string Code = string.Empty;
            string NewCode = string.Empty;

            if (modem.GetPowerStatus() != MODEM_POWER_STATUS.MODEM_POWER_STATUS_ON)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString("Not powered on");
                return;
            }

            if (!modem.IsOpen())
            {
                EnableControls(false);
                modem.Open();
            }

            EnableControls(false);
            status = modem.SimQueryAuthStatus();
            switch (status)
            {
                case MODEM_SIM_PIN_AUTH_STATUS.SIM_READY:
                    AddString("Ready");
                    break;

                case MODEM_SIM_PIN_AUTH_STATUS.SIM_PIN:
                    AddString("PIN");
                    frmAuthWnd = new Form2("SIM_PIN");
                    if (frmAuthWnd.ShowDialog() == DialogResult.OK)
                    {
                        Code = frmAuthWnd.Code;
                        status = modem.SimEnterPin(Code);
                        switch (status)
                        {
                            case MODEM_SIM_PIN_AUTH_STATUS.INCORRECT_PASSWORD:
                                AddString("Incorrect password");
                                break;
                            case MODEM_SIM_PIN_AUTH_STATUS.INVALID_INPUT_VALUE:
                                AddString("Incorrect input value");
                                break;
                        }

                        if (modem.SimQueryAuthStatus() == MODEM_SIM_PIN_AUTH_STATUS.SIM_READY)
                            AddString("Ready");
                        else
                            AddString("SimEnterPin failed");
                    }
                    break;

                case MODEM_SIM_PIN_AUTH_STATUS.SIM_PUK:
                    AddString("PUK");
                    frmAuthWnd = new Form2("SIM_PUK");
                    if (frmAuthWnd.ShowDialog() == DialogResult.OK)
                    {
                        Code = frmAuthWnd.Code;
                        NewCode = frmAuthWnd.NewCode;
                        status = modem.SimEnterPuk(Code, NewCode);
                        switch (status)
                        {
                            case MODEM_SIM_PIN_AUTH_STATUS.INCORRECT_PASSWORD:
                                AddString("Incorrect password");
                                break;
                            case MODEM_SIM_PIN_AUTH_STATUS.INVALID_INPUT_VALUE:
                                AddString("Incorrect input value");
                                break;
                        }

                        if (modem.SimQueryAuthStatus() == MODEM_SIM_PIN_AUTH_STATUS.SIM_READY)
                            AddString("Ready");
                        else
                            AddString("SimEnterPin failed");
                    }
                    break;

                case MODEM_SIM_PIN_AUTH_STATUS.CARD_HOLDER_TRAY_REMOVED:
                    AddString("SIM not inserted");
                    break;
            }

            EnableControls(true);
        }
       
    }
}