﻿using System;

using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.WindowsCE.Forms;
using Microsoft.Win32;

namespace _Native
{
    public enum PlaySoundFlags : int
    {
        SND_SYNC = 0x0,     // play synchronously (default)
        SND_ASYNC = 0x1,    // play asynchronously
        SND_NODEFAULT = 0x2,    // silence (!default) if sound not found
        SND_MEMORY = 0x4,       // pszSound points to a memory file
        SND_LOOP = 0x8,     // loop the sound until next sndPlaySound
        SND_NOSTOP = 0x10,      // don't stop any currently playing sound
        SND_NOWAIT = 0x2000,    // don't wait if the driver is busy
        SND_ALIAS = 0x10000,    // name is a registry alias
        SND_ALIAS_ID = 0x110000,// alias is a predefined ID
        SND_FILENAME = 0x20000, // name is file name
        SND_RESOURCE = 0x40004, // name is resource name or atom
    }

    public class BarcodeEmulatorInfo
    {
        public static string FileName
        {
            get
            {

                return Utils.IsWinCE ? @"\Windows\DEMO_ScanEmulator.exe"
                                     : @"\Program Files\ATID_DemoApps\BarcodeEmulatorWM.exe";
            }
        }
        public static string WindowClass
        {
            get
            {

                return Utils.IsWinCE ? "ScanEmul"
                                     : "__ATID__BARCODE__EMULATOR__";
            }
        }
        public static string WindowTitle
        {
            get
            {

                return Utils.IsWinCE ? "ScanEmul"
                                     : "BarcodeEmulator";
            }
        }
        public static uint CloseID
        {
            get
            {
                return Utils.IsWinCE ? (uint)40011
                                     : (uint)40001;
            }
        }
    }

    public class Utils
    {
        public const uint WM_DESTROY = 0x0002;
        public const uint WM_CLOSE = 0x0010;
        public const uint WM_QUIT = 0x0012;
        public const uint WM_COMMAND = 0x0111;
        public const string NETCF_WND_CLASS_NAME = "#NETCF_AGL_BASE_";

        [DllImport("coredll.dll", EntryPoint = "SendMessage", SetLastError = true)]
        public static extern IntPtr SendMessage(IntPtr hWnd, uint Msg, uint wParam, uint lParam);

        [DllImport("coredll.dll", EntryPoint = "FindWindowW", SetLastError = true)]
        public static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        [DllImport("coredll.dll", EntryPoint = "SetForegroundWindow", SetLastError = true)]
        public static extern bool SetForegroundWindow(IntPtr hWnd);

        [DllImport("coredll.dll", EntryPoint = "ShowWindow", SetLastError = true)]
        public static extern bool ShowWindow(IntPtr hWnd, int nCmdShow);

        [DllImport("coredll.dll", EntryPoint = "PlaySound", SetLastError = true)]
        public static extern int PlaySound(string szSound, IntPtr hModule, int flags);

        public static bool PlaySound(string szSound)
        {
            int nRtn = PlaySound(szSound, IntPtr.Zero, (int)(PlaySoundFlags.SND_FILENAME | PlaySoundFlags.SND_ASYNC));
            return nRtn > 0 ? true : false;
        }

        public static bool IsWinCE
        {
            get
            {
                return Environment.OSVersion.Version.Minor == 2 ? false : true;
            }
        }
    }

    public class NotiMsgWindow : Microsoft.WindowsCE.Forms.MessageWindow
    {
        public delegate void OnWndMsgEventHandler(int nMessage, IntPtr wParam, IntPtr lParam);
        public event OnWndMsgEventHandler OnWndMsg = null;

        public NotiMsgWindow()
        {
        }

        protected override void WndProc(ref Message m)
        {
            if (OnWndMsg != null)
            {
                OnWndMsg(m.Msg, m.WParam, m.LParam);
            }

            base.WndProc(ref m);
        }
    }

    public class RAS
    {
        public const uint RAS_MaxEntryName = 20;
        public const uint RAS_MaxDeviceName = 128;
        public const uint RAS_MaxDeviceType = 16;
        public const uint RAS_MaxParamKey = 32;
        public const uint RAS_MaxParamValue = 128;
        public const uint RAS_MaxPhoneNumber = 128;
        public const uint RAS_MaxCallbackNumber = 48;
        public const uint RAS_MaxIpAddress = 15;
        public const uint RAS_MaxIpxAddress = 21;

        // Ras extensions

        public const uint RAS_MaxAreaCode = (10);
        public const uint RAS_MaxPadType = (32);
        public const uint RAS_MaxX25Address = (200);
        public const uint RAS_MaxFacilities = (200);
        public const uint RAS_MaxUserData = (200);

        /* RASENTRY 'dwfOptions' bit flags.
        */
        public const uint RASEO_UseCountryAndAreaCodes = 0x00000001;
        public const uint RASEO_SpecificIpAddr = 0x00000002;
        public const uint RASEO_SpecificNameServers = 0x00000004;
        public const uint RASEO_IpHeaderCompression = 0x00000008;
        public const uint RASEO_RemoteDefaultGateway = 0x00000010;
        public const uint RASEO_DisableLcpExtensions = 0x00000020;
        public const uint RASEO_TerminalBeforeDial = 0x00000040;
        public const uint RASEO_TerminalAfterDial = 0x00000080;
        public const uint RASEO_ModemLights = 0x00000100;
        public const uint RASEO_SwCompression = 0x00000200;
        public const uint RASEO_RequireEncryptedPw = 0x00000400;
        public const uint RASEO_RequireMsEncryptedPw = 0x00000800;
        public const uint RASEO_RequireDataEncryption = 0x00001000;
        public const uint RASEO_NetworkLogon = 0x00002000;
        public const uint RASEO_UseLogonCredentials = 0x00004000;
        public const uint RASEO_PromoteAlternates = 0x00008000;
        public const uint RASEO_SecureLocalFiles = 0x00010000;
        public const uint RASEO_DialAsLocalCall = 0x00020000;

        public const uint RASEO_ProhibitPAP = 0x00040000;
        public const uint RASEO_ProhibitCHAP = 0x00080000;
        public const uint RASEO_ProhibitMsCHAP = 0x00100000;
        public const uint RASEO_ProhibitMsCHAP2 = 0x00200000;
        public const uint RASEO_ProhibitEAP = 0x00400000;
        public const uint RASEO_PreviewUserPw = 0x01000000;
        public const uint RASEO_NoUserPwRetryDialog = 0x02000000;
        public const uint RASEO_CustomScript = 0x80000000;


        /* RASENTRY 'dwfNetProtocols' bit flags. (session negotiated protocols)
        */
        public const uint RASNP_NetBEUI = 0x00000001;  // Negotiate NetBEUI
        public const uint RASNP_Ipx = 0x00000002;  // Negotiate IPX
        public const uint RASNP_Ip = 0x00000004;  // Negotiate TCP/IP


        /* RASENTRY 'dwFramingProtocols' (framing protocols used by the server)
        */
        public const uint RASFP_Ppp = 0x00000001;  // Point-to-Point Protocol (PPP)
        public const uint RASFP_Slip = 0x00000002;  // Serial Line Internet Protocol (SLIP)
        public const uint RASFP_Ras = 0x00000004;  // Microsoft proprietary protocol

        /* RASENTRY 'szDeviceType' strings
        */
        public const string RASDT_Direct = "direct";    // Direct Connect (WINCE Extension)
        public const string RASDT_Modem = "modem";     // Modem
        public const string RASDT_Isdn = "isdn";      // ISDN
        public const string RASDT_X25 = "x25";       // X.25
        public const string RASDT_Vpn = "vpn";		  // PPTP
        public const string RASDT_PPPoE = "PPPoE";     // PPPoE

        // Enumerates intermediate states to a connection.  (See RasDial)

        public const uint RASCS_PAUSED = 0x1000;
        public const uint RASCS_DONE = 0x2000;

        // 'dwfOptions' bit flags.

        public const uint RDEOPT_UsePrefixSuffix = 0x00000001;
        public const uint RDEOPT_PausedStates = 0x00000002;
        public const uint RDEOPT_IgnoreModemSpeaker = 0x00000004;
        public const uint RDEOPT_SetModemSpeaker = 0x00000008;
        public const uint RDEOPT_IgnoreSoftwareCompression = 0x00000010;
        public const uint RDEOPT_SetSoftwareCompression = 0x00000020;

        public const uint RASBASE = 600;
        public const string RASDIALEVENT = "RasDialEvent";
        public const uint WM_RASDIALEVENT = 0xCCCD;

        public const string DEVICE_NAME_UART = "Modem(COM8):";
        public const string DEVICE_NAME_AME200 = "ACM1.0 Modem,cdc";
        public const string DEVICE_NAME_HC25_HC28_PH8 = "HSDPA-MDM";
        public const string DEVICE_NAME_BPLM210 = "LTE NMEA Port";

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        public struct RASCONNSTATUS
        {
            public int dwSize;
            public int rasconnstate;
            public int dwError;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxDeviceType + 1)]
            public string szDeviceType;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxDeviceName + 1)]
            public string szDeviceName;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        public struct RASDIALPARAMS
        {
            public int dwSize;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxEntryName + 1)]
            public string szEntryName;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxPhoneNumber + 1)]
            public string szPhoneNumber;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxCallbackNumber + 1)]
            public string szCallbackNumber;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256 + 1)]
            public string szUserName;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256 + 1)]
            public string szPassword;

            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 15 + 1)]
            public string szDomain;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        public struct RASIPADDR
        {
            byte a;
            byte b;
            byte c;
            byte d;
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        public struct RASENTRY
        {
            public int dwSize;
            public uint dwfOptions;
            //
            // Location/phone number.
            //
            public int dwCountryID;
            public int dwCountryCode;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxAreaCode + 1)]
            public string szAreaCode;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxPhoneNumber + 1)]
            public string szLocalPhoneNumber;
            public int dwAlternateOffset;
            //
            // PPP/Ip
            //
            public RASIPADDR ipaddr;
            public RASIPADDR ipaddrDns;
            public RASIPADDR ipaddrDnsAlt;
            public RASIPADDR ipaddrWins;
            public RASIPADDR ipaddrWinsAlt;
            //
            // Framing
            //
            public int dwFrameSize;
            public int dwfNetProtocols;
            public int dwFramingProtocol;
            //
            // Scripting
            //
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]//MAX_PATH
            public string szScript;
            //
            // AutoDial
            //
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]//MAX_PATH
            public string szAutodialDll;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]//MAX_PATH
            public string szAutodialFunc;
            //
            // Device
            //
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxDeviceType + 1)]
            public string szDeviceType;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxDeviceName + 1)]
            public string szDeviceName;
            //
            // X.25
            //
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxPadType + 1)]
            public string szX25PadType;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxX25Address + 1)]
            public string szX25Address;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxFacilities + 1)]
            public string szX25Facilities;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = (int)RAS_MaxUserData + 1)]
            public string szX25UserData;
            public int dwChannels;
            //
            // Reserved
            //
            public int dwReserved1;
            public int dwReserved2;
            public int dwCustomAuthKey;      // authentication key for EAP
        }

        public enum RASError : uint
        {
            SUCCESS = 0,

            ERROR_INVALID_NAME = 123,
            ERROR_ALREADY_EXISTS = 183,

            PENDING = RASBASE + 0,
            ERROR_INVALID_PORT_HANDLE = RASBASE + 1,
            ERROR_PORT_ALREADY_OPEN = RASBASE + 2,
            ERROR_BUFFER_TOO_SMALL = RASBASE + 3,
            ERROR_WRONG_INFO_SPECIFIED = RASBASE + 4,
            ERROR_CANNOT_SET_PORT_INFO = RASBASE + 5,
            ERROR_PORT_NOT_CONNECTED = RASBASE + 6,
            ERROR_EVENT_INVALID = RASBASE + 7,
            ERROR_DEVICE_DOES_NOT_EXIST = RASBASE + 8,
            ERROR_DEVICETYPE_DOES_NOT_EXIST = RASBASE + 9,
            ERROR_BUFFER_INVALID = RASBASE + 10,
            ERROR_ROUTE_NOT_AVAILABLE = RASBASE + 11,
            ERROR_ROUTE_NOT_ALLOCATED = RASBASE + 12,
            ERROR_INVALID_COMPRESSION_SPECIFIED = RASBASE + 13,
            ERROR_OUT_OF_BUFFERS = RASBASE + 14,
            ERROR_PORT_NOT_FOUND = RASBASE + 15,
            ERROR_ASYNC_REQUEST_PENDING = RASBASE + 16,
            ERROR_ALREADY_DISCONNECTING = RASBASE + 17,
            ERROR_PORT_NOT_OPEN = RASBASE + 18,
            ERROR_PORT_DISCONNECTED = RASBASE + 19,
            ERROR_NO_ENDPOINTS = RASBASE + 20,
            ERROR_CANNOT_OPEN_PHONEBOOK = RASBASE + 21,
            ERROR_CANNOT_LOAD_PHONEBOOK = RASBASE + 22,
            ERROR_CANNOT_FIND_PHONEBOOK_ENTRY = RASBASE + 23,
            ERROR_CANNOT_WRITE_PHONEBOOK = RASBASE + 24,
            ERROR_CORRUPT_PHONEBOOK = RASBASE + 25,
            ERROR_CANNOT_LOAD_STRING = RASBASE + 26,
            ERROR_KEY_NOT_FOUND = RASBASE + 27,
            ERROR_DISCONNECTION = RASBASE + 28,
            ERROR_REMOTE_DISCONNECTION = RASBASE + 29,
            ERROR_HARDWARE_FAILURE = RASBASE + 30,
            ERROR_USER_DISCONNECTION = RASBASE + 31,
            ERROR_INVALID_SIZE = RASBASE + 32,
            ERROR_PORT_NOT_AVAILABLE = RASBASE + 33,
            ERROR_CANNOT_PROJECT_CLIENT = RASBASE + 34,
            ERROR_UNKNOWN = RASBASE + 35,
            ERROR_WRONG_DEVICE_ATTACHED = RASBASE + 36,
            ERROR_BAD_STRING = RASBASE + 37,
            ERROR_REQUEST_TIMEOUT = RASBASE + 38,
            ERROR_CANNOT_GET_LANA = RASBASE + 39,
            ERROR_NETBIOS_ERROR = RASBASE + 40,
            ERROR_SERVER_OUT_OF_RESOURCES = RASBASE + 41,
            ERROR_NAME_EXISTS_ON_NET = RASBASE + 42,
            ERROR_SERVER_GENERAL_NET_FAILURE = RASBASE + 43,
            WARNING_MSG_ALIAS_NOT_ADDED = RASBASE + 44,
            ERROR_AUTH_INTERNAL = RASBASE + 45,
            ERROR_RESTRICTED_LOGON_HOURS = RASBASE + 46,
            ERROR_ACCT_DISABLED = RASBASE + 47,
            ERROR_PASSWD_EXPIRED = RASBASE + 48,
            ERROR_NO_DIALIN_PERMISSION = RASBASE + 49,
            ERROR_SERVER_NOT_RESPONDING = RASBASE + 50,
            ERROR_FROM_DEVICE = RASBASE + 51,
            ERROR_UNRECOGNIZED_RESPONSE = RASBASE + 52,
            ERROR_MACRO_NOT_FOUND = RASBASE + 53,
            ERROR_MACRO_NOT_DEFINED = RASBASE + 54,
            ERROR_MESSAGE_MACRO_NOT_FOUND = RASBASE + 55,
            ERROR_DEFAULTOFF_MACRO_NOT_FOUND = RASBASE + 56,
            ERROR_FILE_COULD_NOT_BE_OPENED = RASBASE + 57,
            ERROR_DEVICENAME_TOO_LONG = RASBASE + 58,
            ERROR_DEVICENAME_NOT_FOUND = RASBASE + 59,
            ERROR_NO_RESPONSES = RASBASE + 60,
            ERROR_NO_COMMAND_FOUND = RASBASE + 61,
            ERROR_WRONG_KEY_SPECIFIED = RASBASE + 62,
            ERROR_UNKNOWN_DEVICE_TYPE = RASBASE + 63,
            ERROR_ALLOCATING_MEMORY = RASBASE + 64,
            ERROR_PORT_NOT_CONFIGURED = RASBASE + 65,
            ERROR_DEVICE_NOT_READY = RASBASE + 66,
            ERROR_READING_INI_FILE = RASBASE + 67,
            ERROR_NO_CONNECTION = RASBASE + 68,
            ERROR_BAD_USAGE_IN_INI_FILE = RASBASE + 69,
            ERROR_READING_SECTIONNAME = RASBASE + 70,
            ERROR_READING_DEVICETYPE = RASBASE + 71,
            ERROR_READING_DEVICENAME = RASBASE + 72,
            ERROR_READING_USAGE = RASBASE + 73,
            ERROR_READING_MAXCONNECTBPS = RASBASE + 74,
            ERROR_READING_MAXCARRIERBPS = RASBASE + 75,
            ERROR_LINE_BUSY = RASBASE + 76,
            ERROR_VOICE_ANSWER = RASBASE + 77,
            ERROR_NO_ANSWER = RASBASE + 78,
            ERROR_NO_CARRIER = RASBASE + 79,
            ERROR_NO_DIALTONE = RASBASE + 80,
            ERROR_IN_COMMAND = RASBASE + 81,
            ERROR_WRITING_SECTIONNAME = RASBASE + 82,
            ERROR_WRITING_DEVICETYPE = RASBASE + 83,
            ERROR_WRITING_DEVICENAME = RASBASE + 84,
            ERROR_WRITING_MAXCONNECTBPS = RASBASE + 85,
            ERROR_WRITING_MAXCARRIERBPS = RASBASE + 86,
            ERROR_WRITING_USAGE = RASBASE + 87,
            ERROR_WRITING_DEFAULTOFF = RASBASE + 88,
            ERROR_READING_DEFAULTOFF = RASBASE + 89,
            ERROR_EMPTY_INI_FILE = RASBASE + 90,
            ERROR_AUTHENTICATION_FAILURE = RASBASE + 91,
            ERROR_PORT_OR_DEVICE = RASBASE + 92,
            ERROR_NOT_BINARY_MACRO = RASBASE + 93,
            ERROR_DCB_NOT_FOUND = RASBASE + 94,
            ERROR_STATE_MACHINES_NOT_STARTED = RASBASE + 95,
            ERROR_STATE_MACHINES_ALREADY_STARTED = RASBASE + 96,
            ERROR_PARTIAL_RESPONSE_LOOPING = RASBASE + 97,
            ERROR_UNKNOWN_RESPONSE_KEY = RASBASE + 98,
            ERROR_RECV_BUF_FULL = RASBASE + 99,
            ERROR_CMD_TOO_LONG = RASBASE + 100,
            ERROR_UNSUPPORTED_BPS = RASBASE + 101,
            ERROR_UNEXPECTED_RESPONSE = RASBASE + 102,
            ERROR_INTERACTIVE_MODE = RASBASE + 103,
            ERROR_BAD_CALLBACK_NUMBER = RASBASE + 104,
            ERROR_INVALID_AUTH_STATE = RASBASE + 105,
            ERROR_WRITING_INITBPS = RASBASE + 106,
            ERROR_X25_DIAGNOSTIC = RASBASE + 107,
            ERROR_ACCT_EXPIRED = RASBASE + 108,
            ERROR_CHANGING_PASSWORD = RASBASE + 109,
            ERROR_OVERRUN = RASBASE + 110,
            ERROR_RASMAN_CANNOT_INITIALIZE = RASBASE + 111,
            ERROR_BIPLEX_PORT_NOT_AVAILABLE = RASBASE + 112,
            ERROR_NO_ACTIVE_ISDN_LINES = RASBASE + 113,
            ERROR_NO_ISDN_CHANNELS_AVAILABLE = RASBASE + 114,
            ERROR_TOO_MANY_LINE_ERRORS = RASBASE + 115,
            ERROR_IP_CONFIGURATION = RASBASE + 116,
            ERROR_NO_IP_ADDRESSES = RASBASE + 117,
            ERROR_PPP_TIMEOUT = RASBASE + 118,
            ERROR_PPP_REMOTE_TERMINATED = RASBASE + 119,
            ERROR_PPP_NO_PROTOCOLS_CONFIGURED = RASBASE + 120,
            ERROR_PPP_NO_RESPONSE = RASBASE + 121,
            ERROR_PPP_INVALID_PACKET = RASBASE + 122,
            ERROR_PHONE_NUMBER_TOO_LONG = RASBASE + 123,
            ERROR_IPXCP_NO_DIALOUT_CONFIGURED = RASBASE + 124,
            ERROR_IPXCP_NO_DIALIN_CONFIGURED = RASBASE + 125,
            ERROR_IPXCP_DIALOUT_ALREADY_ACTIVE = RASBASE + 126,
            ERROR_ACCESSING_TCPCFGDLL = RASBASE + 127,
            ERROR_NO_IP_RAS_ADAPTER = RASBASE + 128,
            ERROR_SLIP_REQUIRES_IP = RASBASE + 129,
            ERROR_PROJECTION_NOT_COMPLETE = RASBASE + 130,
            ERROR_PROTOCOL_NOT_CONFIGURED = RASBASE + 131,
            ERROR_PPP_NOT_CONVERGING = RASBASE + 132,
            ERROR_PPP_CP_REJECTED = RASBASE + 133,
            ERROR_PPP_LCP_TERMINATED = RASBASE + 134,
            ERROR_PPP_REQUIRED_ADDRESS_REJECTED = RASBASE + 135,
            ERROR_PPP_NCP_TERMINATED = RASBASE + 136,
            ERROR_PPP_LOOPBACK_DETECTED = RASBASE + 137,
            ERROR_PPP_NO_ADDRESS_ASSIGNED = RASBASE + 138,
            ERROR_CANNOT_USE_LOGON_CREDENTIALS = RASBASE + 139,
            ERROR_TAPI_CONFIGURATION = RASBASE + 140,
            ERROR_NO_LOCAL_ENCRYPTION = RASBASE + 141,
            ERROR_NO_REMOTE_ENCRYPTION = RASBASE + 142,
            ERROR_REMOTE_REQUIRES_ENCRYPTION = RASBASE + 143,
            ERROR_IPXCP_NET_NUMBER_CONFLICT = RASBASE + 144,
            ERROR_INVALID_SMM = RASBASE + 145,
            ERROR_SMM_UNINITIALIZED = RASBASE + 146,
            ERROR_NO_MAC_FOR_PORT = RASBASE + 147,
            ERROR_SMM_TIMEOUT = RASBASE + 148,
            ERROR_BAD_PHONE_NUMBER = RASBASE + 149,
            ERROR_WRONG_MODULE = RASBASE + 150,
            ERROR_PPP_MAC = RASBASE + 151,
            ERROR_PPP_LCP = RASBASE + 152,
            ERROR_PPP_AUTH = RASBASE + 153,
            ERROR_PPP_NCP = RASBASE + 154,
            ERROR_POWER_OFF = RASBASE + 155,
            ERROR_POWER_OFF_CD = RASBASE + 156,


            ERROR_DIAL_ALREADY_IN_PROGRESS = RASBASE + 157,
            ERROR_RASAUTO_CANNOT_INITIALIZE = RASBASE + 158,
            ERROR_UNABLE_TO_AUTHENTICATE_SERVER = RASBASE + 178,


            RASBASEEND = RASBASE + 158
        }

        public enum RasConnState : uint
        {
            RASCS_OpenPort = 0,
            RASCS_PortOpened,
            RASCS_ConnectDevice,
            RASCS_DeviceConnected,
            RASCS_AllDevicesConnected,
            RASCS_Authenticate,
            RASCS_AuthNotify,
            RASCS_AuthRetry,
            RASCS_AuthCallback,
            RASCS_AuthChangePassword,
            RASCS_AuthProject,
            RASCS_AuthLinkSpeed,
            RASCS_AuthAck,
            RASCS_ReAuthenticate,
            RASCS_Authenticated,
            RASCS_PrepareForCallback,
            RASCS_WaitForModemReset,
            RASCS_WaitForCallback,
            RASCS_Projected,

            RASCS_Interactive = 4096,
            RASCS_RetryAuthentication,
            RASCS_CallbackSetByCaller,
            RASCS_PasswordExpired,

            RASCS_Connected = 8192,
            RASCS_Disconnected,

            RASCS_Unknown = 9999
        }

        [DllImport("Coredll.dll", EntryPoint = "RasDial", SetLastError = true, CharSet = CharSet.Unicode)]
        public static extern RASError RasDial(IntPtr dialExtensions,
                                         IntPtr phoneBookPath,
                                         ref RASDIALPARAMS rasDialParam,
                                         uint NotifierType,
                                         IntPtr notifier,
                                         ref IntPtr pRasConn);

        [DllImport("coredll.dll", EntryPoint = "RasHangUp", SetLastError = true, CharSet = CharSet.Unicode)]
        public static extern RASError RasHangUp(IntPtr pRasConn);

        [DllImport("coredll.dll", EntryPoint = "RasGetConnectStatus", SetLastError = true, CharSet = CharSet.Unicode)]
        public static extern RASError RasGetConnectStatus(IntPtr hrasconn, ref RASCONNSTATUS lprasconnstatus);

        [DllImport("coredll.dll", EntryPoint = "RasValidateEntryName", SetLastError = true, CharSet = CharSet.Unicode)]
        public static extern RASError RasValidateEntryName(string lpszPhonebook, string lpszEntry);

        [DllImport("coredll.dll", EntryPoint = "RasSetEntryProperties", SetLastError = true, CharSet = CharSet.Unicode)]
        public static extern RASError RasSetEntryProperties(string lpszPhoneBook, string lpszEntry, ref RASENTRY lpRasEntry,
                                                        int dwEntryInfoSize, byte[] lpbDeviceInfo, int dwDeviceInfoSize);

        [DllImport("coredll.dll", EntryPoint = "RasGetEntryProperties", SetLastError = true, CharSet = CharSet.Unicode)]
        public static extern RASError RasGetEntryProperties(string lpszPhoneBook, string lpszEntry, ref RASENTRY lpRasEntry,
                                                        ref int lpdwEntryInfoSize, byte[] lpbDeviceInfo, ref int lpdwDeviceInfoSize);

        public static void SetDialingParameters(string KeyName, string Location, string LocalCalls,
                                             string LongDistanceCalls, string InternationalCalls,
                                             string AreaCode, string DisableCallWaitingSequence,
                                             string CountryCode, string ToneOrPulse)
        {
            //C++ way
            //string ConfigurationString = Location + "\0" + LocalCalls + "\0" + LongDistanceCalls + "\0" +
            // InternationalCalls + "\0" + AreaCode + "\0" + DisableCallWaitingSequence + "\0" +
            // CountryCode + "\0" + ToneOrPulse + "\0";

            //C# The way to create REG_MULTI_SZ
            string[] ConfigurationString = new string[8];
            ConfigurationString[0] = Location;
            ConfigurationString[1] = LocalCalls;
            ConfigurationString[2] = LongDistanceCalls;
            ConfigurationString[3] = InternationalCalls;
            ConfigurationString[4] = AreaCode;
            ConfigurationString[5] = DisableCallWaitingSequence;
            ConfigurationString[6] = CountryCode;
            ConfigurationString[7] = ToneOrPulse;

            RegistryKey rk = Registry.CurrentUser;
            using (RegistryKey rk1 = rk.CreateSubKey("ControlPanel\\Dial\\Locations"))
            {
                rk1.SetValue(KeyName, ConfigurationString);
                rk1.Close();
            }
        }
    }
}
