﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using NPrinterApi;
using NSysSvcApi;
using _Native;

namespace Printer
{
    public partial class Form1 : Form
    {
        public SysSvcApi sysService = null;
        public PrinterApi prt = null;
        public PRT_RESULT prtRes = PRT_RESULT.PRT_RESULT_FAILURE;
        public const string title = "Printer Example";

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                pnlGeneral.Location = new Point(0, 0);
            }

            // create the system service object
            sysService = new SysSvcApi();
            HW_OPTIONS hwOptions = new HW_OPTIONS();
            // Check what type of printer
            sysService.GetHardwareOptions(ref hwOptions);
            if (hwOptions.Printer == HW_TYPE_PRINTER.NONE)
            {
                sysService.Dispose();
                sysService = null;
                MessageBox.Show("Printer is not found!");
                return;
            }

            // create the printer object
            prt = new PrinterApi();
            prt.SetCallback(new PrinterApi.PRINTERCALLBACK(PrinterCallbackProc));

            // Setting Text Quality ComboBox
            cmbQuality.Items.Add("NORMAL");
            cmbQuality.Items.Add("FINE");
            cmbQuality.SelectedIndex = 0;

            // Setting Text Alignment ComboBox
            cmbAlignment.Items.Add("LEFT");
            cmbAlignment.Items.Add("CENTER");
            cmbAlignment.Items.Add("RIGHT");

            cmbAlignment.SelectedIndex = 0;

            // Setting Text Size ComboBox
            cmbSize.Items.Add("LARGE");
            cmbSize.Items.Add("NORMAL");
            cmbSize.Items.Add("SMALL");

            cmbSize.SelectedIndex = 0;

            // Setting Text Weight ComboBox
            cmbWeight.Items.Add("NORMAL");
            cmbWeight.Items.Add("BOLD");

            cmbWeight.SelectedIndex = 0;

            // Setting Text Underline ComboBox
            cmbUnderline.Items.Add("TRUE");
            cmbUnderline.Items.Add("FALSE");

            cmbUnderline.SelectedIndex = 1;

            cmbBarcodeType.Items.Add("Code 39");
            cmbBarcodeType.Items.Add("Code 93");
            cmbBarcodeType.Items.Add("Code 128");
            cmbBarcodeType.Items.Add("EAN 8");
            cmbBarcodeType.Items.Add("EAN 13");
            cmbBarcodeType.Items.Add("Interleaved 2of5");
            cmbBarcodeType.Items.Add("Codabar");
            cmbBarcodeType.Items.Add("MSI");
            cmbBarcodeType.Items.Add("AZTEC");
            cmbBarcodeType.Items.Add("DATA MATRIX");
            cmbBarcodeType.Items.Add("PDF 417");
            cmbBarcodeType.Items.Add("QR CODE");
            cmbBarcodeType.SelectedIndex = 0;

            cmbImgSize.Items.Add("");
            cmbImgSize.Items.Add("LARGE");
            cmbImgSize.Items.Add("NORMAL");
            cmbImgSize.Items.Add("SMALL");
            cmbImgSize.SelectedIndex = 0;

            // Textbox 
            txtText.Text = "The test printing";

            ContolEnabled(false);
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the printer object
            if (prt != null)
            {
                prt.Close();
                prt = null;
            }

            // destruction of the system service object
            if (sysService != null)
            {
                sysService.Dispose();
                sysService = null;
            }
        }

        private void ContolEnabled(bool Enabled)
        {
            btnTestPrint.Enabled = Enabled;
            btnFeed.Enabled = Enabled;
            btnSpace.Enabled = Enabled;
            btnTextPrint.Enabled = Enabled;
            btnBitmap.Enabled = Enabled;
            btnDensity.Enabled = Enabled;
            btnBarcode.Enabled = Enabled;

            nudFeed.Enabled = Enabled;
            nudSpace.Enabled = Enabled;
            nudDensity.Enabled = Enabled;
            txtText.Enabled = Enabled;
            cmbQuality.Enabled = Enabled;
            cmbAlignment.Enabled = Enabled;
            cmbSize.Enabled = Enabled;
            cmbWeight.Enabled = Enabled;
            cmbUnderline.Enabled = Enabled;
            nWordWeight.Enabled = Enabled;
            cmbBarcodeType.Enabled = Enabled;
            cmbImgSize.Enabled = Enabled;
            chkBarcodeLable.Enabled = Enabled;
        }

        /// <summary>
        /// When printer state is changed, this function is executed.
        /// </summary>
        /// <param name="Status"></param>
        public void PrinterCallbackProc(NPrinterApi.PRT_STATUS Status)
        {
            string strMsg = String.Empty;

            if (!Status.PaperDetect || !Status.PlatenDetect)
            {
                strMsg = String.Format("PaperDetect={0}, PlatenDetect={1}", Status.PaperDetect.ToString(), Status.PlatenDetect.ToString());
                MessageBox.Show(strMsg);
            }
        }

        private bool IsKorean()
        {
            Int16 LANG_KOREAN = 0x0412;
            bool bRet = false;

            Int16 LangID = Utils.GetUserDefaultLangID();
            if (LangID == LANG_KOREAN)
                bRet = true;
            else
                bRet = false;

            return bRet;
        }

        private void PrintKoreanReceipt()
        {
            prt.PrintText("황금동태 ", PrinterApi.PRT_ALIGN_CENTER, 12, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, true); prt.FeedPaper(5);
            prt.PrintText("사업자번호:xxx-xx-xxxxx 대표:문주영", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("서울시 동대문구 답십리동 xx-xx", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("TEL: 02-1111-2222", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("*신한카드", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("카드번호 : 12345667789xxx", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("창구매입  유효기간:2015년8월", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); prt.FeedPaper(5);
            prt.PrintText("거래일자  2010년11월16일 01:41:01", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("가맹번호  12345678", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("거래구분  [일시불]", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false);
            prt.PrintText("거래금액  22,728원", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("부가세액  2,272원", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("합    계  25,000원", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); prt.FeedPaper(5);
            prt.PrintText("매입사    신한카드", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("승인일자  2010년11월16일 01:40:36", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("승인번호  12345678", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);
            prt.PrintText("단말기No  987654321", PrinterApi.PRT_ALIGN_LEFT, 8, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false);

        }

        private void PrintEnglishReceipt()
        {

            if (Utils.IsWinCE)
            {
                const int SizeOfContents = 8;
                const int SizeOfLine = 10;
                string dtNow = String.Format("{0}/{1}/{2} {3}:{4}         ", DateTime.Now.Month.ToString().PadLeft(2, '0'),
                                                                             DateTime.Now.Day.ToString().PadLeft(2, '0'),
                                                                             DateTime.Now.Year.ToString(),
                                                                             DateTime.Now.Hour.ToString().PadLeft(2, '0'),
                                                                             DateTime.Now.Minute.ToString().PadLeft(2, '0'));

                prt.PrintText("ATID", PrinterApi.PRT_ALIGN_CENTER, 12, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); prt.FeedPaper(1);
                prt.PrintText("Test Printing", PrinterApi.PRT_ALIGN_CENTER, SizeOfContents, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); prt.FeedPaper(2);
                prt.PrintText("----------------------------", PrinterApi.PRT_ALIGN_CENTER, SizeOfLine, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("STORE:0681", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("CASHIER:Veronica T", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("----------------------------", PrinterApi.PRT_ALIGN_CENTER, SizeOfLine, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("Skull print boyshort           4.00", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("   90391594                ", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("   _PROMO ITEM             ", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); prt.FeedPaper(4);
                prt.PrintText("============================", PrinterApi.PRT_ALIGN_CENTER, SizeOfLine, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("SUBTOTAL                       4.00", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("SALES TAX                       .33", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("TOTAL                          4.33", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("============================", PrinterApi.PRT_ALIGN_CENTER, SizeOfLine, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("AMOUNT TENDERED            ", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("CASH                           4.35", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); prt.FeedPaper(1);
                prt.PrintText("TOTAL PAYMENT                  4.35", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("CHANGE                          .02", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("----------------------------", PrinterApi.PRT_ALIGN_CENTER, SizeOfLine, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("SALES TAX ANALYSIS       ", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("CODE  RATE%             TAXABLE TAX", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("LOCAL 8.25000           4.0     .33", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("----------------------------", PrinterApi.PRT_ALIGN_CENTER, SizeOfLine, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("Transaction:101424       ", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText(dtNow, PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("----------------------------", PrinterApi.PRT_ALIGN_CENTER, SizeOfLine, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); 
                prt.PrintText("Visit us at www.atid1.com", PrinterApi.PRT_ALIGN_LEFT, SizeOfContents, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); prt.FeedPaper(5);
            }
            else
            {
                string dtNow = String.Format("{0}/{1}/{2} {3}:{4}         ", DateTime.Now.Month.ToString().PadLeft(2, '0'),
                                                             DateTime.Now.Day.ToString().PadLeft(2, '0'),
                                                             DateTime.Now.Year.ToString(),
                                                             DateTime.Now.Hour.ToString().PadLeft(2, '0'),
                                                             DateTime.Now.Minute.ToString().PadLeft(2, '0'));

                prt.PrintText("ATID", PrinterApi.PRT_ALIGN_CENTER, 12, PrinterApi.PRT_WEIGHT_NORMAL, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); prt.FeedPaper(1);

                prt.PrintText("Test Printing", PrinterApi.PRT_ALIGN_CENTER, 7, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_NORMAL, 0, 0, false); prt.FeedPaper(2);
                prt.PrintText("----------------", PrinterApi.PRT_ALIGN_CENTER, 7, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("STORE:0681", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("CASHIER:Veronica T", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("----------------", PrinterApi.PRT_ALIGN_CENTER, 7, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("Skull print boyshort 4.00", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("   90391594                ", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("   _PROMO ITEM             ", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); prt.FeedPaper(4);
                prt.PrintText("================", PrinterApi.PRT_ALIGN_CENTER, 7, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("SUBTOTAL             4.00", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("SALES TAX             .33", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("TOTAL                4.33", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("================", PrinterApi.PRT_ALIGN_CENTER, 7, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("AMOUNT TENDERED            ", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("CASH                 4.35", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); prt.FeedPaper(1);
                prt.PrintText("TOTAL PAYMENT        4.35", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("CHANGE                .02", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("----------------", PrinterApi.PRT_ALIGN_CENTER, 7, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("SALES TAX ANALYSIS       ", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("CODE  RATE%   TAXABLE TAX", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("LOCAL 8.25000 4.0     .33", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("----------------", PrinterApi.PRT_ALIGN_CENTER, 7, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("Transaction:101424       ", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText(dtNow, PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("----------------", PrinterApi.PRT_ALIGN_CENTER, 7, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); 
                prt.PrintText("Visit us at www.atid1.com", PrinterApi.PRT_ALIGN_LEFT, 4, PrinterApi.PRT_WEIGHT_BOLD, PrinterApi.PRT_HEIGHT_DOUBLE, 0, 0, false); prt.FeedPaper(5);

            }

        }

        private void btnOpenClose_Click(object sender, EventArgs e)
        {
            if (prt == null)
                return;

            if (btnOpenClose.Text.CompareTo("Open") == 0)
            {
                // Printer devices are open and required system resource allocation.
                if ((prtRes = prt.Open()) != PRT_RESULT.PRT_RESULT_SUCCESS)
                {
                    MessageBox.Show(prtRes.ToString());
                    return;
                }

                ContolEnabled(true);
                btnOpenClose.Text = "Close";
            }
            else
            {
                // Release system resources allocated to the device close to the Printer.
                if ((prtRes = prt.Close()) != PRT_RESULT.PRT_RESULT_SUCCESS)
                {
                    MessageBox.Show(prtRes.ToString());
                    return;
                }

                ContolEnabled(false);
                btnOpenClose.Text = "Open";
            }
        }

        private void btnTestPrint_Click(object sender, EventArgs e)
        {
            if (prt == null)
                return;

            if ((prtRes = prt.SetDensity((uint)nudDensity.Value)) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }
            
            switch (cmbQuality.SelectedIndex)
            {
                case 0:
                    prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_NORMAL);
                    break;
                case 1:
                    prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_FINE);
                    break;
            }

            Cursor.Current = Cursors.WaitCursor;
            Application.DoEvents();
            // Start the print job to the printer device announces.
            if ((prtRes = prt.Start()) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                Cursor.Current = Cursors.Default;
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }

            if (IsKorean())
                PrintKoreanReceipt();
            else
                PrintEnglishReceipt();

            // The length of the paper to Feed. Avalue of 1 is equivalent to 1mm.
            prt.FeedPaper(20);
            
            // Announces the output to the printer quit working.
            if ((prtRes = prt.Stop()) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }
            Cursor.Current = Cursors.Default;
        }

        private void btnFeed_Click(object sender, EventArgs e)
        {
            if (prt == null)
                return;

            if (!prt.IsOpen())
                return;

            uint nFeedCount = Convert.ToUInt16(nudFeed.Value);

            prt.Start();

            if ((prtRes = prt.FeedPaper(nFeedCount)) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                prt.Stop();
                return;
            }

            prt.Stop();
        }

        private void btnSpace_Click(object sender, EventArgs e)
        {
            if (prt == null)
                return;

            if (!prt.IsOpen())
                return;

            // Set Line spacing value.
            prt.SetLineSpace((uint)nudSpace.Value);
        }

        private void btnTextPrint_Click(object sender, EventArgs e)
        {
            if (prt == null)
                return;

            string strStatus = String.Empty;

            if (!prt.IsOpen())
                return;

            PRT_STATUS PrtStatus = new PRT_STATUS();

            // Reads the status of the printer device.
            if ((prtRes = prt.GetStatus(ref PrtStatus)) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }

            //==========================================================
            // PaperDetect  = if the printer lid is closed, TRUE, otherwise FALSE. 
            // PlatenDetect = For print media, if TRUE, otherwise FALSE.
            if (!PrtStatus.PaperDetect || !PrtStatus.PlatenDetect)
            {
                strStatus = String.Format("PaperDetect={0}, PlatenDetect={1}", PrtStatus.PaperDetect.ToString(), PrtStatus.PlatenDetect.ToString());
                MessageBox.Show(strStatus);
                return;
            }

            if ((prtRes = prt.SetLineSpace((uint)nudSpace.Value)) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }

            if ((prtRes = prt.SetDensity(150)) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }
            
            switch (cmbQuality.SelectedIndex)
            {
                case 0:
                    prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_NORMAL);
                    break;
                case 1:
                    prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_FINE);
                    break;
            }
            System.Diagnostics.Debug.WriteLine("[PRINT_APP] - call SetQuality");

            int ALIGN = 0;              // Sort of printed characters.
            int WEIGHT = 0;             // Value of the bold print text.
            int SIZE = 0;               // The size of the output value of the character.
            int WORDSPACE = 0;          // Spacing between letters.
            int HEIGHT = 0;             // Output of the high value of the character.
            bool UNDERLINE = false;     // The visibility of UnderLine.
            int WORDWEIGHT = 0;

            switch (cmbAlignment.SelectedIndex)
            {
                case 0: // Left
                    ALIGN = PrinterApi.PRT_ALIGN_LEFT;
                    break;
                case 1: // Center
                    ALIGN = PrinterApi.PRT_ALIGN_CENTER;
                    break;
                case 2: // Right
                    ALIGN = PrinterApi.PRT_ALIGN_RIGHT;
                    break;
            }

            switch (cmbWeight.SelectedIndex)
            {
                case 0: // Normal
                    WEIGHT = PrinterApi.PRT_WEIGHT_NORMAL;
                    break;
                case 1: // Blod
                    WEIGHT = PrinterApi.PRT_WEIGHT_BOLD;
                    break;

            }

            switch (cmbSize.SelectedIndex)
            {
                case 0: // Left
                    SIZE = PrinterApi.PRT_SIZE_LARGE;
                    break;
                case 1: // Center
                    SIZE = PrinterApi.PRT_SIZE_NORMAL;
                    break;
                case 2: // Right
                    SIZE = PrinterApi.PRT_SIZE_SMALL;
                    break;
            }

            switch (cmbUnderline.SelectedIndex)
            {
                case 0:
                    UNDERLINE = true;
                    break;
                case 1:
                    UNDERLINE = false;
                    break;
            }

            if ((prtRes = prt.Start()) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }

            WORDSPACE = (int)nudSpace.Value;
            WORDWEIGHT = (int)nWordWeight.Value;

            if ((prtRes = prt.PrintText(txtText.Text.Trim(), ALIGN, SIZE, WEIGHT, HEIGHT, WORDSPACE, WORDWEIGHT, UNDERLINE)) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }

            if ((prtRes = prt.Stop()) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }
        }

        private void btnBitmap_Click(object sender, EventArgs e)
        {
            if (prt == null)
                return;

            PRT_RESULT result = new PRT_RESULT();
            PRT_STATUS PrtStatus = new PRT_STATUS();
            string strStatus = String.Empty;
            string strFileName = String.Empty;

            if (!prt.IsOpen())
                return;

            if ((prtRes = prt.GetStatus(ref PrtStatus)) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }

            if (!PrtStatus.PaperDetect || !PrtStatus.PlatenDetect)
            {
                strStatus = String.Format("PaperDetect={0}, PlatenDetect={1}", PrtStatus.PaperDetect.ToString(), PrtStatus.PlatenDetect.ToString());
                MessageBox.Show(strStatus);
                return;
            }

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                // Output file name includeing the path to a bitmap image.
                strFileName = openFileDialog1.FileName;

                switch (cmbQuality.SelectedIndex)
                {
                    case 0:
                        prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_LOW);
                        break;
                    case 1:
                        prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_NORMAL);
                        break;
                    case 2:
                        prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_FINE);
                        break;
                }

                prt.Start();
                //=======================================================
                // Bitmap image output.
                // Alignment = Bitmap alignment.
                // sFileName = Output file name includeing the path to a bitmap image.
                result = prt.PrintBitmap(NPrinterApi.PrinterApi.PRT_ALIGN_CENTER, strFileName);

                prt.Stop();

                if (result != PRT_RESULT.PRT_RESULT_SUCCESS)
                    MessageBox.Show(result.ToString());
            }
        }

        private void btnDensity_Click(object sender, EventArgs e)
        {
            if (prt == null)
                return;

            if (!prt.IsOpen())
                return;

            // Set the value of output levels.
            prt.SetDensity((uint)nudDensity.Value);
        }

        private void btnBarcode_Click(object sender, EventArgs e)
        {
            if (prt == null)
                return;

            PRT_RESULT result;
            PRT_STATUS PrtStatus = new PRT_STATUS();
            string strStatus = String.Empty;
            string barcodeData = "1234567890";

            if (!prt.IsOpen())
                return;

            if ((prtRes = prt.GetStatus(ref PrtStatus)) != PRT_RESULT.PRT_RESULT_SUCCESS)
            {
                MessageBox.Show("GetStatus " + prtRes.ToString());
                return;
            }

            if (!PrtStatus.PaperDetect || !PrtStatus.PlatenDetect)
            {
                strStatus = String.Format("PaperDetect={0}, PlatenDetect={1}", PrtStatus.PaperDetect.ToString(), PrtStatus.PlatenDetect.ToString());
                MessageBox.Show(strStatus);
                return;
            }

            switch (cmbQuality.SelectedIndex)
            {
                case 0:
                    prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_LOW);
                    break;
                case 1:
                    prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_NORMAL);
                    break;
                case 2:
                    prt.SetQuality(NPrinterApi.PrinterApi.PRT_QUALITY_FINE);
                    break;
            }

            prt.Start();

            //=================================================================
            // Barcode image output.
            // Type             = Barcode image output.
            // sBarcodeValue    = Output of the barcode value.
            // Alignment        = Alignment of the barcode.
            // blncludelabel    = Whether to include labels.

            uint nAlign = 0;

            switch (cmbAlignment.SelectedIndex)
            {
                case 0:
                    nAlign = NPrinterApi.PrinterApi.PRT_ALIGN_LEFT;
                    break;
                case 1:
                    nAlign = NPrinterApi.PrinterApi.PRT_ALIGN_CENTER;
                    break;
                case 2:
                    nAlign = NPrinterApi.PrinterApi.PRT_ALIGN_RIGHT;
                    break;
            }

            BARCODE_TYPE eBarcodeType = BARCODE_TYPE.BCODE_TYPE_CODE39;
            switch (cmbBarcodeType.SelectedIndex)
            {
                case 0:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_CODE39;
                    break;
                case 1:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_CODE93;
                    break;
                case 2:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_CODE128;
                    break;
                case 3:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_EAN8;
                    barcodeData = "1234567";
                    break;
                case 4:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_EAN13;
                    barcodeData = "123456790123";
                    break;
                case 5:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_INTERLEAVED2OF5;
                    break;
                case 6:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_CODABAR;
                    break;
                case 7:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_MSI_MOD10;
                    break;
                case 8:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_AZTEC;
                    break;
                case 9:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_DATA_MATRIX;
                    break;
                case 10:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_PDF_417;
                    break;
                case 11:
                    eBarcodeType = BARCODE_TYPE.BCODE_TYPE_QR_CODE;
                    break;
            }

            switch (cmbImgSize.SelectedIndex)
            {
                case 0:
                default:
                    result = prt.PrintBarcode(eBarcodeType, barcodeData, nAlign, chkBarcodeLable.Checked);
                    break;
                case 1:
                    result = prt.PrintBarcodeEx(eBarcodeType, barcodeData, nAlign, chkBarcodeLable.Checked, NPrinterApi.PrinterApi.PRT_IMG_SIZE_LARGE);
                    break;
                case 2:
                    result = prt.PrintBarcodeEx(eBarcodeType, barcodeData, nAlign, chkBarcodeLable.Checked, NPrinterApi.PrinterApi.PRT_IMG_SIZE_NORMAL);
                    break;
                case 3:
                    result = prt.PrintBarcodeEx(eBarcodeType, barcodeData, nAlign, chkBarcodeLable.Checked, NPrinterApi.PrinterApi.PRT_IMG_SIZE_SMALL);
                    break;
            }

            prt.FeedPaper(10);
            prt.Stop();

            if (result != PRT_RESULT.PRT_RESULT_SUCCESS)
                MessageBox.Show(result.ToString());
        }

       
    }
}