﻿using System;

using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace _Native
{
    public enum PlaySoundFlags : int
    {
        SND_SYNC = 0x0,     // play synchronously (default)
        SND_ASYNC = 0x1,    // play asynchronously
        SND_NODEFAULT = 0x2,    // silence (!default) if sound not found
        SND_MEMORY = 0x4,       // pszSound points to a memory file
        SND_LOOP = 0x8,     // loop the sound until next sndPlaySound
        SND_NOSTOP = 0x10,      // don't stop any currently playing sound
        SND_NOWAIT = 0x2000,    // don't wait if the driver is busy
        SND_ALIAS = 0x10000,    // name is a registry alias
        SND_ALIAS_ID = 0x110000,// alias is a predefined ID
        SND_FILENAME = 0x20000, // name is file name
        SND_RESOURCE = 0x40004, // name is resource name or atom
    }

    public class BarcodeEmulatorInfo
    {
        public static string FileName
        {
            get
            {

                return Utils.IsWinCE ? @"\Windows\DEMO_ScanEmulator.exe"
                                     : @"\Program Files\ATID_DemoApps\BarcodeEmulatorWM.exe";
            }
        }
        public static string WindowClass
        {
            get
            {

                return Utils.IsWinCE ? "ScanEmul"
                                     : "__ATID__BARCODE__EMULATOR__";
            }
        }
        public static string WindowTitle
        {
            get
            {

                return Utils.IsWinCE ? "ScanEmul"
                                     : "BarcodeEmulator";
            }
        }
        public static uint CloseID
        {
            get
            {
                return Utils.IsWinCE ? (uint)40011
                                     : (uint)40001;
            }
        }
    }

    public class Utils
    {
        public const uint WM_DESTROY = 0x0002;
        public const uint WM_CLOSE = 0x0010;
        public const uint WM_QUIT = 0x0012;
        public const uint WM_COMMAND = 0x0111;
        public const string NETCF_WND_CLASS_NAME = "#NETCF_AGL_BASE_";

        [DllImport("coredll.dll", EntryPoint = "SendMessage", SetLastError = true)]
        public static extern IntPtr SendMessage(IntPtr hWnd, uint Msg, uint wParam, uint lParam);

        [DllImport("coredll.dll", EntryPoint = "FindWindowW", SetLastError = true)]
        public static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        [DllImport("coredll.dll", EntryPoint = "SetForegroundWindow", SetLastError = true)]
        public static extern bool SetForegroundWindow(IntPtr hWnd);

        [DllImport("coredll.dll", EntryPoint = "ShowWindow", SetLastError = true)]
        public static extern bool ShowWindow(IntPtr hWnd, int nCmdShow);

        [DllImport("coredll.dll", EntryPoint = "PlaySound", SetLastError = true)]
        public static extern int PlaySound(string szSound, IntPtr hModule, int flags);

        [DllImport("Coredll.dll", EntryPoint = "GetUserDefaultLangID")]
        public static extern Int16 GetUserDefaultLangID();

        public static bool PlaySound(string szSound)
        {
            int nRtn = PlaySound(szSound, IntPtr.Zero, (int)(PlaySoundFlags.SND_FILENAME | PlaySoundFlags.SND_ASYNC));
            return nRtn > 0 ? true : false;
        }

        public static bool IsWinCE
        {
            get
            {
                return Environment.OSVersion.Version.Minor == 2 ? false : true;
            }
        }
    }
}
