﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Threading;
using NRfidApi;
using NSysSvcApi;
using _Native;

namespace RFID
{
    public partial class Form1 : Form
    {
        public const string title = "RFID Example";
        public SysSvcApi sysService = null;
        public static RfidApi rfid = null;
        public RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;
        public static Queue qTags = null;
        public static Hashtable htTags = null;
        public AutoResetEvent eventStartDequeue = null;
        public Thread threadDequeue = null;
        public static bool exitThread = false;
        public static bool isRunning = false;
        uint m_nMaxPower = 0;
        /// <summary>
        /// Operates the module for 3 seconds.
        /// </summary>
        public const int ACCESS_TIME_SECONDS = 3;  
        /// <summary>
        /// Wait until the operating is successful.
        /// If you want to operate synchronously, you should not use this value.
        /// </summary>
        public const int ACCESS_TIME_UNLIMITED = 0;
        public delegate void delegateSetButtons(bool enable);
        public delegate void delegateAddListViewItem(ListViewItem item);
        public delegate void delegateModifyListViewItem(int index, int count);

        public struct TagInfo
        {
            public int index;
            public int count;
        }

        public Form1()
        {
            InitializeComponent();
         
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }
            
            // create the system service object
            sysService = new SysSvcApi();
            HW_OPTIONS hwOptions = new HW_OPTIONS();
            // Check what type of RFID(UHF)
            sysService.GetHardwareOptions(ref hwOptions);
            if (hwOptions.UHF == HW_TYPE_UHF.NONE)
            {
                sysService.Dispose();
                sysService = null;
                MessageBox.Show("RFID is not found!");
                return;
            }
            
            // create the rfid object
            rfid = new RfidApi();
            result = rfid.PowerOn();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                MessageBox.Show("PowerOn : " + result.ToString());
                return;
            }
            result = rfid.Open();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                rfid.PowerOff();
                MessageBox.Show("Open : " + result.ToString());
                return;
            }

            // choose a session for the inventory round
            cbxSession.Items.Add("S0");
            cbxSession.Items.Add("S1");
            cbxSession.Items.Add("S2");
            cbxSession.Items.Add("S3");
            switch (rfid.Session)
            {
                case NRfidApi.SESSION_TYPE.SESSION_0:
                    cbxSession.SelectedIndex = 0;
                    break;
                case SESSION_TYPE.SESSION_1:
                    cbxSession.SelectedIndex = 1;
                    break;
                case SESSION_TYPE.SESSION_2:
                    cbxSession.SelectedIndex = 2;
                    break;
                case SESSION_TYPE.SESSION_3:
                    cbxSession.SelectedIndex = 3;
                    break;
                default:
                    cbxSession.Enabled = false;
                    break;
            }

            // selects whether Tags whose inventoried flag is A, B, AorB
            cbxTarget.Items.Add("A");
            cbxTarget.Items.Add("B");
            cbxTarget.Items.Add("AB");
            switch (rfid.InventoryTarget)
            {
                case INVENTORIED_STATE.STATE_A:
                    cbxTarget.SelectedIndex = 0;
                    break;
                case INVENTORIED_STATE.STATE_B:
                    cbxTarget.SelectedIndex = 1;
                    break;
                case INVENTORIED_STATE.STATE_AB:
                    cbxTarget.SelectedIndex = 2;
                    break;
                default:
                    cbxTarget.Enabled = false;
                    break;
            }

            // sets the number of slots in the round
            nudQValue.Value = rfid.QValue;

            for (int i = 0; i <= 24; i++)
            {
                cbxPower.Items.Add(i+"");
            }
            uint nMaxPower = 0;
            uint nPower = 0;

            // sets the power level
            nMaxPower = rfid.MaxPwr;
            System.Diagnostics.Debug.WriteLine("nPower:" + nMaxPower.ToString());
            nPower = rfid.PowerLevel;

            if (nMaxPower == 0)
                nMaxPower = 300;

            switch (nMaxPower.ToString().Remove(2, 1))
            {
                case "30":
                    for (int i = 25; i <= 30; i++)
                    {
                        cbxPower.Items.Add(i + "");
                    }
                    m_nMaxPower = 30;
                    break;
                case "28":
                    for (int i = 25; i <= 28; i++)
                    {
                        cbxPower.Items.Add(i + "");
                    }
                    m_nMaxPower = 28;
                    break;
                case "27":
                    for (int i = 25; i <= 27; i++)
                    {
                        cbxPower.Items.Add(i + "");
                    }
                    m_nMaxPower = 27;
                    break;
                case "24":
                    m_nMaxPower = 24;
                    break;
                default:
                    m_nMaxPower = 30;
                    for (int i = 25; i <= 30; i++)
                    {
                        cbxPower.Items.Add(i + "");
                    }
                    break;
            }
            cbxPower.Text =(m_nMaxPower- rfid.PowerLevel).ToString();
            

            // In order to receive the results of the executed asynchronously, the callback function must be registered.
            // If you're using only the functions synchronously, callback does not need to be registered.
            rfid.SetCallback(new RfidCallbackProc(rfidCallbackProc));

            // A buffer to store the information of the tag to be displayed on the screen
            htTags = new Hashtable();

            // A buffer to store the tag data received from the module,
            // If you have a task that requires a lot of time to process the tag data, 
            // you save the tag data to a separate buffer and you need a tag data processing in a separate thread.
            qTags = new Queue();

            eventStartDequeue = new AutoResetEvent(false);
            exitThread = false;
            threadDequeue = new Thread(new ThreadStart(threadDequeueProc));
            threadDequeue.IsBackground = true;
            threadDequeue.Start();
        }
       
        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the system serivce object
            if (sysService != null)
            {
                sysService.Dispose();
                sysService = null;
            }

            // destruction of the rfid object
            if (rfid != null)
            {
                if (rfid.IsRunning())
                    rfid.Stop();

                rfid.Close();
                rfid.PowerOff();
                rfid = null;
            }

            if (threadDequeue != null)
            {
                exitThread = true;
                eventStartDequeue.Set();
                threadDequeue.Join();
                threadDequeue = null;
            }

            if (eventStartDequeue != null)
            {
                eventStartDequeue.Close();
                eventStartDequeue = null;
            }
        }

        public void SetButtons(bool enable)
        {
            btnClear.Enabled = enable;
            btnRead.Enabled = enable;
            btnWrite.Enabled = enable;
            btnLock.Enabled = enable;
            btnKill.Enabled = enable;
            cbxSession.Enabled = enable;
            cbxTarget.Enabled = enable;
            cbxPower.Enabled = enable;
            nudQValue.Enabled = enable;
            btnInventory.Text = enable ? "Start inventory" : "Stop inventory";
        }
        public void AddListViewItem(ListViewItem lvItem)
        {
            listView1.Items.Add(lvItem);
        }
        public void ModifyListViewItem(int index, int count)
        {
            listView1.Items[index].SubItems[1].Text = count.ToString();
        }

        /// <summary>
        /// Callback function for receiving a result of the asynchronous operation
        /// </summary>
        /// <param name="cbData"></param>
        public void rfidCallbackProc(RFIDCALLBACKDATA cbData)
        {
            // A buffer to store the data
            string buff = new string(new char[1024]);
            if (rfid == null)
                return;

            // In order to read the results you should call GetResult function.
            rfid.GetResult(buff, cbData.CallbackType, cbData.wParam, cbData.lParam);

            // When the memory value is received from the RFID module
            if (cbData.CallbackType == RFID_CALLBACK_TYPE.RFIDCALLBACKTYPE_DATA) 
            {
                // Save the readings on the queue.
                qTags.Enqueue(buff);
            }
            else // When the response is received from the RFID module
            {
                // inventory has been discontinued.
                if (buff.StartsWith("Multi Read Stop"))
                    isRunning = false;

                System.Diagnostics.Debug.WriteLine("Reply : " + buff);
            }
        }

        public void threadDequeueProc()
        {
            System.Diagnostics.Debug.WriteLine("+++ threadDequeueProc ");
            string temp = string.Empty;
            string data = string.Empty;
            TagInfo tagInfo;

            while (!exitThread)
            {
                eventStartDequeue.WaitOne();
                while (qTags.Count > 0 || isRunning)
                {
                    if (qTags.Count < 1)
                    {
                        Thread.Sleep(1);
                        continue;
                    }

                    temp = (string)qTags.Dequeue();
                    if (!string.IsNullOrEmpty(temp))
                    {
                        // Cut only the data from buffer
                        data = temp.Substring(0, temp.IndexOf(char.MinValue));

                        if (htTags.Contains(data))
                        {
                            tagInfo = (TagInfo)htTags[data];
                            tagInfo.count++;
                            htTags[data] = tagInfo;
                            BeginInvoke(new delegateModifyListViewItem(ModifyListViewItem), tagInfo.index, tagInfo.count);
                        }
                        else
                        {
                            tagInfo = new TagInfo();
                            tagInfo.index = htTags.Count;
                            tagInfo.count = 1;
                            htTags.Add(data, tagInfo);
                            ListViewItem item = new ListViewItem(data);
                            item.SubItems.Add(tagInfo.count.ToString());
                            BeginInvoke(new delegateAddListViewItem(AddListViewItem), item);
                        }
                    }
                    else
                    {
                        Thread.Sleep(1);
                    }
                }
                BeginInvoke(new delegateSetButtons(SetButtons), true);
            }

            System.Diagnostics.Debug.WriteLine("--- threadDequeueProc ");
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            htTags.Clear();
            qTags.Clear();
            listView1.Items.Clear();
        }

        private void btnInventory_Click(object sender, EventArgs e)
        {
            if (rfid ==null)
                return;

            if (btnInventory.Text.Equals("Start inventory"))
            {
                // Starts inventory(Read multiple tags)
                result = rfid.ReadEpc(false, RFID_READ_TYPE.EPC_GEN2_MULTI_TAG, null);
                if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    MessageBox.Show("ReadEpc : " + result.ToString());
                    return;
                }
                isRunning = true;
                SetButtons(false);
                eventStartDequeue.Set();
            }
            else
            {
                // Stop inventory
                result = rfid.Stop();
                if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    MessageBox.Show("Stop : " + result.ToString());
                    return;
                }
            }
        }

        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView1.SelectedIndices.Count < 1)
                return;

            int idx = listView1.SelectedIndices[0];
            string epc = listView1.Items[idx].Text;
            System.Diagnostics.Debug.WriteLine(epc);
            
        }

        public string GetSelectedTag()
        {
            if (listView1.SelectedIndices.Count < 1)
                return string.Empty;

            int idx = listView1.SelectedIndices[0];
            return listView1.Items[idx].Text;
        }

        private void btnRead_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            new frmRead(GetSelectedTag()).ShowDialog();
        }

        private void btnWrite_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            new frmWrite(GetSelectedTag()).ShowDialog();
        }

        private void btnLock_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            new frmLock(GetSelectedTag()).ShowDialog();
        }

        private void btnKill_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            new frmKill(GetSelectedTag()).ShowDialog();
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cbxSession.SelectedIndex)
            {
                case 0:
                    rfid.Session = SESSION_TYPE.SESSION_0;
                    break;
                case 1:
                    rfid.Session = SESSION_TYPE.SESSION_1;
                    break;
                case 2:
                    rfid.Session = SESSION_TYPE.SESSION_2;
                    break;
                case 3:
                    rfid.Session = SESSION_TYPE.SESSION_3;
                    break;
            }
        }

        private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cbxTarget.SelectedIndex)
            {
                case 0:
                    rfid.InventoryTarget = INVENTORIED_STATE.STATE_A;
                    break;
                case 1:
                    rfid.InventoryTarget = INVENTORIED_STATE.STATE_B;
                    break;
                case 2:
                    rfid.InventoryTarget = INVENTORIED_STATE.STATE_AB;
                    break;
            }
        }

        private void comboBox3_SelectedIndexChanged(object sender, EventArgs e)
        {
            uint nPower = Convert.ToUInt32(cbxPower.Text);
            System.Diagnostics.Debug.WriteLine("Value : " + nPower.ToString());
            rfid.PowerLevel =m_nMaxPower- nPower;
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            rfid.QValue = (uint)nudQValue.Value;
        }
    }
}