﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NRfidApi;
using _Native;

namespace RFID
{
    public partial class frmKill : Form
    {
        public string taretTag = string.Empty;
        RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;

        public frmKill(string _epc)
        {
            InitializeComponent();
            this.Text = "Kill Example";
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }
            this.taretTag = _epc;

            // The maximum length of the mask is 32 bytes.
            int maxTargetLength = RfidApi.MAX_MASK_BYTES * 2;
            if (this.taretTag.Length > maxTargetLength)
                this.taretTag = this.taretTag.Substring(0, maxTargetLength);

            tbxTargetTag.Text = taretTag;

            // If you want to operate synchronously, the operation time must be set to a value other than zero.
            // If set to 0, you may not be able to stop the operation.
            Form1.rfid.OperationTime = Form1.ACCESS_TIME_SECONDS;
        }

        private void frmKill_Closing(object sender, CancelEventArgs e)
        {
            // Set the operating time is unlimited.
            Form1.rfid.OperationTime = Form1.ACCESS_TIME_UNLIMITED;
        }

        public void AddString(string str)
        {
            listBox1.Items.Add(str);
            listBox1.SelectedIndex = listBox1.Items.Count - 1;
        }

        private void btnKill_Click(object sender, EventArgs e)
        {
            string killPassword = null;
            RFIDMASKPARAMS maskParam = new RFIDMASKPARAMS();

            if (!string.IsNullOrEmpty(tbxKillPassword.Text))
            {
                // kill password is a 4-byte data.
                // The 4-byte data that has already been written on the tag.
                if (tbxKillPassword.Text.Length != 8)
                {
                    MessageBox.Show("Kill password is 4 bytes");
                    tbxKillPassword.Focus();
                    tbxKillPassword.SelectAll();
                    return;
                }
            }
            killPassword = tbxKillPassword.Text.Trim();

            if (!string.IsNullOrEmpty(tbxTargetTag.Text))
            {
                // To kill only tags that you selected on the previous screen
                maskParam.MemBank = MEM_BANK.EPC;
                maskParam.nOffSet = 4; // nibble offset
                maskParam.MaskPattern = tbxTargetTag.Text.Trim();
            }

            btnKill.Enabled = false;
            Cursor.Current = Cursors.WaitCursor;

            // Warning : After completig kill operation, Tag can not be recovered.
            // The kill operation will be executed synchronously.
            result = Form1.rfid.KillTag(true, killPassword, false, ref maskParam);

            Cursor.Current = Cursors.Default;
            btnKill.Enabled = true;

            // if you call a function asynchronously, function will return immediately.
            // You need to check the execution results are received by the callback function.
            // But If you call a function synchronously, the return value is a function execution results.
            AddString(result.ToString());
            Utils.PlaySound(result == RFID_RESULT.RFID_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
        }
    }
}