﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NRfidApi;
using _Native;

namespace RFID
{
    public partial class frmLock : Form
    {
        public string taretTag = string.Empty;
        RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;

        public frmLock(string _epc)
        {
            InitializeComponent();
            this.Text = "Lock Example";
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }
            this.taretTag = _epc;

            // The maximum length of the mask is 32 bytes.
            int maxTargetLength = RfidApi.MAX_MASK_BYTES * 2;
            if (this.taretTag.Length > maxTargetLength)
                this.taretTag = this.taretTag.Substring(0, maxTargetLength);

            tbxTargetTag.Text = taretTag;

            cbxAction.Items.Add("Lock");
            cbxAction.Items.Add("UnLock");
            cbxAction.Items.Add("Permalock");
            cbxAction.Items.Add("Permaunlock");
            cbxAction.SelectedIndex = 0;

            // If you want to operate synchronously, the operation time must be set to a value other than zero.
            // If set to 0, you may not be able to stop the operation.
            Form1.rfid.OperationTime = Form1.ACCESS_TIME_SECONDS;
        }

        private void frmLock_Closing(object sender, CancelEventArgs e)
        {
            // Set the operating time is unlimited.
            Form1.rfid.OperationTime = Form1.ACCESS_TIME_UNLIMITED;
        }

        public void AddString(string str)
        {
            listBox1.Items.Add(str);
            listBox1.SelectedIndex = listBox1.Items.Count - 1;
        }

        private void cbxAction_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cbxAction.SelectedIndex)
            {
                case 0:
                    btnAction.Text = "Do Lock";
                    break;
                case 1:
                    btnAction.Text = "Do UnLock";
                    break;
                case 2:
                    btnAction.Text = "Do Perma-lock";
                    break;
                case 3:
                    btnAction.Text = "Do Perma-unlock";
                    break;
            }
        }

        private void btnAction_Click(object sender, EventArgs e)
        {
            string accessPassword = null;
            RFIDMASKPARAMS maskParam = new RFIDMASKPARAMS();
            LOCKUNLOCKFIELD lockField = new LOCKUNLOCKFIELD();

            if (!string.IsNullOrEmpty(tbxAccessPassword.Text))
            {
                // access password is a 4-byte data.
                if (tbxAccessPassword.Text.Length != 8)
                {
                    MessageBox.Show("Access password is 4 bytes");
                    tbxAccessPassword.Focus();
                    tbxAccessPassword.SelectAll();
                    return;
                }
            }
            accessPassword = tbxAccessPassword.Text.Trim();

            if (!string.IsNullOrEmpty(tbxTargetTag.Text))
            {
                // To lock only tags that you selected on the previous screen
                maskParam.MemBank = MEM_BANK.EPC;
                maskParam.nOffSet = 4; // nibble offset
                maskParam.MaskPattern = tbxTargetTag.Text.Trim();
            }

            btnAction.Enabled = false;
            Cursor.Current = Cursors.WaitCursor;

            switch (cbxAction.SelectedIndex)
            {
                case 0: // Change the tag to the password accessible state
                    lockField.bAccessPassword = chkAccessPassword.Checked;
                    lockField.bEPC = chkEPC.Checked;
                    lockField.bKillPassword = chkKillPassword.Checked;
                    lockField.bTID = chkTID.Checked;
                    lockField.bUSER = chkUSER.Checked;

                    result = Form1.rfid.LockField(true, ref lockField, false, accessPassword, ref maskParam);
                    break;

                case 1: // Change the tag to the accessible state
                    lockField.bAccessPassword = chkAccessPassword.Checked;
                    lockField.bEPC = chkEPC.Checked;
                    lockField.bKillPassword = chkKillPassword.Checked;
                    lockField.bTID = chkTID.Checked;
                    lockField.bUSER = chkUSER.Checked;

                    result = Form1.rfid.UnLockField(true, ref lockField, false, accessPassword, ref maskParam);
                    break;

                case 2: // Permanently change the tag to the alway not accessible state

                    if (chkKillPassword.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.KILL_PASSWORD, true, false, accessPassword, ref maskParam);
                        AddString(result.ToString());
                    }

                    if (chkAccessPassword.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.ACCESS_PASSWORD, true, false, accessPassword, ref maskParam);
                        AddString(result.ToString());
                    }

                    if (chkEPC.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.EPC, true, false, accessPassword, ref maskParam);
                        AddString(result.ToString());
                    }

                    if (chkTID.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.TID, true, false, accessPassword, ref maskParam);
                        AddString(result.ToString());
                    }

                    if (chkUSER.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.USER, true, false, accessPassword, ref maskParam);
                    }

                    break;

                case 3: // Permanently change the tag to the alway accessible state

                    if (chkKillPassword.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.KILL_PASSWORD, false, false, accessPassword, ref maskParam);
                        AddString(result.ToString());
                    }

                    if (chkAccessPassword.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.ACCESS_PASSWORD, false, false, accessPassword, ref maskParam);
                        AddString(result.ToString());
                    }

                    if (chkEPC.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.EPC, false, false, accessPassword, ref maskParam);
                        AddString(result.ToString());
                    }

                    if (chkTID.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.TID, false, false, accessPassword, ref maskParam);
                        AddString(result.ToString());
                    }

                    if (chkUSER.Checked)
                    {
                        result = Form1.rfid.Permalock(true, PERMALOCK_FIELD.USER, false, false, accessPassword, ref maskParam);
                    }

                    break;
            }

            Cursor.Current = Cursors.Default;
            btnAction.Enabled = true;
            AddString(result.ToString());

            // if you call a function asynchronously, function will return immediately.
            // You need to check the execution results are received by the callback function.
            // But If you call a function synchronously, the return value is a function execution results.
            Utils.PlaySound(result == RFID_RESULT.RFID_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
        }
    }
}