﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NRfidApi;
using _Native;

namespace RFID
{
    public partial class frmRead : Form
    {
        public string taretTag = string.Empty;
        RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;
        string buff = new string(new char[1024]);

        public frmRead(string _epc)
        {
            InitializeComponent();
            this.Text = "Read Example";
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }
            this.taretTag = _epc;

            // The maximum length of the mask is 32 bytes.
            int maxTargetLength = RfidApi.MAX_MASK_BYTES * 2;
            if (this.taretTag.Length > maxTargetLength)
                this.taretTag = this.taretTag.Substring(0, maxTargetLength);

            tbxTargetTag.Text = taretTag;

            cbxMemBank.Items.Add("RESERVED");
            cbxMemBank.Items.Add("EPC");
            cbxMemBank.Items.Add("TID");
            cbxMemBank.Items.Add("USER");
            cbxMemBank.SelectedIndex = 1;

            int idx = 0;
            for (idx = 0; idx <= 255; idx++)
                cbxWordPtr.Items.Add(idx.ToString());
            cbxWordPtr.SelectedIndex = 2;

            for (idx = 1; idx <= 255; idx++)
                cbxWordCnt.Items.Add(idx.ToString());
            cbxWordCnt.SelectedIndex = 5;

            cbxPreDefined.Items.Add("User defined");
            cbxPreDefined.Items.Add("Access password");
            cbxPreDefined.Items.Add("Kill password");
            cbxPreDefined.Items.Add("Protocol-control word");
            cbxPreDefined.SelectedIndex = 0;

            // If you want to operate synchronously, the operation time must be set to a value other than zero.
            // If set to 0, you may not be able to stop the operation.
            Form1.rfid.OperationTime = Form1.ACCESS_TIME_SECONDS;
        }

        //public void testCallbackProc(RFIDCALLBACKDATA cbData)
        //{
        //    string result = new string(new char[512]);
        //    Form1.rfid.GetResult(result, cbData.CallbackType, cbData.wParam, cbData.lParam);
        //    result = result.Substring(0, result.IndexOf(char.MinValue));
        //    System.Diagnostics.Debug.WriteLine("====>>>>> result : " + result);
        //}

        private void frmRead_Closing(object sender, CancelEventArgs e)
        {
            // Set the operating time is unlimited.
            Form1.rfid.OperationTime = Form1.ACCESS_TIME_UNLIMITED;
        }

        public void AddString(string str)
        {
            listBox1.Items.Add(str);
            listBox1.SelectedIndex = listBox1.Items.Count - 1;
        }

        private void cbxPreDefined_SelectedIndexChanged(object sender, EventArgs e)
        {
            int idx = cbxPreDefined.SelectedIndex;
            switch (idx)
            {
                case 0: // user defined
                    cbxMemBank.SelectedIndex = 1;
                    cbxWordPtr.SelectedIndex = 2;
                    cbxWordCnt.SelectedIndex = 5;
                    break;
                case 1: // access password
                    cbxMemBank.SelectedIndex = 0; // RESERVED bank
                    cbxWordPtr.SelectedIndex = 2; // WordPtr = 2
                    cbxWordCnt.SelectedIndex = 1; // WordCnt = 2
                    break;
                case 2: // kill password
                    cbxMemBank.SelectedIndex = 0; // RESERVED bank
                    cbxWordPtr.SelectedIndex = 0; // WordPtr = 0
                    cbxWordCnt.SelectedIndex = 1; // WordCnt = 2
                    break;
                case 3: // protocol-control word
                    cbxMemBank.SelectedIndex = 1; // EPC bank
                    cbxWordPtr.SelectedIndex = 1; // WordPtr = 1
                    cbxWordCnt.SelectedIndex = 0; // WordCnt = 1
                    break;
            }

            cbxMemBank.Enabled = (idx == 0 ? true : false);
            cbxWordPtr.Enabled = (idx == 0 ? true : false);
            cbxWordCnt.Enabled = (idx == 0 ? true : false);
        }

        private void btnRead_Click(object sender, EventArgs e)
        {
            MEM_BANK memBank = MEM_BANK.EPC;
            uint wordPtr = 0;
            uint wordCnt = 0;
            string accessPassword = null;
            RFIDMASKPARAMS maskParam = new RFIDMASKPARAMS();
            string data = string.Empty;

            switch (cbxMemBank.SelectedIndex)
            {
                case 0:
                    memBank = MEM_BANK.RESERVED;
                    break;
                case 1:
                    memBank = MEM_BANK.EPC;
                    break;
                case 2:
                    memBank = MEM_BANK.TID;
                    break;
                case 3:
                    memBank = MEM_BANK.USER;
                    break;
            }

            // The memory area to be accessed is represented by WORD(2 bytes) unit.
            // If you increase the wordPtr 1, start address increases 2 bytes.
            // If you increase the wordCnt 1, the amount of memory increases 2byte.
            wordPtr = Convert.ToUInt32(cbxWordPtr.SelectedItem.ToString());
            wordCnt = Convert.ToUInt32(cbxWordCnt.SelectedItem.ToString());

            if (!string.IsNullOrEmpty(tbxAccessPassword.Text))
            {
                // access password is a 4-byte data.
                if (tbxAccessPassword.Text.Length != 8)
                {
                    MessageBox.Show("Access password is 4 bytes");
                    tbxAccessPassword.Focus();
                    tbxAccessPassword.SelectAll();
                    return;
                }
            }
            accessPassword = tbxAccessPassword.Text.Trim();

            if (!string.IsNullOrEmpty(tbxTargetTag.Text))
            {
                // To read only tags that you selected on the previous screen
                maskParam.MemBank = MEM_BANK.EPC;
                maskParam.nOffSet = 4; // nibble offset
                maskParam.MaskPattern = tbxTargetTag.Text.Trim();
            }

            btnRead.Enabled = false;
            Cursor.Current = Cursors.WaitCursor;
            // The read operation will be executed synchronously.
            result = Form1.rfid.ReadMemBank(true, memBank, wordPtr, wordCnt, false, accessPassword, ref maskParam, buff);
            Cursor.Current = Cursors.Default;
            btnRead.Enabled = true;

            // if you call a function asynchronously, function will return immediately.
            // You need to check the execution results are received by the callback function.
            // But If you call a function synchronously, the return value is a function execution results.
            if (result == RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                // The read operation was executed successfully.
                // Cut only the data
                data = buff.Substring(0, buff.IndexOf(char.MinValue));
                AddString(data);
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                // The read operation has failed.
                AddString(result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
        }

    }
}