﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NRfidApi;
using _Native;

namespace RFID
{
    public partial class frmWrite : Form
    {
        public string taretTag = string.Empty;
        RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;

        public frmWrite(string _epc)
        {
            InitializeComponent();
            this.Text = "Write Example";
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }
            this.taretTag = _epc;

            // The maximum length of the mask is 32 bytes.
            int maxTargetLength = RfidApi.MAX_MASK_BYTES * 2;
            if (this.taretTag.Length > maxTargetLength)
                this.taretTag = this.taretTag.Substring(0, maxTargetLength);

            tbxTargetTag.Text = taretTag;

            cbxMemBank.Items.Add("RESERVED");
            cbxMemBank.Items.Add("EPC");
            cbxMemBank.Items.Add("TID");
            cbxMemBank.Items.Add("USER");
            cbxMemBank.SelectedIndex = 1;

            int idx = 0;
            for (idx = 0; idx <= 255; idx++)
                cbxWordPtr.Items.Add(idx.ToString());
            cbxWordPtr.SelectedIndex = 2;

            tbxWriteData.Text = "41424344";

            // If you want to operate synchronously, the operation time must be set to a value other than zero.
            // If set to 0, you may not be able to stop the operation.
            Form1.rfid.OperationTime = Form1.ACCESS_TIME_SECONDS;
        }

        private void frmWrite_Closing(object sender, CancelEventArgs e)
        {
            // Set the operating time is unlimited.
            Form1.rfid.OperationTime = Form1.ACCESS_TIME_UNLIMITED;
        }

        public void AddString(string str)
        {
            listBox1.Items.Add(str);
            listBox1.SelectedIndex = listBox1.Items.Count - 1;
        }

        private void btnWrite_Click(object sender, EventArgs e)
        {
            MEM_BANK memBank = MEM_BANK.EPC;
            uint wordPtr = 0;
            string accessPassword = null;
            string writeData = null;
            RFIDMASKPARAMS maskParam = new RFIDMASKPARAMS();

            switch (cbxMemBank.SelectedIndex)
            {
                case 0:
                    memBank = MEM_BANK.RESERVED;
                    break;
                case 1:
                    memBank = MEM_BANK.EPC;
                    break;
                case 2:
                    memBank = MEM_BANK.TID;
                    break;
                case 3:
                    memBank = MEM_BANK.USER;
                    break;
            }

            // The memory area to be accessed is represented by WORD(2 bytes) unit.
            // If you increase the wordPtr 1, start address increases 2 bytes.
            wordPtr = Convert.ToUInt32(cbxWordPtr.SelectedItem.ToString());

            if (!string.IsNullOrEmpty(tbxAccessPassword.Text))
            {
                // access password is a 4-byte data.
                if (tbxAccessPassword.Text.Length != 8)
                {
                    MessageBox.Show("Access password is 4 bytes");
                    tbxAccessPassword.Focus();
                    tbxAccessPassword.SelectAll();
                    return;
                }
            }

            accessPassword = tbxAccessPassword.Text.Trim();
            // All data on the RFID module for reading and writing must be a hexadecimal string.
            // For example, if you want to write the 1234, you must input "31323334".
            // In order to write "ABCD", you must enter "41424344".
            writeData = tbxWriteData.Text.Trim();

            if (!string.IsNullOrEmpty(tbxTargetTag.Text))
            {
                // To write only tags that you selected on the previous screen
                maskParam.MemBank = MEM_BANK.EPC;
                maskParam.nOffSet = 4; // nibble offset
                maskParam.MaskPattern = tbxTargetTag.Text.Trim();
            }

            btnWrite.Enabled = false;
            Cursor.Current = Cursors.WaitCursor;
            // The write operation will be executed synchronously.
            result = Form1.rfid.WriteMemBank(true, memBank, wordPtr, writeData, false, accessPassword, ref maskParam);
            Cursor.Current = Cursors.Default;
            btnWrite.Enabled = true;
            AddString(result.ToString());
            // if you call a function asynchronously, function will return immediately.
            // You need to check the execution results are received by the callback function.
            // But If you call a function synchronously, the return value is a function execution results.
            Utils.PlaySound(result == RFID_RESULT.RFID_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
        }

        
    }
}