﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NRfid1356Api;
using NSysSvcApi;
using _Native;

namespace RFID_1356
{
    /// <summary>
    /// All functions of the RFID 1356 API will operate synchronously.
    /// Therefore, the callback function or Windows message is not necessary.
    /// </summary>
    
    public partial class Form1 : Form
    {
        public static Rfid1356Api rfid = null;
        public SysSvcApi sysService = null;
        public HF_RESULT result = HF_RESULT.HF_RESULT_FAILURE;
        public const string title = "RFID 13.56Mhz Example";

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            // create the system service object
            sysService = new SysSvcApi();
            HW_OPTIONS hwOptions = new HW_OPTIONS();
            // Check what type of RFID(HF)
            sysService.GetHardwareOptions(ref hwOptions);
            if (hwOptions.HF == HW_TYPE_HF.NONE)
            {
                sysService.Dispose();
                sysService = null;
                MessageBox.Show("RFID 13.56Mhz is not found!");
                return;
            }

            // create the rfid object
            rfid = new Rfid1356Api();

            for (int i = 0; i <= 255; i++)
            {
                cbxBlock.Items.Add(string.Format("{0:X2}", Convert.ToByte(i)));
            }
            cbxBlock.SelectedIndex = 2;

            cbxKeyType.Items.Add("A");
            cbxKeyType.Items.Add("B");
            cbxKeyType.SelectedIndex = 0;

            cbxKeySet.Items.Add("0");
            cbxKeySet.Items.Add("1");
            cbxKeySet.Items.Add("2");
            cbxKeySet.SelectedIndex = 0;

            tbxWriteData.Text = "2C010000D3FEFFFF2C01000000FF00FF";
            tbxKeyData.Text = "FFFFFFFFFFFFFFFF";
            tbxIncDecValue.Text = "100";

            cbxCardType.Items.Add("True");
            cbxCardType.Items.Add("False");
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the system serivce object
            if (sysService != null)
            {
                sysService.Dispose();
                sysService = null;
            }

            // destruction of the rfid object
            if (rfid != null)
            {
                if (rfid.IsOpen())
                    rfid.Close();

                rfid = null;
            }
        }

        public void AddString(string str)
        {
            if (tbxResponse.Text.Length > 1024)
                tbxResponse.Text = string.Empty;

            tbxResponse.Text = tbxResponse.Text.Insert(tbxResponse.Text.Length, str + "\r\n");
            tbxResponse.Select(tbxResponse.Text.Length, 1);
            tbxResponse.ScrollToCaret();
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            if (rfid.IsOpen())
            {
                AddString("already opened.");
                return;
            }

            result = rfid.Open();
            Utils.PlaySound(result == HF_RESULT.HF_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
            AddString("Open: " + result.ToString());

            cbxCardType.SelectedIndex = rfid.AppendCardType ? 0 : 1;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                return;
            }

            result = rfid.Close();
            Utils.PlaySound(result == HF_RESULT.HF_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
            AddString("Close: " + result.ToString());
        }

        private void btnVer_Click(object sender, EventArgs e)
        {
            if (rfid == null)
            {
                AddString("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string temp = new string(new char[64]);
            string version = string.Empty;

            result = rfid.ReadFirmwareVersion(temp);
            if (result == HF_RESULT.HF_RESULT_SUCCESS)
            {
                version = temp.Substring(0, temp.IndexOf(char.MinValue));
                AddString(version);
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                AddString("ReadFirmwareVersion: " + result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
        }

        private void btnAntOff_Click(object sender, EventArgs e)
        {
            if (rfid == null)
            {
                AddString("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            result = rfid.AntennaOff();
            Utils.PlaySound(result == HF_RESULT.HF_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
            AddString("AntennaOff: " + result.ToString());
        }

        private void btnIso15693_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            new Form2().ShowDialog();
        }

        private void btnUid_Click(object sender, EventArgs e)
        {
            if (rfid == null)
            {
                AddString("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string temp = new string(new char[2048]);
            string data = string.Empty;

            result = rfid.MifareReadUid(temp);
            if (result == HF_RESULT.HF_RESULT_SUCCESS)
            {
                data = temp.Substring(0, temp.IndexOf(char.MinValue));
                AddString(data);
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                AddString("MifareReadUid: " + result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
        }

        private void btnReadBlock_Click(object sender, EventArgs e)
        {
            if (rfid == null)
            {
                AddString("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string temp = new string(new char[2048]);
            string data = string.Empty;
            string block = cbxBlock.SelectedItem.ToString();
            string keyType = cbxKeyType.SelectedItem.ToString();
            string keySet = cbxKeySet.SelectedItem.ToString();

            result = rfid.MifareReadBlock(block, keyType, keySet, temp);
            if (result == HF_RESULT.HF_RESULT_SUCCESS)
            {
                data = temp.Substring(0, temp.IndexOf(char.MinValue));
                AddString(data);
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                AddString("MifareReadBlock: " + result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
        }

        private void btnWriteBlock_Click(object sender, EventArgs e)
        {
            if (rfid == null)
            {
                AddString("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string block = cbxBlock.SelectedItem.ToString();
            string keyType = cbxKeyType.SelectedItem.ToString();
            string keySet = cbxKeySet.SelectedItem.ToString();
            string writeData = tbxWriteData.Text.Trim();

            result = rfid.MifareWriteBlock(block, keyType, keySet, writeData);
            Utils.PlaySound(result == HF_RESULT.HF_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
            AddString("MifareWriteBlock: " + result.ToString());
        }

        private void btnApdu_Click(object sender, EventArgs e)
        {
            if (rfid == null)
            {
                AddString("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string temp = new string(new char[64]);
            string response = string.Empty;
            string apdu = tbxWriteData.Text.Trim();

            result = rfid.Iso14443APDU_Hx(apdu, temp);
            if (result == HF_RESULT.HF_RESULT_SUCCESS)
            {
                response = temp.Substring(0, temp.IndexOf(char.MinValue));
                AddString(response);
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                AddString("Iso14443APDU_Hx: " + result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
        }

        private void btnKeyLoad_Click(object sender, EventArgs e)
        {
            if (rfid == null)
            {
                AddString("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string block = cbxBlock.SelectedItem.ToString();
            string keyType = cbxKeyType.SelectedItem.ToString();
            string keySet = cbxKeySet.SelectedItem.ToString();
            string keyValue = tbxKeyData.Text.Trim();

            result = rfid.MifareKeyLoad(block, keyType, keySet, keyValue);
            Utils.PlaySound(result == HF_RESULT.HF_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
            AddString("MifareKeyLoad: " + result.ToString());
        }

        private void btnInc_Click(object sender, EventArgs e)
        {
            if (rfid == null)
            {
                AddString("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string block = cbxBlock.SelectedItem.ToString();
            string keyType = cbxKeyType.SelectedItem.ToString();
            string keySet = cbxKeySet.SelectedItem.ToString();
            uint incValue = 0;
            try
            {
                incValue = Convert.ToUInt32(tbxIncDecValue.Text.Trim());
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                AddString("invalid data");
                Utils.PlaySound(@"\Windows\Fail.wav");
                tbxIncDecValue.Focus();
                tbxIncDecValue.SelectAll();
                return;
            }

            result = rfid.MifareIncrement(block, keyType, keySet, incValue);
            Utils.PlaySound(result == HF_RESULT.HF_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
            AddString("MifareIncrement: " + result.ToString());
        }

        private void btnDec_Click(object sender, EventArgs e)
        {
            if (rfid == null)
            {
                AddString("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!rfid.IsOpen())
            {
                AddString("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string block = cbxBlock.SelectedItem.ToString();
            string keyType = cbxKeyType.SelectedItem.ToString();
            string keySet = cbxKeySet.SelectedItem.ToString();
            uint incValue = 0;
            try
            {
                incValue = Convert.ToUInt32(tbxIncDecValue.Text.Trim());
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
                AddString("invalid data");
                Utils.PlaySound(@"\Windows\Fail.wav");
                tbxIncDecValue.Focus();
                tbxIncDecValue.SelectAll();
                return;
            }

            result = rfid.MifareDecrement(block, keyType, keySet, incValue);
            Utils.PlaySound(result == HF_RESULT.HF_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
            AddString("MifareDecrement: " + result.ToString());
        }

        private void cbxCardType_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool enable = cbxCardType.SelectedIndex == 0 ? true : false;
            rfid.AppendCardType = enable;
        }
    }
}