﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NRfid1356Api;
using _Native;

namespace RFID_1356
{
    public partial class Form2 : Form
    {
        HF_RESULT result = HF_RESULT.HF_RESULT_FAILURE;

        public Form2()
        {
            InitializeComponent();
            this.Text = "ISO-15693";
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            textBox1.Text = "4Bytes / Block";
            for (int i = 0; i <= 15; i++)
            {
                for (int j = 0; j < 8; j++)
                {
                    tbxWriteData.Text = tbxWriteData.Text + string.Format("{0:X}", Convert.ToByte(i));
                }
            }

            for (int i = 0; i <= 255; i++)
                cbxBlockPtr.Items.Add(string.Format("{0:X2}", Convert.ToByte(i)));
            cbxBlockPtr.SelectedIndex = 0;

            for(int i=1; i<=47; i++)
                cbxBlockCount.Items.Add(string.Format("{0:X2}", Convert.ToByte(i)));
            cbxBlockCount.SelectedIndex = 1;
        }

        public void AddListView(string str)
        {
            ListViewItem item = new ListViewItem(Convert.ToString(listView1.Items.Count + 1));
            item.SubItems.Add(str);

            listView1.Items.Add(item);
            listView1.EnsureVisible(listView1.Items.Count - 1);
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            listView1.Items.Clear();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnUid_Click(object sender, EventArgs e)
        {
            if (Form1.rfid == null)
            {
                AddListView("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!Form1.rfid.IsOpen())
            {
                AddListView("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string temp = new string(new char[2048]);
            string data = string.Empty;

            result = Form1.rfid.Iso15693ReadUid(temp);
            if (result == HF_RESULT.HF_RESULT_SUCCESS)
            {
                data = temp.Substring(0, temp.IndexOf(char.MinValue));
                AddListView(data);
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                AddListView("Iso15693ReadUid: " + result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
        }

        private void btnReadBlock_Click(object sender, EventArgs e)
        {
            if (Form1.rfid == null)
            {
                AddListView("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!Form1.rfid.IsOpen())
            {
                AddListView("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string temp = new string(new char[2048]);
            string data = string.Empty;
            string blockPtr = cbxBlockPtr.SelectedItem.ToString();
            string blockCount = cbxBlockCount.SelectedItem.ToString();

            result = Form1.rfid.Iso15693ReadBlock(blockPtr, blockCount, temp);
            if (result == HF_RESULT.HF_RESULT_SUCCESS)
            {
                data = temp.Substring(0, temp.IndexOf(char.MinValue));
                AddListView(data);
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            else
            {
                AddListView("Iso15693ReadBlock: " + result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
        }

        private void btnWriteBlock_Click(object sender, EventArgs e)
        {
            if (Form1.rfid == null)
            {
                AddListView("rfid is NULL");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            if (!Form1.rfid.IsOpen())
            {
                AddListView("not opened.");
                Utils.PlaySound(@"\Windows\Fail.wav");
                return;
            }

            string blockPtr = cbxBlockPtr.SelectedItem.ToString();
            string writeData = tbxWriteData.Text.Trim();

            result = Form1.rfid.Iso15693WriteBlock(blockPtr, writeData);
            Utils.PlaySound(result == HF_RESULT.HF_RESULT_SUCCESS ? @"\Windows\Success.wav" : @"\Windows\Fail.wav");
            AddListView("Iso15693WriteBlock: " + result.ToString());
        }
    }
}