﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using NRfidApi;
using _Native;

namespace RFID_EPCLengthControl
{
    public partial class Form1 : Form
    {

        // For more information about EPC length field 
        // refer to the Prococol-control(PC) word (6.3.2.1.2.2) of uhfc1g2_1_2_0 document.

        const int BITS_PER_BYTE = 8;
        const int BYTES_PER_WORD = 2;
        const int MAX_MASK_LENGTH = RfidApi.MAX_MASK_BYTES * 2;
        const int OPERATING_SECONDS = 2; // Synchronously operating time

        public RfidApi rfid = null;
        public RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;
        public const string title = "EPC length control Example";
        public ArrayList Tags = null;
        public string oldPC = string.Empty;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            for (int i = 0; i <= 31; i++)
                cbxLength.Items.Add(Convert.ToString(i * BITS_PER_BYTE * BYTES_PER_WORD));
            cbxLength.SelectedIndex = 0;

            // create the rfid object
            rfid = new RfidApi();
            RFID_RESULT result = rfid.PowerOn();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                MessageBox.Show("PowerOn : " + result.ToString());
                return;
            }
            result = rfid.Open();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                rfid.PowerOff();
                MessageBox.Show("Open : " + result.ToString());
                return;
            }

            // In order to receive the results of the executed asynchronously, the callback function must be registered.
            // If you're using only the functions synchronously, callback does not need to be registered.
            rfid.SetCallback(new RfidCallbackProc(rfidCallbackProc));

            Tags = new ArrayList();
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the rfid object
            if (rfid != null)
            {
                rfid.Close();
                rfid.PowerOff();
                rfid = null;
            }
        }

        public void rfidCallbackProc(RFIDCALLBACKDATA cbData)
        {
            string temp = new string(new char[1024]);
            string Msg = string.Empty;

            rfid.GetResult(temp, cbData.CallbackType, cbData.wParam, cbData.lParam);
            Msg = temp.Substring(0, temp.IndexOf(Char.MinValue));

            // When the memory value is received from the RFID module
            if (cbData.CallbackType == RFID_CALLBACK_TYPE.RFIDCALLBACKTYPE_DATA)
            {
                if (Tags.Contains(Msg))
                {
                    foreach (ListViewItem item in listView1.Items)
                    {
                        if (item.SubItems[0].Text.Equals(Msg))
                        {
                            item.SubItems[1].Text = Convert.ToString(Convert.ToInt32(item.SubItems[1].Text) + 1);
                            break;
                        }
                    }
                }
                else
                {
                    ListViewItem item = new ListViewItem(Msg);
                    item.SubItems.Add("1");
                    int count = listView1.Items.Count;
                    listView1.Items.Add(item);
                    Tags.Add(Msg);
                }
            }
            else // When the response is received from the RFID module
            {
                // inventory has been discontinued.
                if (Msg.Equals("Multi Read Stop"))
                    btnInventory.Text = "Inventory";
            }
        }

        public int GetEpcLength(string PC)
        {
            Byte nTemp = 0;
            Byte nEpcLength = 0;
            int nRtn = 0;

            if (PC.Length != 4)
                return -1;

            try
            {
                // EPC length information in the first byte of the PC.
                nTemp = Convert.ToByte(PC.Substring(0, 2), 16);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return -1;
            }

            // Bits 4 ~ 8 is EPC length field, The length of the EPC, in Words
            nEpcLength = (Byte)(nTemp >> 3);

            // Converts into the bit unit.
            nRtn = nEpcLength * BYTES_PER_WORD * BITS_PER_BYTE;

            System.Diagnostics.Debug.WriteLine(String.Format("nRtn : {0}bits ", nRtn));

            return nRtn;
        }

        public bool SetEpcLength(string OldPC, int Length, ref string NewPC)
        {
            Byte PC_HI = 0x00;
            Byte PC_LO = 0x00;
            Byte PC_HI_NEW = 0x00;
            Byte umi = 0x04;    // User-memory indicator
            Byte xi = 0x02;     // XPC_W1 indicator
            Byte nsi = 0x01;    // numbering system identifier
            int UMI = 0;
            int XI = 0;
            int NSI = 0;
            int nTemp = 0;

            if (OldPC.Length != 4)
                return false;

            // EPC length can be set to 0-496bits(31words).
            if (Length < 0 || Length > 496)
                return false;

            try
            {
                // EPC length information in the first byte of the PC.
                PC_HI = Convert.ToByte(OldPC.Substring(0, 2), 16);
                PC_LO = Convert.ToByte(OldPC.Substring(2, 2), 16);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                return false;
            }

            // Save the current values.(bit 1 ~3)
            if ((PC_HI & umi) == umi)
                UMI = umi;

            if ((PC_HI & xi) == xi)
                XI = xi;

            if ((PC_HI & nsi) == nsi)
                NSI = nsi;

            // Converts into the word unit.
            nTemp = Length / BITS_PER_BYTE / BYTES_PER_WORD;

            // Makes a one byte 
            PC_HI_NEW = Convert.ToByte((nTemp <<= 3) | UMI | XI | NSI);

            // Makes a new PC(protocol-control word)
            NewPC = String.Format("{0:X2}{1:X2}", PC_HI_NEW, PC_LO);

            return true;
        }

        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listView1.SelectedIndices.Count > 0)
            {
                int idx = listView1.SelectedIndices[0];
                string epc = listView1.Items[idx].Text;
                string maskingPattern = string.Empty;

                if (epc.Length <= 4)
                    return;

                oldPC = epc.Substring(0, 4);
                maskingPattern = epc.Substring(4, epc.Length - 4);

                if(maskingPattern.Length > MAX_MASK_LENGTH)
                    maskingPattern = maskingPattern.Substring(0, MAX_MASK_LENGTH);

                tbxTarget.Text = maskingPattern;
            }
        }

        private void btnInventory_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            if (btnInventory.Text.Equals("Inventory"))
            {
                rfid.OperationTime = 0;
                // Starts inventory(Read multiple tags)
                result = rfid.ReadEpc(false, RFID_READ_TYPE.EPC_GEN2_MULTI_TAG, null);
                if (result == RFID_RESULT.RFID_RESULT_SUCCESS)
                    btnInventory.Text = "Stop";
                else
                    MessageBox.Show("ReadEpc : " + result.ToString());
            }
            else
            {
                // Stop inventory
                result = rfid.Stop();
                if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
                    MessageBox.Show("Stop : " + result.ToString());
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            Tags.Clear();
            listView1.Items.Clear();
            tbxTarget.Text = string.Empty;
            tbxPassword.Text = string.Empty;
            cbxLength.SelectedIndex = 0;
        }

        public void EnableControl(bool enable)
        {
            Cursor.Current = enable ? Cursors.Default : Cursors.WaitCursor;

            btnClear.Enabled = enable;
            btnInventory.Enabled = enable;
            tbxTarget.Enabled = enable;
            tbxPassword.Enabled = enable;
            cbxLength.Enabled = enable;
            btnWrite.Enabled = enable;
            btnRead.Enabled = enable;
        }

        private void btnRead_Click(object sender, EventArgs e)
        {
            string readData = new string(new char[512]);
            string targetTag = string.Empty;
            string accessPassword = string.Empty;
            int epcLength = 0;
            string pc = string.Empty;

            // If you select a tag
            if (listView1.SelectedIndices.Count > 0)
            {
                int idx = listView1.SelectedIndices[0];
                string data = listView1.Items[idx].Text;
                if (data.Length < 4)
                {
                    Utils.PlaySound(@"\Windows\Fail.wav");
                    MessageBox.Show("Invalid data : " + data);
                    return;
                }

                // Cut the PC(protocol-control word)
                pc = data.Substring(0, 4);
                // calculate EPC length
                epcLength = GetEpcLength(pc);
                // display EPC length
                cbxLength.SelectedItem = epcLength.ToString();
                Utils.PlaySound(@"\Windows\Success.wav");
                return;
            }

            if (rfid == null)
                return;

            // If you did not select a tag, The module will read any tag.
            targetTag = null;

            if (!string.IsNullOrEmpty(tbxPassword.Text))
                accessPassword = tbxPassword.Text.Trim();

            if (rfid == null)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("rfid is null");
                return;
            }

            EnableControl(false);

            // If you want to operate synchronously, the operation time must be set to a value other than zero.
            // If set to 0, you may not be able to stop the operation.
            rfid.OperationTime = OPERATING_SECONDS;
            // Try to read PC
            result = rfid.ReadMemBank(true, MEM_BANK.EPC, 1, 1, false, accessPassword, readData);
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                EnableControl(true);
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("ReadMemBank : " + result.ToString());
                return;
            }

            // Set the operating time is unlimited.
            rfid.OperationTime = 0;

            // Cut only the data from buffer
            pc = readData.Substring(0, readData.IndexOf(char.MinValue));
            oldPC = pc;
            // calculate EPC length
            epcLength = GetEpcLength(pc);

            // Get EPC value of Tag that had been accessed
            rfid.GetAccessedEPC(readData);
            tbxTarget.Text = readData.Substring(4, readData.IndexOf(Char.MinValue));

            cbxLength.SelectedItem = epcLength.ToString();
            EnableControl(true);
            Utils.PlaySound(@"\Windows\Success.wav");
        }

        private void btnWrite_Click(object sender, EventArgs e)
        {
            string readData = new string(new char[512]);
            string maskingPattern = string.Empty;
            string newPC = string.Empty;
            string accessPassword = string.Empty;
            RFIDMASKPARAMS mask = new RFIDMASKPARAMS();
            int LenOfNewEPC = 0;

            if (rfid == null)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("rfid is null");
                return;
            }

            if (string.IsNullOrEmpty(tbxTarget.Text))
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("Specify the EPC to write EPC length");
                tbxTarget.Focus();
                tbxTarget.SelectAll();
                return;
            }

            // Make new EPC length
            LenOfNewEPC = Convert.ToInt32(cbxLength.SelectedItem.ToString());
            if (!SetEpcLength(oldPC, LenOfNewEPC, ref newPC))
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetEpcLength failed!");
                return;
            }

            // The maximum length of the mask is 32 bytes.
            maskingPattern = tbxTarget.Text.Trim();
            if (maskingPattern.Length > MAX_MASK_LENGTH)
                maskingPattern = maskingPattern.Substring(0, MAX_MASK_LENGTH);

            mask.nOffSet = 8;
            mask.MemBank = MEM_BANK.EPC;
            mask.MaskPattern = maskingPattern;

            accessPassword = tbxPassword.Text.Trim();

            EnableControl(false);
            rfid.OperationTime = OPERATING_SECONDS;
            // Try to write new PC value synchronously.
            result = rfid.WriteMemBank(true, MEM_BANK.EPC, 1, newPC, false, accessPassword, ref mask);
            if (result == RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                // Writing new PC successfully completed
                EnableControl(true);
                Utils.PlaySound(@"\Windows\Success.wav");
                return;
            }
            else if (result == RFID_RESULT.RFID_RESULT_ACCESS_ERROR) // access error has occurred
            {
                // attempts to read access password
                result = rfid.ReadMemBank(true, MEM_BANK.RESERVED, 2, 2, false, null, ref mask, readData);
                if (result == RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    // Reading successfully completed
                    accessPassword = readData.Substring(0, 8);
                    tbxPassword.Text = accessPassword;
                    // Retry to write new PC value
                    result = rfid.WriteMemBank(true, MEM_BANK.EPC, 1, newPC, false, accessPassword, ref mask);
                    if (result == RFID_RESULT.RFID_RESULT_SUCCESS)
                    {
                        // Writing new PC successfully completed
                        EnableControl(true);
                        Utils.PlaySound(@"\Windows\Success.wav");
                        return;
                    }
                    else
                    {
                        // Writing new PC failed!
                        EnableControl(true);
                        Utils.PlaySound(@"\Windows\Fail.wav");
                        MessageBox.Show(result.ToString());
                        return;
                    }
                }
                else
                {
                    // Writing new PC failed!
                    EnableControl(true);
                    Utils.PlaySound(@"\Windows\Fail.wav");
                    MessageBox.Show(result.ToString());
                    return;
                }
            }
            else
            {
                // Writing new PC failed!
                EnableControl(true);
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show(result.ToString());
                return;
            }
        }

        
    }
}