﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NRfidApi;
using _Native;

namespace RFID_GetEPCValueOfAccessedTag
{
    public partial class Form1 : Form
    {
        public const string title = "Get accessed EPC Example";
        public static RfidApi rfid = null;
        public RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;
        public bool isSync = false;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            // create the rfid object
            rfid = new RfidApi();
            result = rfid.PowerOn();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                MessageBox.Show("PowerOn : " + result.ToString());
                return;
            }
            result = rfid.Open();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                rfid.PowerOff();
                MessageBox.Show("Open : " + result.ToString());
                return;
            }
            // In order to receive the results of the executed asynchronously, the callback function must be registered.
            // If you're using only the functions synchronously, callback does not need to be registered.
            rfid.SetCallback(new RfidCallbackProc(rfidCallbackProc));

            cbxMemBank.Items.Add("RESERVED");
            cbxMemBank.Items.Add("EPC");
            cbxMemBank.Items.Add("TID");
            cbxMemBank.Items.Add("USER");
            cbxMemBank.SelectedIndex = 1;

            int idx = 0;
            for (idx = 0; idx <= 255; idx++)
                cbxWordPtr.Items.Add(idx.ToString());
            cbxWordPtr.SelectedIndex = 2;

            for (idx = 1; idx <= 255; idx++)
                cbxWordCnt.Items.Add(idx.ToString());
            cbxWordCnt.SelectedIndex = 1;

            isSync = radioSync.Checked;
            System.Diagnostics.Debug.WriteLine("isSync : " + isSync.ToString());
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the rfid object
            if (rfid != null)
            {
                rfid.Close();
                rfid.PowerOff();
                rfid = null;
            }
        }

        public void AddString(string str)
        {
            listBox1.Items.Add(str);
            listBox1.SelectedIndex = listBox1.Items.Count - 1;
        }

        /// <summary>
        /// Callback function for receiving a result of the asynchronous operation
        /// </summary>
        /// <param name="cbData"></param>
        public void rfidCallbackProc(RFIDCALLBACKDATA cbData)
        {
            string buff = new string(new char[1024]);
            string data = string.Empty;
            if (rfid == null)
                return;

            // In order to read the results you should call GetResult function.
            rfid.GetResult(buff, cbData.CallbackType, cbData.wParam, cbData.lParam);
            data = buff.Substring(0, buff.IndexOf(char.MinValue));

            if (btnAccess.Text.Equals("Stop"))
                btnAccess.Text = "Access(Read)";

            if (cbData.CallbackType == RFID_CALLBACK_TYPE.RFIDCALLBACKTYPE_DATA)
            {
                AddString("AccessedData : " + data);

                // If Access(read) is successfully completed, you can get EPC value of Tag that had been accessed
                if (chkGetEPC.Checked)
                {
                    rfid.GetAccessedEPC(buff);
                    data = buff.Substring(0, buff.IndexOf(char.MinValue));
                    AddString("AccessedEPC : " + data);
                }
            }
            else
            {
                if(!data.Equals("Success") && !data.Equals("OK"))
                    AddString("Reply : " + data);
            }
        }

        private void btnAccess_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            MEM_BANK memBank = MEM_BANK.EPC;
            uint wordPtr = 0;
            uint wordCnt = 0;
            string data = string.Empty;
            string buff = new string(new char[1024]);

            switch (cbxMemBank.SelectedIndex)
            {
                case 0:
                    memBank = MEM_BANK.RESERVED;
                    break;
                case 1:
                    memBank = MEM_BANK.EPC;
                    break;
                case 2:
                    memBank = MEM_BANK.TID;
                    break;
                case 3:
                    memBank = MEM_BANK.USER;
                    break;
            }

            // The memory area to be accessed is represented by WORD(2 bytes) unit.
            // If you increase the wordPtr 1, start address increases 2 bytes.
            // If you increase the wordCnt 1, the amount of memory increases 2byte.
            wordPtr = Convert.ToUInt32(cbxWordPtr.SelectedItem.ToString());
            wordCnt = Convert.ToUInt32(cbxWordCnt.SelectedItem.ToString());

            // Of course, you can use other Access commands other than Read.(Write/Lock/Kill)
            if (isSync)
            {
                btnAccess.Enabled = false;
                Cursor.Current = Cursors.WaitCursor;
                rfid.OperationTime = 2;
                // The read operation will be executed synchronously.
                result = rfid.ReadMemBank(isSync, memBank, wordPtr, wordCnt, false, null, buff);
                rfid.OperationTime = 0;

                // if you call a function asynchronously, function will return immediately.
                // You need to check the execution results are received by the callback function.
                // But If you call a function synchronously, the return value is a function execution results.
                if (result == RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    data = buff.Substring(0, buff.IndexOf(char.MinValue));
                    AddString("AccessedData : " + data);

                    if (chkGetEPC.Checked)
                    {
                        // If Access(read) is successfully completed, you can get EPC value of Tag that had been accessed
                        rfid.GetAccessedEPC(buff);
                        data = buff.Substring(0, buff.IndexOf(char.MinValue));
                        AddString("AccessedEPC : " + data);
                    }
                }
                else
                {
                    AddString(result.ToString());
                }
                Cursor.Current = Cursors.Default;
                btnAccess.Enabled = true;
            }
            else
            {
                if (btnAccess.Text.Equals("Access(Read)"))
                {
                    btnAccess.Text = "Stop";
                    // The read operation will be executed asynchronously.
                    result = rfid.ReadMemBank(isSync, memBank, wordPtr, wordCnt, false, null, null);
                    if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
                    {
                        Utils.PlaySound(@"\Windows\Fail.wav");
                        AddString("ReadMemBank : " + result.ToString());
                    }
                }
                else
                {
                    // stop reading operation
                    rfid.Stop();
                }
            }
        }

        private void radioSync_CheckedChanged(object sender, EventArgs e)
        {
            isSync = radioSync.Checked;
        }
    }
}