﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NRfidApi;
using _Native;

namespace RFID_IndicateRSSI
{
    public partial class Form1 : Form
    {
        public const string title = "Indicate RSSI Example";
        public static RfidApi rfid = null;
        public RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            // create the rfid object
            rfid = new RfidApi();
            result = rfid.PowerOn();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                MessageBox.Show("PowerOn : " + result.ToString());
                return;
            }
            result = rfid.Open();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                rfid.PowerOff();
                MessageBox.Show("Open : " + result.ToString());
                return;
            }
            // In order to receive the results of the executed asynchronously, the callback function must be registered.
            // If you're using only the functions synchronously, callback does not need to be registered.
            rfid.SetCallback(new RfidCallbackProc(rfidCallbackProc));
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the rfid object
            if (rfid != null)
            {
                rfid.Close();
                rfid.PowerOff();
                rfid = null;
            }
        }

        public void AddString(string str)
        {
            if (textBox1.Text.Length > 4096)
                textBox1.Text = string.Empty;

            textBox1.Text = textBox1.Text.Insert(textBox1.Text.Length, str + "\r\n");
            textBox1.Select(textBox1.Text.Length, 1);
            textBox1.ScrollToCaret();
        }

        /// <summary>
        /// Callback function for receiving a result of the asynchronous operation
        /// </summary>
        /// <param name="cbData"></param>
        public void rfidCallbackProc(RFIDCALLBACKDATA cbData)
        {
            string temp = new string(new char[1024]);
            string Msg = string.Empty;

            // In order to read the results you should call GetResult function.
            rfid.GetResult(temp, cbData.CallbackType, cbData.wParam, cbData.lParam);
            // Cut only the data from buffer
            Msg = temp.Substring(0, temp.IndexOf(char.MinValue));

            AddString(Msg);
            if (cbData.CallbackType == RFID_CALLBACK_TYPE.RFIDCALLBACKTYPE_REPLY)
            {
                if (radioInventory.Checked)
                {
                    if (Msg.Equals("Multi Read Stop"))
                    {
                        chkRSSI.Enabled = true;
                        radioAccess.Enabled = true;
                        radioInventory.Enabled = true;
                        btnInventory.Text = "Start";
                    }
                }
                else
                {
                    chkRSSI.Enabled = true;
                    radioAccess.Enabled = true;
                    radioInventory.Enabled = true;
                    btnInventory.Text = "Start";
                }
            }
        }

        private void btnInventory_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            if (btnInventory.Text.Equals("Start"))
            {
                textBox1.Text = string.Empty;
                chkRSSI.Enabled = false;
                radioAccess.Enabled = false;
                radioInventory.Enabled = false;
                btnInventory.Text = "Stop";

                if (radioInventory.Checked)
                    result = rfid.ReadEpc(false, RFID_READ_TYPE.EPC_GEN2_MULTI_TAG, null);
                else
                    result = rfid.ReadMemBank(false, MEM_BANK.EPC, 2, 2, false, null, null);

                if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    chkRSSI.Enabled = true;
                    radioAccess.Enabled = true;
                    radioInventory.Enabled = true;
                    btnInventory.Text = "Start";
                    Utils.PlaySound(@"\Windows\Fail.wav");
                    AddString("ReadEpc : " + result.ToString());
                }
            }
            else
            {
                // Stop inventory
                rfid.Stop();
            }
        }

        private void chkRSSI_CheckStateChanged(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            // In order to receive the RSSI information, sets this function to true.
            // Tag data including the RSSI is to be received.
            // For example, "30000000000B1107000000000000,RSSI=-65"
            rfid.EnableExtendedInformation(chkRSSI.Checked);
        }
    }
}