﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NRfidApi;
using _Native;

namespace RFID_SelectCommands
{

    public partial class Form1 : Form
    {
        public const string title = "Select commands Example";
        public static RfidApi rfid = null;
        public RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;

        public Form1()
        {
            InitializeComponent();

            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            rfid = new RfidApi();
            result = rfid.PowerOn();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                MessageBox.Show("PowerOn : " + result.ToString());
                return;
            }
            result = rfid.Open();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                rfid.PowerOff();
                MessageBox.Show("Open : " + result.ToString());
                return;
            }

            rfid.SetCallback(new RfidCallbackProc(rfidCallbackProc));

            radioNone.Checked = true;

            tbxPattern1.Text = "30003036192C8035074000284A0A";
            tbxPattern2.Text = "3400313233340000000000000500";

            cbxSession.Items.Add(SESSION_TYPE.SESSION_0.ToString());
            cbxSession.Items.Add(SESSION_TYPE.SESSION_1.ToString());
            cbxSession.Items.Add(SESSION_TYPE.SESSION_2.ToString());
            cbxSession.Items.Add(SESSION_TYPE.SESSION_3.ToString());
            cbxSession.SelectedIndex = 0;

            cbxTarget.Items.Add(INVENTORIED_STATE.STATE_A.ToString());
            cbxTarget.Items.Add(INVENTORIED_STATE.STATE_B.ToString());
            cbxTarget.Items.Add(INVENTORIED_STATE.STATE_AB.ToString());
            cbxTarget.SelectedIndex = 0;
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (rfid != null)
            {
                rfid.Close();
                rfid.PowerOff();
                rfid = null;
            }
        }

        public SESSION_TYPE getSesstion()
        {
            switch (cbxSession.SelectedIndex)
            {
                case 0:
                    return SESSION_TYPE.SESSION_0;
                case 1:
                    return SESSION_TYPE.SESSION_1;
                case 2:
                    return SESSION_TYPE.SESSION_2;
                case 3:
                    return SESSION_TYPE.SESSION_3;
                default:
                    return SESSION_TYPE.SESSION_0;
            }
        }

        public INVENTORIED_STATE getTarget()
        {
            switch (cbxTarget.SelectedIndex)
            {
                case 0:
                    return INVENTORIED_STATE.STATE_A;
                case 1:
                    return INVENTORIED_STATE.STATE_B;
                case 2:
                    return INVENTORIED_STATE.STATE_AB;
                default:
                    return INVENTORIED_STATE.STATE_A;
            }
        }

        public void AddString(string str)
        {
            listBox1.Items.Add(str);
            listBox1.SelectedIndex = listBox1.Items.Count - 1;
        }

        public void rfidCallbackProc(RFIDCALLBACKDATA cbData)
        {
            string buff = new string(new char[1024]);
            string data = string.Empty;
            if (rfid == null)
                return;

            rfid.GetResult(buff, cbData.CallbackType, cbData.wParam, cbData.lParam);
            data = buff.Substring(0, buff.IndexOf(char.MinValue));

            if (cbData.CallbackType == RFID_CALLBACK_TYPE.RFIDCALLBACKTYPE_DATA)
            {
                AddString(data);
            }
            else
            {
                if (!data.Equals("Success") && !data.Equals("OK"))
                    AddString("Reply : " + data);
            }
        }

        public static bool IsHexString(string str)
        {
            if (string.IsNullOrEmpty(str))
                return false;

            if (str.Length % 2 != 0)
                return false;

            bool bRet = false;
            int loop = str.Length / 2;
            int incr = 0;
            string s = String.Empty;

            for (int i = 0; i < loop; i++)
            {
                try
                {
                    s = str.Substring(incr, 2);
                    Byte.Parse(s, System.Globalization.NumberStyles.HexNumber);
                    incr += 2;
                    bRet = true;
                }
                catch (Exception ex)
                {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                    bRet = false;
                    break;
                }
            }

            return bRet;
        }

        private void btnInventory_Click(object sender, EventArgs e)
        {
            RFIDMASKPARAMS maskParams;
            RFIDMASKPARAMS_EX maskParamsEx;
            RFIDSELMASKPARAMS_EX[] selMaskParams;
            string funcName = string.Empty;

            // In this example, the data of EPC Bank was used as the masking pattern.
            // Of course, other Bank data may be used as the masking pattern.

            // The Tag data including the PC(Protocol-Control) word.
            string targetTag1 = tbxPattern1.Text.Trim();
            string targetTag2 = tbxPattern2.Text.Trim();

            int maxLength = RfidApi.MAX_MASK_BYTES * 2;

            if (targetTag1.Length > maxLength)
                targetTag1 = targetTag1.Substring(0, maxLength);

            if (targetTag2.Length > maxLength)
                targetTag2 = targetTag2.Substring(0, maxLength);

            if(rfid == null)
                return;

            listBox1.Items.Clear();

            // selects whether Tags whose inventoried flag ia A or B participate in the inventory round
            rfid.InventoryTarget = getTarget();
            // choose a session for the inventory round
            rfid.Session = getSesstion();


            // Run the inventory without using select.
            // Read all tags within the effective range.
            if (radioNone.Checked)
            {
                funcName = "ReadEpc : ";
                result = rfid.ReadEpc(false, RFID_READ_TYPE.EPC_GEN2_MULTI_TAG, null);
            }

            // Select a tag in one piece and run the inventory.
            // using nibble offset and the masking pattern in string form
            if (radioMask.Checked)
            {
                funcName = "ReadEpc : ";

                if (!IsHexString(targetTag1))
                {
                    Utils.PlaySound(@"\Windows\Fail.wav");
                    AddString("invalid masking pattern");
                    return;
                }

                maskParams = new RFIDMASKPARAMS();
                maskParams.MemBank = MEM_BANK.EPC;
                maskParams.nOffSet = 4; // Start of PC(nibble offset)
                maskParams.MaskPattern = targetTag1;

                result = rfid.ReadEpc(false, RFID_READ_TYPE.EPC_GEN2_MULTI_TAG, ref maskParams, null);
            }

            // Select a tag in one piece and run the inventory.
            // using bit offset and the masking pattern in byte array
            if (radioMaskEx.Checked)
            {
                funcName = "ReadEpcEx : ";

                if (!IsHexString(targetTag1))
                {
                    Utils.PlaySound(@"\Windows\Fail.wav");
                    AddString("invalid masking pattern");
                    return;
                }

                maskParamsEx = new RFIDMASKPARAMS_EX();
                maskParamsEx.MemBank = MEM_BANK.EPC;
                maskParamsEx.nBits = (uint)((targetTag1.Length / 2) * 8); // bit count of masking pattern
                maskParamsEx.nOffSet = 16; // Start of PC(bit offset)
                maskParamsEx.MaskPattern = new byte[RfidApi.MAX_MASK_BYTES];
                for (int i = 0; i < targetTag1.Length / 2; i++)
                {
                    maskParamsEx.MaskPattern[i] = Convert.ToByte(targetTag1.Substring(i * 2, 2), 16);
                }

                result = rfid.ReadEpcEx(false, RFID_READ_TYPE.EPC_GEN2_MULTI_TAG, ref maskParamsEx, null);
            }

            // Select multiple tags and run the inventory.
            // using bit offset and the masking pattern in byte array
            if (radioSelMask.Checked)
            {
                int i = 0;
                funcName = "ReadEpcEx : ";

                if (!IsHexString(targetTag1) || !IsHexString(targetTag2))
                {
                    Utils.PlaySound(@"\Windows\Fail.wav");
                    AddString("invalid masking pattern");
                    return;
                }

                selMaskParams = new RFIDSELMASKPARAMS_EX[2]; // You can set up to 8

                // For more information about the following parameters, 
                // refer to the Select commands (6.3.2.11.1) of uhfc1g2_1_2_0 document.

                selMaskParams[0].ActionCode = SELECT_ACTION.ACTION_0;
                selMaskParams[0].SelectTarget = SELECT_TARGET.S0;
                selMaskParams[0].MemBank = MEM_BANK.EPC;
                selMaskParams[0].Bits = (uint)((targetTag1.Length / 2) * 8); // bit count of masking pattern
                selMaskParams[0].OffSet = 16; // Start of PC(bit offset)
                selMaskParams[0].MaskPattern = new byte[RfidApi.MAX_MASK_BYTES];
                for (i = 0; i < targetTag1.Length / 2; i++)
                {
                    selMaskParams[0].MaskPattern[i] = Convert.ToByte(targetTag1.Substring(i * 2, 2), 16);
                }

                selMaskParams[1].ActionCode = SELECT_ACTION.ACTION_1;
                selMaskParams[1].SelectTarget = SELECT_TARGET.S0;
                selMaskParams[1].MemBank = MEM_BANK.EPC;
                selMaskParams[1].Bits = (uint)((targetTag2.Length / 2) * 8); // bit count of masking pattern
                selMaskParams[1].OffSet = 16; // Start of PC(bit offset)
                selMaskParams[1].MaskPattern = new byte[RfidApi.MAX_MASK_BYTES];
                for (i = 0; i < targetTag2.Length / 2; i++)
                {
                    selMaskParams[1].MaskPattern[i] = Convert.ToByte(targetTag2.Substring(i * 2, 2), 16);
                }

                // If this property is less than 1, select will not be executed.
                rfid.Selects = 1;

                rfid.ReadEpcEx(false, RFID_READ_TYPE.EPC_GEN2_MULTI_TAG, selMaskParams, (uint)selMaskParams.Length, null);
            }

            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                AddString(funcName + result.ToString());
            }
        }

        private void btnRead_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            rfid.Stop();
        }

        private void radioNone_CheckedChanged(object sender, EventArgs e)
        {
            if (radioNone.Checked)
            {
                tbxPattern1.Enabled = false;
                tbxPattern2.Enabled = false;
            }
            else if (radioMask.Checked || radioMaskEx.Checked)
            {
                tbxPattern1.Enabled = true;
                tbxPattern2.Enabled = false;
            }
            else
            {
                tbxPattern1.Enabled = true;
                tbxPattern2.Enabled = true;
            }
        }
    }
}