﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using NRfidApi;
using _Native;

namespace RFID_Read
{
    public partial class Form1 : Form
    {
        // Sometimes, You will need to process the data in sequence.
        // In this example, we will sequentially read the selected memory bank.
        // If you call the function synchronously, it is possible.
        // If you call the function asynchronously, You have to use a separate thread to wait for a response.

        public static RfidApi rfid = null;
        public const string title = "RFID SequentialReadingExample";
        public static AutoResetEvent eventDone = null;
        public static bool isRunning = false;
        public static bool TidChecked = true;
        public static uint TidPtr = 0;
        public static uint TidCount = 0;
        public static bool EpcChecked = true;
        public static uint EpcPtr = 0;
        public static uint EpcCount = 0;
        public static bool UserChecked = true;
        public static uint UserPtr = 0;
        public static uint UserCount = 0;

        public Form1()
        {
            InitializeComponent();

            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                listBox1.Location = new Point(0, 0);
            }

            // create the rfid object
            rfid = new RfidApi();
            RFID_RESULT result = rfid.PowerOn();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                MessageBox.Show("PowerOn : " + result.ToString());
                return;
            }
            result = rfid.Open();
            if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
            {
                rfid.PowerOff();
                MessageBox.Show("Open : " + result.ToString());
                return;
            }
            // In order to receive the results of the executed asynchronously, the callback function must be registered.
            // If you're using only the functions synchronously, callback does not need to be registered.
            rfid.SetCallback(new RfidCallbackProc(rfidCallbackProc));

            chkTID.Checked = true;
            chkEPC.Checked = true;
            chkUSER.Checked = true;

            TidPtr = (uint)nudTidPtr.Value;
            TidCount = (uint)nudTidCount.Value;
            EpcPtr = (uint)nudEpcPtr.Value;
            EpcCount = (uint)nudEpcCount.Value;
            UserPtr = (uint)nudUserPtr.Value;
            UserCount = (uint)nudUserCount.Value;

            // Sets the time of operation
            nudOperationTime.Value = 3;
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the rfid object
            if (rfid != null)
            {
                if (rfid.IsOpen())
                {
                    rfid.Stop();
                    rfid.Close();
                }

                rfid = null;
            }
        }

        private void chkTID_CheckStateChanged(object sender, EventArgs e)
        {
            TidChecked = chkTID.Checked;
            nudTidPtr.Enabled = chkTID.Checked;
            nudTidCount.Enabled = chkTID.Checked;
        }

        private void chkEPC_CheckStateChanged(object sender, EventArgs e)
        {
            EpcChecked = chkEPC.Checked;
            nudEpcPtr.Enabled = chkEPC.Checked;
            nudEpcCount.Enabled = chkEPC.Checked;
        }

        private void chkUSER_CheckStateChanged(object sender, EventArgs e)
        {
            UserChecked = chkUSER.Checked;
            nudUserPtr.Enabled = chkUSER.Checked;
            nudUserCount.Enabled = chkUSER.Checked;
        }

        private void nudTidPtr_ValueChanged(object sender, EventArgs e)
        {
            TidPtr = (uint)nudTidPtr.Value;
            TidCount = (uint)nudTidCount.Value;
            EpcPtr = (uint)nudEpcPtr.Value;
            EpcCount = (uint)nudEpcCount.Value;
            UserPtr = (uint)nudUserPtr.Value;
            UserCount = (uint)nudUserCount.Value;
        }

        private void nudOperationTime_ValueChanged(object sender, EventArgs e)
        {
            if (rfid != null)
            {
                // If you want to operate synchronously, the operation time must be set to a value other than zero.
                // If set to 0, you may not be able to stop the operation.
                rfid.OperationTime = (uint)nudOperationTime.Value;
            }
        }

        /// <summary>
        /// Callback function for receiving a result of the asynchronous operation
        /// </summary>
        /// <param name="cbData"></param>
        public void rfidCallbackProc(RFIDCALLBACKDATA cbData)
        {
            string temp = new string(new char[1024]);
            string readData = string.Empty;

            // In order to read the results you should call GetResult function.
            rfid.GetResult(temp, cbData.CallbackType, cbData.wParam, cbData.lParam);
            // Cut only the data from buffer
            readData = temp.Substring(0, temp.IndexOf(char.MinValue));
            System.Diagnostics.Debug.WriteLine("readData : " + readData);
            if (readData.Equals("OK"))
                return;

            if (cbData.CallbackType == RFID_CALLBACK_TYPE.RFIDCALLBACKTYPE_REPLY)
            {
                // The SET event for the waiting thread(asyncReadWaitCallbackProc).
                if (eventDone != null)
                    eventDone.Set();
            }

            // Let only display data and error codes.
            if (!readData.Equals("Success"))
                AddString("async data : " + readData);
        }

        public delegate void delegateAddString(string str);
        public void AddString(string str)
        {
            listBox1.Items.Add(str);
            listBox1.SelectedIndex = listBox1.Items.Count - 1;
        }

        /// <summary>
        /// Threads that call the function asynchronously
        /// </summary>
        /// <param name="obj"></param>
        public void asyncReadWaitCallbackProc(Object obj)
        {
            RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;
            isRunning = true;

            if(eventDone == null)
                eventDone = new AutoResetEvent(false);

            if (TidChecked)
            {
                // The read operation will be executed asynchronously.
                result = rfid.ReadMemBank(false, MEM_BANK.TID, TidPtr, TidCount, false, null, null);
                if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    BeginInvoke(new delegateAddString(AddString), "ReadMemBank : " + result.ToString());
                }
                //In asynchronous mode, because the function returns immediately, 
                // you have to wait for the result of executing the ReadMemBank.
                // if you call the next function without waiting for a response, the error will occur.(RFID_RESULT_COMMAND_ERROR)
                eventDone.WaitOne();
                Thread.Sleep(0);
            }

            if (EpcChecked)
            {
                // The read operation will be executed asynchronously.
                result = rfid.ReadMemBank(false, MEM_BANK.EPC, EpcPtr, EpcCount, false, null, null);
                if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    BeginInvoke(new delegateAddString(AddString), "ReadMemBank : " + result.ToString());
                }
                //In asynchronous mode, because the function returns immediately, 
                // you have to wait for the result of executing the ReadMemBank.
                // if you call the next function without waiting for a response, the error will occur.(RFID_RESULT_COMMAND_ERROR)
                eventDone.WaitOne();
                Thread.Sleep(0);
            }

            if (UserChecked)
            {
                // The read operation will be executed asynchronously.
                result = rfid.ReadMemBank(false, MEM_BANK.USER, UserPtr, UserCount, false, null, null);
                if (result != RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    BeginInvoke(new delegateAddString(AddString), "ReadMemBank : " + result.ToString());
                }
                //In asynchronous mode, because the function returns immediately, 
                // you have to wait for the result of executing the ReadMemBank.
                // if you call the next function without waiting for a response, the error will occur.(RFID_RESULT_COMMAND_ERROR)
                eventDone.WaitOne();
            }

            if (eventDone != null)
            {
                eventDone.Close();
                eventDone = null;
            }

            isRunning = false;
        }

        /// <summary>
        /// Threads that call the function synchronously
        /// </summary>
        /// <param name="obj"></param>
        public void syncReadWaitCallbackProc(Object obj)
        {
            RFID_RESULT result = RFID_RESULT.RFID_RESULT_FAILURE;
            string temp = new string(new char[1024]);
            string readData = string.Empty;
            isRunning = true;

            if (TidChecked)
            {
                // The read operation will be executed synchronously.
                result = rfid.ReadMemBank(true, MEM_BANK.TID, TidPtr, TidCount, false, null, temp);
                // if you call a function asynchronously, function will return immediately.
                // You need to check the execution results are received by the callback function.
                // But If you call a function synchronously, the return value is a function execution results.
                if (result == RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    // Cut only the data from buffer
                    readData = temp.Substring(0, temp.IndexOf(char.MinValue));
                    // Display data
                    BeginInvoke(new delegateAddString(AddString), "sync data : " + readData);
                }
                else
                {
                    // Display error codes.
                    BeginInvoke(new delegateAddString(AddString), "ReadMemBank : " + result.ToString());
                }
            }

            if (EpcChecked)
            {
                // The read operation will be executed synchronously.
                result = rfid.ReadMemBank(true, MEM_BANK.EPC, EpcPtr, EpcCount, false, null, temp);
                // if you call a function asynchronously, function will return immediately.
                // You need to check the execution results are received by the callback function.
                // But If you call a function synchronously, the return value is a function execution results.
                if (result == RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    // Cut only the data from buffer
                    readData = temp.Substring(0, temp.IndexOf(char.MinValue));
                    BeginInvoke(new delegateAddString(AddString), "sync data : " + readData);
                }
                else
                {
                    // Display error codes.
                    BeginInvoke(new delegateAddString(AddString), "ReadMemBank : " + result.ToString());
                }
            }

            if (UserChecked)
            {
                // The read operation will be executed synchronously.
                result = rfid.ReadMemBank(true, MEM_BANK.USER, UserPtr, UserCount, false, null, temp);
                // if you call a function asynchronously, function will return immediately.
                // You need to check the execution results are received by the callback function.
                // But If you call a function synchronously, the return value is a function execution results.
                if (result == RFID_RESULT.RFID_RESULT_SUCCESS)
                {
                    // Cut only the data from buffer
                    readData = temp.Substring(0, temp.IndexOf(char.MinValue));
                    BeginInvoke(new delegateAddString(AddString), "sync data : " + readData);
                }
                else
                {
                    // Display error codes.
                    BeginInvoke(new delegateAddString(AddString), "ReadMemBank : " + result.ToString());
                }
            }

            isRunning = false;
        }

        public bool synchronousRead()
        {
            if (!isRunning)
                ThreadPool.QueueUserWorkItem(new WaitCallback(syncReadWaitCallbackProc));

            return true;
        }

        public bool asynchronousRead()
        {
            if(!isRunning)
                ThreadPool.QueueUserWorkItem(new WaitCallback(asyncReadWaitCallbackProc));

            return true;
        }

        private void btnSynchronousRead_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            listBox1.Items.Clear();
            synchronousRead();
        }

        private void btnAsynchronousRead_Click(object sender, EventArgs e)
        {
            if (rfid == null)
                return;

            listBox1.Items.Clear();
            asynchronousRead();
        }
    }
}