﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NSmartCardApi;
using NSysSvcApi;
using _Native;

namespace SmartCard
{
    public partial class Form1 : Form
    {
        public const string title = "SmartCard Example";
        SmartCardApi smartCard = null;
        public SysSvcApi sysService = null;
        SCARD_RESULT result = SCARD_RESULT.SCARD_RESULT_FAILURE;
        STATUS_CODE Status = STATUS_CODE.BSTATUS_FAILED;
        ERROR_CODE Error = ERROR_CODE.CMD_FAILED;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            // create the system service object
            sysService = new SysSvcApi();
            HW_OPTIONS hwOptions = new HW_OPTIONS();
            // Check what type of smartCard.
            sysService.GetHardwareOptions(ref hwOptions);
            if (hwOptions.SmartCard == HW_TYPE_SMARTCARD.NONE)
            {
                sysService.Dispose();
                sysService = null;
                MessageBox.Show("SmartCard is not found!");
                return;
            }

            // 1st cmb indicate slot's number.
            cmbSlot.Items.Add("Slot 0");
            cmbSlot.Items.Add("Slot 1");
            cmbSlot.Items.Add("Slot 2");
            cmbSlot.Items.Add("Slot 3");
            cmbSlot.Items.Add("Slot 4");
            cmbSlot.SelectedIndex = 0;

            // 2nd cmb is Voltage mode
            cmbVolt.Items.Add("AUTO");
            cmbVolt.Items.Add("VOLT_1_8");
            cmbVolt.Items.Add("VOLT_3_0");
            cmbVolt.Items.Add("VOLT_5_0");
            cmbVolt.SelectedIndex = 2;

            // 3rd cmb is emv and iso mode.
            cmbMode.Items.Add("EMV_MODE");
            cmbMode.Items.Add("ISO_MODE");
            cmbMode.SelectedIndex = 1;

            // create the smart card object
            smartCard = new SmartCardApi();
            result = smartCard.ProtocolInit();
            if (result != SCARD_RESULT.SCARD_RESULT_SUCCESS)
            {
                smartCard = null;
                MessageBox.Show("ProtocolInit : " + result.ToString());
                return;
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the system serivce object
            if (sysService != null)
            {
                sysService.Dispose();
                sysService = null;
            }

            // destruction of the smart card object
            if (smartCard != null)
            {
                smartCard.ProtocolDeinit();
                smartCard = null;
            }
        }

        private void SetStateError(STATUS_CODE State, ERROR_CODE Error)
        {
            State = State & STATUS_CODE.BSTATUS_ICC_STATUS_MASK;

            tbxStatus.Text = State.ToString();
            tbxError.Text = Error.ToString();
        }

        private void ClearStateError()
        {
            txtData.Text = String.Empty;
            tbxStatus.Text = String.Empty;
            tbxError.Text = String.Empty;
        }

        private void btnSlotStatus_Click(object sender, EventArgs e)
        {
            if (smartCard == null)
                return;

            ClearStateError();

            //=====================================================================
            // Brings the value of the state of the smart card slot.
            // cmbSlot.SelectedIndex = [IN] smart card inserted, slot number
            // Status                = [OUT] command, the status of a smart card for performing.
            // Error                 = [OUT] command, according to the performance results of running the command.
            result = smartCard.GetSlotStatus(Convert.ToByte(cmbSlot.SelectedIndex), ref Status, ref Error);
            if (result != SCARD_RESULT.SCARD_RESULT_SUCCESS)
            {
                MessageBox.Show("GetSlotStatus : " + result.ToString());
                return;
            }

            SetStateError(Status, Error);
        }

        private void btnPowerOn_Click(object sender, EventArgs e)
        {
            if (smartCard == null)
                return;

            VOLTAGE VoltSel = 0;
            OP_MODE IPwrOn = 0;
            byte[] pATR = new byte[1024];
            string strATR = String.Empty;
            uint AtrSize = new uint();
            string sTemp;

            ClearStateError();

            // Module, the voltage applied to.
            switch (cmbVolt.SelectedIndex)
            {
                case 0:
                    VoltSel = VOLTAGE.AUTO_VOLTAGE_SELECTION;
                    break;
                case 1:
                    VoltSel = VOLTAGE.VOLTS_1_8;
                    break;
                case 2:
                    VoltSel = VOLTAGE.VOLTS_3_0;
                    break;
                case 3:
                    VoltSel = VOLTAGE.VOLTS_5_0;
                    break;
            }

            // Module's operationg mode.
            switch (cmbMode.SelectedIndex)
            {
                case 0:
                    IPwrOn = OP_MODE.EMV_MODE;
                    break;
                case 1:
                    IPwrOn = OP_MODE.ISO_MODE;
                    break;
            }

            // power on the Smart Card
            result = smartCard.PowerOn(Convert.ToByte(cmbSlot.SelectedIndex), IPwrOn, VoltSel, ref Status, ref Error, pATR, ref AtrSize);
            if (result != SCARD_RESULT.SCARD_RESULT_SUCCESS)
            {
                SetStateError(Status, Error);
                return;
            }

            sTemp = "ATR : ";

            for (int idx = 0; idx < AtrSize; idx++)
            {
                sTemp += pATR[idx].ToString("X2") + " ";
            }

            txtData.Text = "";
            txtData.Text = sTemp;
            SetStateError(Status, Error);
        }

        private void btnPowerOff_Click(object sender, EventArgs e)
        {
            if (smartCard == null)
                return;

            ClearStateError();

            // power off the Smart Card
            result = smartCard.PowerOff(Convert.ToByte(cmbSlot.SelectedIndex), ref Status, ref Error);
            SetStateError(Status, Error);
        }

        private void btnSetParam_Click(object sender, EventArgs e)
        {
            if (smartCard == null)
                return;

            ClearStateError();

            // this function is used to negotiate with the card per ISO7816 specification's PPS command.
            result = smartCard.SetParameters(Convert.ToByte(cmbSlot.SelectedIndex), TPROTOCOL.PROTOCOL_T0, 0x18, ref Status, ref Error);
            SetStateError(Status, Error);
        }

        private void btnGetParam_Click(object sender, EventArgs e)
        {
            if (smartCard == null)
                return;

            TPROTOCOL TProtocol = new TPROTOCOL();
            byte FiDi = new byte();

            ClearStateError();

            // this function is uset to obatin current smart card's protocol T and FiDi vlaue per ISO7816 specification.
            result = smartCard.GetParameters(Convert.ToByte(cmbSlot.SelectedIndex), ref TProtocol, ref FiDi, ref Status, ref Error);
            if(result != SCARD_RESULT.SCARD_RESULT_SUCCESS)
            {
                SetStateError(Status, Error);
                return;
            }

            string strMsg = string.Format("Protocol={0}, FiDi={1}", TProtocol.ToString(), FiDi.ToString("X2"));
            txtData.Text = strMsg;

            SetStateError(Status, Error);
        }

        private void btnDefault_Click(object sender, EventArgs e)
        {
            if (smartCard == null)
                return;

            ClearStateError();

            // Set to default values.
            smartCard.SetDefaultParameter(Convert.ToByte(cmbSlot.SelectedIndex), ref Status, ref Error);

            SetStateError(Status, Error);
        }

        private void btnTxBlock_Click(object sender, EventArgs e)
        {
            if (smartCard == null)
                return;

            // Be sent in APDU command.
            byte[] txData =             
            {
		        0x00,    /* CLA */
		        0xA4,    /* INS */
		        0x04,    /* P1 : Select by name */
		        0x00,    /* P2 : First or only occurrence */
		        0x0E,    /* Lc */
		        0x31,    /* 1 */
		        0x50,    /* P */
		        0x41,    /* A */
		        0x59,    /* Y */
		        0x2E,    /* . */
		        0x53,    /* S */
		        0x59,    /* Y */
		        0x53,    /* S */
		        0x2E,    /* . */
		        0x44,    /* D */
		        0x44,    /* D */
		        0x46,    /* F */
		        0x30,    /* 0 */
		        0x31,    /* 1 */
		        0x00,    /* Le */
	        };

            byte[] RxData = new byte[1024];
            uint rxDataSize = new uint();
            string sTemp;

            ClearStateError();

            // Send commands to Smart Card
            result = smartCard.TransferBlock(Convert.ToByte(cmbSlot.SelectedIndex), (uint)txData.Length, txData, ref rxDataSize, RxData, ref Status, ref Error);
            if(result != SCARD_RESULT.SCARD_RESULT_SUCCESS)
            {
                SetStateError(Status, Error);
                return;
            }

            sTemp = "RxData : ";

            for (int idx = 0; idx < rxDataSize; idx++)
            {
                sTemp += RxData[idx].ToString("X2") + " ";
            }

            txtData.Text = "";
            txtData.Text = sTemp;
            SetStateError(Status, Error);
        }

        private void btnFwVersion_Click(object sender, EventArgs e)
        {
            if (smartCard == null)
                return;

            byte[] FWVersion = new byte[2];
            string sTemp = "";

            ClearStateError();

            // get firmware version
            result = smartCard.GetFWVersion(Convert.ToByte(cmbSlot.SelectedIndex), FWVersion, ref Status, ref Error);
            if (result != SCARD_RESULT.SCARD_RESULT_SUCCESS)
            {
                MessageBox.Show("GetFWVersion : " + result.ToString());
                return;
            }

            sTemp = "Library("
                + Convert.ToString(FWVersion[0] >> 4) + "."
                + Convert.ToString(FWVersion[0] & 0x0F) + ") Firmware("
                + Convert.ToString(FWVersion[1] >> 4) + "."
                + Convert.ToString(FWVersion[1] & 0x0F) + ")";

            txtData.Text = "";
            txtData.Text = sTemp;

            SetStateError(Status, Error);
        }
    }
}