﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NSysSvcApi;
using _Native;

namespace SystemService
{
    public partial class Form1 : Form
    {
        public const string title = "SystemService Example";
        public SysSvcApi sysService = null;
        public SYSSVC_RESULT result = SYSSVC_RESULT.SYSSVC_RESULT_FAILURE;
        public HW_OPTIONS hwOptions;
        public KBD_STATUS keypadStatus = KBD_STATUS.KBD_STATUS_NUMERIC;
        public const int WIRELESS_DEVICE_WLAN = 0;
        public const int WIRELESS_DEVICE_BLUETOOTH = 2;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            // create the system service object
            sysService = new SysSvcApi();
            hwOptions = new HW_OPTIONS();

            // Get Device ID
            tbxDeviceID.Text = sysService.DeviceID;
            // Get hardware device options
            result = sysService.GetHardwareOptions(ref hwOptions);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                sysService.Dispose();
                sysService = null;
                MessageBox.Show("GetHardwareOptions : " + result.ToString());
                return;
            }

            int nStatus = 0;
            // Check WiFi Power Status
            chkWiFi.Enabled = hwOptions.WLAN == HW_TYPE_WLAN.NONE ? false : true;
            if (chkWiFi.Enabled)
            {
                result = sysService.GetWirelessDevicesPowerState(WIRELESS_DEVICE_WLAN, ref nStatus);
                if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
                {
                    sysService.Dispose();
                    sysService = null;
                    MessageBox.Show("GetWirelessDevicesPowerState : " + result.ToString());
                    return;
                }
                chkWiFi.Checked = nStatus == 1 ? true : false;
            }

            // Check Bluetooth Power Status
            chkBluetooth.Enabled = hwOptions.Bluetooth == HW_TYPE_BLUETOOTH.NONE ? false : true;
            if (chkBluetooth.Enabled)
            {
                result = sysService.GetWirelessDevicesPowerState(WIRELESS_DEVICE_BLUETOOTH, ref nStatus);
                if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
                {
                    sysService.Dispose();
                    sysService = null;
                    MessageBox.Show("GetWirelessDevicesPowerState : " + result.ToString());
                    return;
                }
                chkBluetooth.Checked = nStatus == 1 ? true : false;
            }

            cbxMode.SelectedIndex = 0;

            // Check hardware keyboard input status
            result = sysService.GetKBDStatus(ref keypadStatus);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                sysService.Dispose();
                sysService = null;
                MessageBox.Show("GetKBDStatus : " + result.ToString());
                return;
            }

            switch (keypadStatus)
            {
                case KBD_STATUS.KBD_STATUS_LOWER_CASE:
                    cbxKeypadStatus.SelectedIndex = 0;
                    break;
                case KBD_STATUS.KBD_STATUS_NUMERIC:
                    cbxKeypadStatus.SelectedIndex = 1;
                    break;
                case KBD_STATUS.KBD_STATUS_UPPER_CASE:
                    cbxKeypadStatus.SelectedIndex = 2;
                    break;
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the system service object
            if (sysService != null)
            {
                sysService.Dispose();
                sysService = null;
            }
        }

        public void ReadPowerAndBacklightStatus(byte mode)
        {
            if (sysService == null)
                return;

            ushort nValue = 0;

            result = sysService.SuspendReadTimeoutValue(mode, ref nValue);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
                MessageBox.Show("SuspendReadTimeoutValue : " + result.ToString());
            nudTurnOffDevice.Value = nValue;
            System.Diagnostics.Debug.WriteLine(String.Format("SuspendReadTimeoutValue mode:{0}, value:{1}", mode, nValue));

            result = sysService.BacklightReadTimeoutValue(mode, ref nValue);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
                MessageBox.Show("BacklightReadTimeoutValue : " + result.ToString());
            nudTurnOffBacklight.Value = nValue;
            System.Diagnostics.Debug.WriteLine(String.Format("BacklightReadTimeoutValue mode:{0}, value:{1}", mode, nValue));

            result = sysService.BacklightReadBrightness(mode, ref nValue);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
                MessageBox.Show("BacklightReadBrightness : " + result.ToString());
            nudBacklightBrightness.Value = nValue;
            System.Diagnostics.Debug.WriteLine(String.Format("BacklightReadBrightness mode:{0}, value:{1}", mode, nValue));
        }

        private void chkWiFi_CheckStateChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            bool turnOn = chkWiFi.Checked;

            Cursor.Current = Cursors.WaitCursor;
            // Turn on/off the WiFi power.
            result = sysService.SetWirelessDevicesPowerState(WIRELESS_DEVICE_WLAN, turnOn ? 1 : 0);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetWirelessDevicesPowerState : " + result.ToString());
            }
            else
            {
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            Cursor.Current = Cursors.Default;
        }

        private void chkBluetooth_CheckStateChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            bool turnOn = chkBluetooth.Checked;

            Cursor.Current = Cursors.WaitCursor;
            // Turn on/off the Bluetooth power.
            result = sysService.SetWirelessDevicesPowerState(WIRELESS_DEVICE_BLUETOOTH, turnOn ? 1 : 0);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetWirelessDevicesPowerState : " + result.ToString());
            }
            else
            {
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            Cursor.Current = Cursors.Default;
        }

        private void chkVibrator_CheckStateChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            bool turnOn = chkVibrator.Checked;

            Cursor.Current = Cursors.WaitCursor;
            // Turn on/off the Vibrator power.
            result = sysService.SetVibratorEnable(turnOn);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetVibratorEnable : " + result.ToString());
            }
            else
            {
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            Cursor.Current = Cursors.Default;
        }

        private void chkFlashLight_CheckStateChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            bool turnOn = chkFlashLight.Checked;

            Cursor.Current = Cursors.WaitCursor;
            // Turn on/off the flash light
            result = sysService.EnableFlashLight(turnOn);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("EnableFlashLight : " + result.ToString());
            }
            else
            {
                Utils.PlaySound(@"\Windows\Success.wav");
            }
            Cursor.Current = Cursors.Default;
        }

        private void cbxMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            byte mode = cbxMode.SelectedIndex == 0 ? SysSvcApi.MODE_BAT : SysSvcApi.MODE_AC;
            ReadPowerAndBacklightStatus(mode);
        }

        private void cbxKeypadStatus_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            switch (cbxKeypadStatus.SelectedIndex)
            {
                case 0:
                    keypadStatus = KBD_STATUS.KBD_STATUS_LOWER_CASE;
                    break;
                case 1:
                    keypadStatus = KBD_STATUS.KBD_STATUS_NUMERIC;
                    break;
                case 2:
                    keypadStatus = KBD_STATUS.KBD_STATUS_UPPER_CASE;
                    break;
            }

            // Change hardware keyboard input status
            // Make sure that the icon changes in the task bar.
            result = sysService.SetKBDStatus(keypadStatus);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SetKBDStatus : " + result.ToString());
            }
        }

        private void nudTurnOffDevice_ValueChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            /*
             *  - Windows Mobile -
             *  MODE_BAT : Min:0 ~ Max:300
             *  MODE_AC : Min:0 ~ Max:1800
             *  
             * - Windows CE -
             *  MODE_BAT : Min:0 ~ Max:300
             *  MODE_AC : Min:0 ~ Max:1800
             * */

            byte mode = cbxMode.SelectedIndex == 0 ? SysSvcApi.MODE_BAT : SysSvcApi.MODE_AC;
            ushort nValue = (ushort)nudTurnOffDevice.Value;

            // Set the in seconds.
            // if nValue is 0, The PDA will not turn off.
            result = sysService.SuspendWriteTimeoutValue(mode, nValue);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("SuspendWriteTimeoutValue : " + result.ToString());
            }
        }

        private void nudTurnOffBacklight_ValueChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            /*
            *  - Windows Mobile -
            *  MODE_BAT : Min:0 ~ Max:300
            *  MODE_AC : Min:0 ~ Max:600
            *  
            * - Windows CE -
            *  MODE_BAT : Min:0 ~ Max:300
            *  MODE_AC : Min:0 ~ Max:300
            * */

            byte mode = cbxMode.SelectedIndex == 0 ? SysSvcApi.MODE_BAT : SysSvcApi.MODE_AC;
            ushort nValue = (ushort)nudTurnOffBacklight.Value;

            // Set the value in seconds.
            // if nValue is 0, The backlight will not turn off.
            result = sysService.BacklightWriteTimeoutValue(mode, nValue);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("BacklightWriteTimeoutValue : " + result.ToString());
            }
        }

        private void nudBacklightBrightness_ValueChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            /*
            *  - Windows Mobile -
            *  Min:0 ~ Max:25
            *  
            * - Windows CE -
            *  Min:0 ~ Max:100
            
            * */

            byte mode = cbxMode.SelectedIndex == 0 ? SysSvcApi.MODE_BAT : SysSvcApi.MODE_AC;
            ushort nValue = (ushort)nudBacklightBrightness.Value;

            // Sets the brightness value corresponding to the mode.
            result = sysService.BacklightWriteBrightness(mode, nValue);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("BacklightWriteBrightness : " + result.ToString());
            }
        }

        private void btnSoftReset_Click(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            // perform a soft reset.
            sysService.SoftReset();
        }

        private void btnEnterSuspendState_Click(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            // enter the suspend state
            sysService.EnterSuspendState();
        }

        private void btnHWOptions_Click(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            // Get hardware device options
            result = sysService.GetHardwareOptions(ref hwOptions);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("GetHardwareOptions : " + result.ToString());
            }
            else
            {
                // List the hardware device option status of the PDA you are using.
                Utils.PlaySound(@"\Windows\Success.wav");
                string msg = string.Empty;
                msg = msg + string.Format("Barcode : {0}\r\n", hwOptions.Barcode == HW_TYPE_BARCODE.NONE ? "---" : "Available");
                msg = msg + string.Format("Bluetooth : {0}\r\n", hwOptions.Bluetooth == HW_TYPE_BLUETOOTH.NONE ? "---" : "Available");
                msg = msg + string.Format("Camera : {0}\r\n", hwOptions.Camera == HW_TYPE_CAM.NONE ? "---" : "Available");
                msg = msg + string.Format("FingerPrint : {0}\r\n", hwOptions.FingerPrint == HW_TYPE_FINGERPRINT.NONE ? "---" : "Available");
                msg = msg + string.Format("GPS : {0}\r\n", hwOptions.GPS == HW_TYPE_GPS.NONE ? "---" : "Available");
                msg = msg + string.Format("RFID HF : {0}\r\n", hwOptions.HF == HW_TYPE_HF.NONE ? "---" : "Available");
                msg = msg + string.Format("Modem : {0}\r\n", hwOptions.Modem == HW_TYPE_MODEM.NONE ? "---" : "Available");
                msg = msg + string.Format("MSR : {0}\r\n", hwOptions.MSR == HW_TYPE_MSR.NONE ? "---" : "Available");
                msg = msg + string.Format("Printer : {0}\r\n", hwOptions.Printer == HW_TYPE_PRINTER.NONE ? "---" : "Available");
                msg = msg + string.Format("SmartCard : {0}\r\n", hwOptions.SmartCard == HW_TYPE_SMARTCARD.NONE ? "---" : "Available");
                msg = msg + string.Format("RFID UHF : {0}\r\n", hwOptions.UHF == HW_TYPE_UHF.NONE ? "---" : "Available");
                msg = msg + string.Format("WiFi : {0}\r\n", hwOptions.WLAN == HW_TYPE_WLAN.NONE ? "---" : "Available");

                MessageBox.Show(msg);
            }
        }
    }
}