﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using Microsoft.Win32;
using NSysSvcApi;
using _Native;

namespace SystemService_CheckKeyPresses
{
    /// <summary>
    /// There is two way that your application will detect the keystrokes.
    /// 1. using KeyPreview property
    ///    -> The application must be the activated state. Otherwise, application will not be able to detect keystrokes.
    /// 2. using Key Hooks
    ///    -> Although the application is deactivated state, application can detect keystrokes.
    /// 
    /// </summary>

    public partial class Form1 : Form
    {
        public const string title = "CheckKeyPresses Example";
        public ArrayList specifiedKeys = null;
        public static SysSvcApi sysService = null;

        public enum DEVICENAME : int
        {
            UNKNOWN = 0,
            AT880,
            AT980,
            AT280,
            AT380,
            AB700
        }

        public const int WM_KEYUP = 0x0101;
        public const int WM_KEYDOWN = 0x0100;

        int KEY_F7 = 0;
        int KEY_F8 = 0;
        int KEY_F9 = 0;
        int KEY_F10 = 0;
        int KEY_F11 = 0;
        int KEY_F19 = 0;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);

                KEY_F7 = Convert.ToInt32(Keys.F7);
                KEY_F8 = Convert.ToInt32(Keys.F8);
                KEY_F9 = Convert.ToInt32(Keys.F9);
                KEY_F10 = Convert.ToInt32(Keys.F10); // side key of AB700
                KEY_F11 = Convert.ToInt32(Keys.F11); // side key of AB700
                KEY_F19 = Convert.ToInt32(Keys.F19);
            }
            else
            {
                KEY_F7 = Convert.ToInt32(0xEF);
                KEY_F8 = Convert.ToInt32(0xF0);
                KEY_F9 = Convert.ToInt32(0xD3);
                KEY_F10 = Convert.ToInt32(0xF2); // side key of AB700
                KEY_F11 = Convert.ToInt32(0xF3); // side key of AB700
                KEY_F19 = Convert.ToInt32(0xD2);
            }

            specifiedKeys = new ArrayList();
            getSpecifiedKeys();

            // Enable KeyPreview property
            radioKeyPreview.Checked = true;

            switch (this.GetDeviceName())
            {
                case DEVICENAME.AT980:
                    chkScanKey.Text = "\"SCAN\" Key (both side of PDA)";
                    break;
                case DEVICENAME.AB700:
                    chkSideKey1.Text = "\"F11\" Key (right side of PDA)";
                    chkSideKey2.Text = "\"F10\" Key (right side of PDA)";
                    break;
                case DEVICENAME.AT380:
                    chkSideKey1.Checked = false;
                    chkSideKey1.Enabled = false;
                    chkSideKey2.Text = "\"F8\" Key (left side of PDA)";
                    break;
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (sysService != null)
            {
                // stop the key hook service
                sysService.KeyEventNotificationReset();
                // destruction of the system service object
                sysService.Dispose();
                sysService = null;
            }
        }

        public DEVICENAME GetDeviceName()
        {
            DEVICENAME devName = DEVICENAME.UNKNOWN;

            try
            {
                if (!Utils.IsWinCE) // WM
                {
                    string regValue = (string)Registry.GetValue(@"HKEY_LOCAL_MACHINE\Ident", "Name", string.Empty);
                    if (string.IsNullOrEmpty(regValue))
                        return devName;

                    regValue = regValue.Substring(0, 5);
                    switch (regValue)
                    {
                        case "SI203":
                        case "AT880":
                            devName = DEVICENAME.AT880;
                            break;
                        case "SI303":
                        case "AT980":
                            devName = DEVICENAME.AT980;
                            break;
                        case "SI890":
                        case "AB700":
                            devName = DEVICENAME.AB700;
                            break;
                        case "AT380":
                            devName = DEVICENAME.AT380;
                            break;
                        case "AT280":
                            devName = DEVICENAME.AT280;
                            break;
                    }
                }
                else // CE
                {
                    int regValue = (int)Registry.GetValue(@"HKEY_CURRENT_USER\ControlPanel\KeyTrans", "ModelNumber", 0);
                    devName = (DEVICENAME)regValue;
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }

            return devName;
        }

        private void chkSideKey1_CheckStateChanged(object sender, EventArgs e)
        {
            getSpecifiedKeys();
        }

        /// <summary>
        /// Specifies the key to the key input detected.
        /// </summary>
        public void getSpecifiedKeys()
        {
            specifiedKeys.Clear();
            
            if (chkSideKey1.Checked)
            {
                specifiedKeys.Add(KEY_F7);
                specifiedKeys.Add(KEY_F11);
            }

            if (chkSideKey2.Checked)
            {
                specifiedKeys.Add(KEY_F8);
                specifiedKeys.Add(KEY_F10);
            }

            if (chkScanKey.Checked)
                specifiedKeys.Add(KEY_F9);

            if (chkTriggerKey.Checked)
                specifiedKeys.Add(KEY_F19);
        }

        public void AddString(string str)
        {
            if (lbxLog.Items.Count > 1024)
                lbxLog.Items.Clear();

            lbxLog.Items.Add(str);
            lbxLog.SelectedIndex = lbxLog.Items.Count - 1;
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            if (specifiedKeys.Contains(e.KeyValue))
            {
                Utils.PlaySound(@"\Windows\Success.wav");
                AddString(string.Format("[KeyPreview] KeyValue:0x{0:X2}, State:down", e.KeyValue));
            }
            
            base.OnKeyDown(e);
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            if (specifiedKeys.Contains(e.KeyValue))
            {
                AddString(string.Format("[KeyPreview] KeyValue:0x{0:X2}, State:up", e.KeyValue));
            }
            
            base.OnKeyUp(e);
        }

        public void keyEventCallbackProc(int KeyValue, int KeyState)
        {
            if (specifiedKeys.Contains(KeyValue))
            {
                if(KeyState == WM_KEYDOWN)
                    Utils.PlaySound(@"\Windows\Success.wav");

                AddString(string.Format("[KeyHooks] KeyValue:0x{0:X2},  State:{1}", KeyValue, KeyState == WM_KEYUP ? "up" : "down"));
            }
        }

        private void radioKeyPreview_CheckedChanged(object sender, EventArgs e)
        {
            if (radioKeyPreview.Checked)
            {
                if (sysService != null)
                {
                    // stop the key hook service
                    sysService.KeyEventNotificationReset();
                    // destruction of the system service object
                    sysService.Dispose();
                    sysService = null;
                }
                this.KeyPreview = true;
            }
            else
            {
                this.KeyPreview = false;
                // create the system service object
                sysService = new SysSvcApi();
                // starts the key hook service.
                // In order to receive the key strokes, the callback function must be registered.
                sysService.KeyEventNotificationSet(new SysSvcApi.KeyEventNotifyCALLBACK(keyEventCallbackProc));
            }
        }

        
    }
}