﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using _Native;

namespace SystemService_CommunicationWithCable
{
    /// <summary>
    /// In order to use the serial communication, you need to buy cradle or serial cable.
    /// 
    /// If you're using a cradle, you simply purchase a standard serial cable from the shops around,
    /// because cradle has the standard serial port.
    /// 
    /// But if you want to communicate without the cradle, you will need to purchase the serial cable from the ATID.
    /// 
    /// And, In order to communicate with an external device, you must use the port "COM4:".
    /// </summary>
    public partial class Form1 : Form
    {
        public const string title = "Communication with cable Example";
        public static bool isHexFormat = false;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            // external port name is "COM4:"
            // you must use the port "COM4:".
            for (int i = 0; i <= 20; i++)
                cbxPortName.Items.Add(string.Format("COM{0}:", i));
            cbxPortName.SelectedIndex = 4;

            cbxBaudrate.Items.Add("2400");
            cbxBaudrate.Items.Add("4800");
            cbxBaudrate.Items.Add("9600");
            cbxBaudrate.Items.Add("14400");
            cbxBaudrate.Items.Add("19200");
            cbxBaudrate.Items.Add("38400");
            cbxBaudrate.Items.Add("57600");
            cbxBaudrate.Items.Add("115200");
            cbxBaudrate.SelectedIndex = 7;

            cbxHandShake.Items.Add("None");
            cbxHandShake.Items.Add("RTS");
            cbxHandShake.Items.Add("RTSXOnXOff");
            cbxHandShake.Items.Add("XOnXOff");
            cbxHandShake.SelectedIndex = 0;

            tbxSendData.Focus();
            tbxSendData.SelectAll();

            radioChr.Checked = true;

            tbxSendData.Text = "test message";

            serialPort1.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(serialPort1_DataReceived);
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (serialPort1.IsOpen)
                serialPort1.Close();

            serialPort1.Dispose();
            serialPort1 = null;
        }

        public delegate void AddString(string str);
        public void AddStringProc(string str)
        {
            if (tbxResponse.Text.Length > 2048)
                tbxResponse.Text = string.Empty;

            tbxResponse.Text = tbxResponse.Text.Insert(tbxResponse.Text.Length, str + "\r\n");
            tbxResponse.Select(tbxResponse.Text.Length, 1);
            tbxResponse.ScrollToCaret();
        }

        void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            string data = string.Empty;
            byte[] buff = new byte[serialPort1.BytesToRead];
            serialPort1.Read(buff, 0, buff.Length);

            if (isHexFormat)
            {
                foreach (byte b in buff)
                    data = data + string.Format("{0:X2} ", b);

                BeginInvoke(new AddString(AddStringProc), data);
            }
            else
            {
                BeginInvoke(new AddString(AddStringProc), Encoding.Default.GetString(buff, 0, buff.Length));
            }
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            string portName = cbxPortName.SelectedItem.ToString();
            int baudRate = Convert.ToInt32(cbxBaudrate.SelectedItem);
            System.IO.Ports.Handshake handShake = System.IO.Ports.Handshake.None;

            if (serialPort1.IsOpen)
            {
                MessageBox.Show("already opened");
                return;
            }

            switch (cbxHandShake.SelectedIndex)
            {
                case 0:
                    handShake = System.IO.Ports.Handshake.None;
                    break;
                case 1:
                    handShake = System.IO.Ports.Handshake.RequestToSend;
                    break;
                case 2:
                    handShake = System.IO.Ports.Handshake.RequestToSendXOnXOff;
                    break;
                case 3:
                    handShake = System.IO.Ports.Handshake.XOnXOff;
                    break;
            }

            try
            {
                //In order to communicate with an external device, you must use the port "COM4:".
                serialPort1.PortName = portName;

                serialPort1.BaudRate = baudRate;
                serialPort1.Handshake = handShake;
                
                serialPort1.Open();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen)
            {
                MessageBox.Show("not opened");
                return;
            }

            serialPort1.Close();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            tbxResponse.Text = string.Empty;
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen)
            {
                MessageBox.Show("not opened");
                return;
            }

            string data = tbxSendData.Text.Trim();
            try
            {
                serialPort1.Write(data);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void radioChr_CheckedChanged(object sender, EventArgs e)
        {
            isHexFormat = !radioChr.Checked;
        }

    }
}