﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;
using NSysSvcApi;
using _Native;

namespace SystemService_IdentifyPDA
{
    public partial class Form1 : Form
    {
        public const string title = "Identify PDA Example";
        public SysSvcApi sysService = null;
        public SYSSVC_RESULT result = SYSSVC_RESULT.SYSSVC_RESULT_FAILURE;

        public enum DEVICENAME : int
        {
            UNKNOWN = 0,
            AT880,
            AT980,
            AT280,
            AT380,
            AB700
        }

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            // create the system service object
            sysService = new SysSvcApi();
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (sysService != null)
            {
                // destruction of the system service object
                sysService.Dispose();
                sysService = null;
            }
        }

        public DEVICENAME GetDeviceName()
        {
            DEVICENAME devName = DEVICENAME.UNKNOWN;

            try
            {
                if (!Utils.IsWinCE) // WM
                {
                    string regValue = (string)Registry.GetValue(@"HKEY_LOCAL_MACHINE\Ident", "Name", string.Empty);
                    if (string.IsNullOrEmpty(regValue))
                        return devName;

                    regValue = regValue.Substring(0, 5);
                    switch (regValue)
                    {
                        case "SI203":
                        case "AT880":
                            devName = DEVICENAME.AT880;
                            break;
                        case "SI303":
                        case "AT980":
                            devName = DEVICENAME.AT980;
                            break;
                        case "SI890":
                        case "AB700":
                            devName = DEVICENAME.AB700;
                            break;
                        case "AT380":
                            devName = DEVICENAME.AT380;
                            break;
                        case "AT280":
                            devName = DEVICENAME.AT280;
                            break;
                    }
                }
                else // CE
                {
                    int regValue = (int)Registry.GetValue(@"HKEY_CURRENT_USER\ControlPanel\KeyTrans", "ModelNumber", 0);
                    devName = (DEVICENAME)regValue;
                }
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }

            return devName;
        }

        private void btnIdentify_Click(object sender, EventArgs e)
        {

            if (sysService == null)
                return;

            tbxBoot.Text = string.Empty;
            tbxOs.Text = string.Empty;
            tbxName.Text = string.Empty;
            tbxDeviceID.Text = string.Empty;

            VERSION_INOFS version = new VERSION_INOFS();
            // Get the version informations
            result = sysService.GetSoftwareVersion(ref version);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                Utils.PlaySound(@"\Windows\Fail.wav");
                MessageBox.Show("GetSoftwareVersion : " + result.ToString());
            }
            else
            {
                // Since you've got a byte array, you need a process of converting to a string.
                // Display Boot version
                tbxBoot.Text = Encoding.Default.GetString(version.ATBTVer, 0, version.ATBTVer.Length);
                // Display OS version
                tbxOs.Text = Encoding.Default.GetString(version.CEOSVer, 0, version.CEOSVer.Length);
            }

            // display device name
            tbxName.Text = GetDeviceName().ToString();
            // display device id
            tbxDeviceID.Text = sysService.DeviceID;
            Utils.PlaySound(@"\Windows\Success.wav");
        }
    }
}