﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NSysSvcApi;
using _Native;

namespace SystemService_KeyHooks
{
    /// <summary>
    /// There is two way that your application will detect the keystrokes.
    /// 1. using KeyPreview property
    ///    -> The application must be the activated state. Otherwise, application will not be able to detect keystrokes.
    /// 2. using Key Hooks
    ///    -> Although the application is deactivated state, application can detect keystrokes.
    /// 
    /// This sample uses the key hooks.
    /// </summary>

    public partial class Form1 : Form
    {
        public const string title = "KeyHooks Example";
        public SysSvcApi sysService = null;
        public SYSSVC_RESULT result = SYSSVC_RESULT.SYSSVC_RESULT_FAILURE;

        public const int WM_KEYUP = 0x0101;
        public const int WM_KEYDOWN = 0x0100;

        public Form1()
        {
            InitializeComponent();
            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            // create the system service object
            sysService = new SysSvcApi();
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (sysService != null)
            {
                // stop the key hook service
                sysService.KeyEventNotificationReset();
                // destruction of the system service object
                sysService.Dispose();
                sysService = null;
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            lbxLog.Items.Clear();
        }

        public void AddString(string str)
        {
            lbxLog.Items.Add(str);
            lbxLog.SelectedIndex = lbxLog.Items.Count - 1;
        }

        /// <summary>
        /// Callback function for receiving a key strokes
        /// </summary>
        /// <param name="KeyValue">key value</param>
        /// <param name="KeyState">key state</param>
        public void keyEventCallbackProc(int KeyValue, int KeyState)
        {
            string msg = string.Format("Value:0x{0:X},  State:{1}", KeyValue, KeyState == WM_KEYUP ? "Up" : "Down");
            AddString(msg);
        }

        private void btnKeyHooks_Click(object sender, EventArgs e)
        {
            if (btnKeyHooks.Text.Equals("Enable key hooks"))
            {
                // starts the key hook service.
                // In order to receive the key strokes, the callback function must be registered.
                result = sysService.KeyEventNotificationSet(new SysSvcApi.KeyEventNotifyCALLBACK(keyEventCallbackProc));
                AddString("Enable : " + result.ToString());
                if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
                {
                    Utils.PlaySound(@"\Windows\Fail.wav");
                }
                else
                {
                    Utils.PlaySound(@"\Windows\Success.wav");
                    btnKeyHooks.Text = "Disable key hooks";
                }
            }
            else
            {
                // stop the key hook service
                result = sysService.KeyEventNotificationReset();
                AddString("Disable : " + result.ToString());
                if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
                {
                    Utils.PlaySound(@"\Windows\Fail.wav");
                }
                else
                {
                    Utils.PlaySound(@"\Windows\Success.wav");
                    btnKeyHooks.Text = "Enable key hooks";
                }
            }
        }
    }
}