﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using NSysSvcApi;
using _Native;

namespace SystemService_PowerControlForRFIDPort
{
    /// <summary>
    /// if you want to communicate with the other device which has been connected RFID port, you must control the pin in below.
    /// PWR_EN, RF_5V, FW_EN
    /// You can do this by using the system service object.
    /// </summary>

    public partial class Form1 : Form
    {
        public const string title = "PowerControl for RFID port Example";
        public SysSvcApi sysService = null;
        public SYSSVC_RESULT result = SYSSVC_RESULT.SYSSVC_RESULT_FAILURE;

        public Form1()
        {
            InitializeComponent();

            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
            }

            // RFID port name is "COM9:"
            serialPort1.PortName = "COM9:";

            serialPort1.BaudRate = 115200;
            serialPort1.Handshake = System.IO.Ports.Handshake.None;
            serialPort1.DataReceived += new System.IO.Ports.SerialDataReceivedEventHandler(serialPort1_DataReceived);

            // create the system service object
            sysService = new SysSvcApi();

            tbxSendData.Text = "1234567890";
        }

        public delegate void AddString(string str);
        public void AddStringProc(string str)
        {
            if (tbxResponse.Text.Length > 2048)
                tbxResponse.Text = string.Empty;

            tbxResponse.Text = tbxResponse.Text.Insert(tbxResponse.Text.Length, str + "\r\n");
            tbxResponse.Select(tbxResponse.Text.Length, 1);
            tbxResponse.ScrollToCaret();
        }

        void serialPort1_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            string data = string.Empty;
            byte[] buff = new byte[serialPort1.BytesToRead];
            serialPort1.Read(buff, 0, buff.Length);

            foreach (byte b in buff)
            {
                data = data + string.Format("{0:X2} ", b);
            }

            BeginInvoke(new AddString(AddStringProc), data);
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            // destruction of the system service object
            if (sysService != null)
            {
                sysService.Dispose();
                sysService = null;
            }
        }

        private void chkPwrEn_CheckStateChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            // controls the pin named PWR_EN.
            result = sysService.SetPWR_EN(chkPwrEn.Checked);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                MessageBox.Show("SetPWR_EN" + result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
            else
            {
                Utils.PlaySound(@"\Windows\Success.wav");
            }
        }

        private void chkRf5V_CheckStateChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            // controls the pin named RF_5V.
            result = sysService.SetRF_5V(chkRf5V.Checked);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                MessageBox.Show("SetRF_5V" + result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
            else
            {
                Utils.PlaySound(@"\Windows\Success.wav");
            }
        }

        private void chkFwEn_CheckStateChanged(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            // controls the pin named FW_EN.
            result = sysService.SetFW_EN(chkFwEn.Checked);
            if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
            {
                MessageBox.Show("SetFW_EN" + result.ToString());
                Utils.PlaySound(@"\Windows\Fail.wav");
            }
            else
            {
                Utils.PlaySound(@"\Windows\Success.wav");
            }
        }

        private void btnSend_Click(object sender, EventArgs e)
        {
            if (sysService == null)
                return;

            if (!serialPort1.IsOpen)
            {
                MessageBox.Show("not opened");
                return;
            }

            string data = tbxSendData.Text.Trim();
            try
            {
                serialPort1.Write(data);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            if (serialPort1 == null)
                return;

            if (serialPort1.IsOpen)
                return;

            try
            {
                serialPort1.Open();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            if (serialPort1 == null)
                return;

            if (!serialPort1.IsOpen)
                return;

            try
            {
                serialPort1.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            tbxResponse.Text = string.Empty;
        }
    }
}