﻿using System;

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using NSysSvcApi;
using _Native;

namespace SystemService_ReceiveSleepWakeupNotification
{
    /// <summary>
    /// When the PDA wakes up from the suspend state or enters the suspend state, you can be notified 
    /// CE ( enters the suspend state and wakes up from the suspend state)
    /// WM ( supported only, wakes up from suspend state)
    /// </summary>
    public partial class Form1 : Form
    {
        public const string title = "Receive SleepWakeup Notification Example";

        #region for Windows CE
        public SysSvcApi sysService = null;
        public SYSSVC_RESULT result = SYSSVC_RESULT.SYSSVC_RESULT_FAILURE;
        #endregion

        #region for Windows Mobile
        [DllImport("PowerNotifications.dll", EntryPoint = "RequestResumeNotificationByMessage", SetLastError = true, CharSet = CharSet.Unicode)]
        internal static extern bool RequestResumeNotificationByMessage(IntPtr hWnd);

        [DllImport("PowerNotifications.dll", EntryPoint = "StopResumeNotification", SetLastError = true, CharSet = CharSet.Unicode)]
        internal static extern bool StopResumeNotification();

        public const int WM_USER = 0x0400;
        public const int WM_WAKEUP_FROM_SUSPENDMODE = WM_USER + 0x12010000;

        NotiMsgWindow msgWnd = null;
        #endregion

        public Form1()
        {
            InitializeComponent();

            this.Text = title;
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            // in CE
            if (Utils.IsWinCE)
            {
                this.Menu = null;
                panel1.Location = new Point(0, 0);
                // create the system service object
                sysService = new SysSvcApi();
            }
            else // in WM
            {
                msgWnd = new NotiMsgWindow();
                msgWnd.OnWndMsg += new NotiMsgWindow.OnWndMsgEventHandler(msgWnd_OnWndMsg);
            }
        }

        private void Form1_Closing(object sender, CancelEventArgs e)
        {
            if (Utils.IsWinCE)
            {
                if (sysService != null)
                {
                    // stop the notify service
                    sysService.SleepWakeupNotificationReset();
                    // destruction of the system service object
                    sysService.Dispose();
                    sysService = null;
                }
            }
            else
            {
                StopResumeNotification();

                if (msgWnd != null)
                {
                    msgWnd.Dispose();
                    msgWnd = null;
                }
            }
        }

        public void AddString(string str)
        {
            if (lbxLog.Items.Count > 500)
                lbxLog.Items.Clear();

            lbxLog.Items.Add(str);
            lbxLog.SelectedIndex = lbxLog.Items.Count - 1;
        }

        /// <summary>
        ///  In Windows CE, Callback function for receiving a notification
        /// </summary>
        /// <param name="enterSleep"></param>
        public void notifyCallbackProc(bool enterSleep)
        {
            AddString(enterSleep ? "+++ enter suspend mode" : "+++ wake up from suspend mode");

            for (int i = 0; i < 3; i++)
            {
                AddString(Convert.ToString(i + 1));
                System.Threading.Thread.Sleep(1000);
            }

            AddString(enterSleep ? "--- enter suspend mode" : "--- wake up from suspend mode");
        }

        /// <summary>
        /// In Windows Moibile, Message Window for receiving a notification message
        /// </summary>
        /// <param name="nMessage"></param>
        /// <param name="wParam"></param>
        /// <param name="lParam"></param>
        void msgWnd_OnWndMsg(int nMessage, IntPtr wParam, IntPtr lParam)
        {
            if (nMessage == WM_WAKEUP_FROM_SUSPENDMODE)
            {
                AddString("+ wake up from suspend mode");
                for (int i = 0; i < 3; i++)
                {
                    AddString(Convert.ToString(i + 1));
                    System.Threading.Thread.Sleep(1000);
                }
                AddString("- wake up from suspend mode");
            }
        }

        private void btnEnable_Click(object sender, EventArgs e)
        {
            // start the notify service

            if (Utils.IsWinCE) // in CE
            {
                if (sysService == null)
                    return;

                result = sysService.SleepWakeupNotificationSet(new SysSvcApi.SleepWakeupNotifyCALLBACK(notifyCallbackProc));
                if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
                    Utils.PlaySound(@"\Windows\Fail.wav");
                else
                    Utils.PlaySound(@"\Windows\Success.wav");

                AddString("Enable Notify : " + result.ToString());
            }
            else // in WM
            {
                if (msgWnd == null)
                    return;

                bool bRet = RequestResumeNotificationByMessage(msgWnd.Hwnd);
                if(bRet)
                    Utils.PlaySound(@"\Windows\Success.wav");
                else
                    Utils.PlaySound(@"\Windows\Fail.wav");

                AddString("Enable Notify : " + (bRet ? "Success" : "Fail"));
            }
        }

        private void btnDisable_Click(object sender, EventArgs e)
        {
            // stop the notify service

            if (Utils.IsWinCE) // in CE
            {
                if (sysService == null)
                    return;
    
                result = sysService.SleepWakeupNotificationReset();
                if (result != SYSSVC_RESULT.SYSSVC_RESULT_SUCCESS)
                    Utils.PlaySound(@"\Windows\Fail.wav");
                else
                    Utils.PlaySound(@"\Windows\Success.wav");

                AddString("Disable Notify : " + result.ToString());
            }
            else // in WM
            {
                bool bRet = StopResumeNotification();
                if (bRet)
                    Utils.PlaySound(@"\Windows\Success.wav");
                else
                    Utils.PlaySound(@"\Windows\Fail.wav");

                AddString("Disable Notify : " + (bRet ? "Success" : "Fail"));
            }
        }
    }
}