#ifndef __CBARCODE__API__H__
#define __CBARCODE__API__H__

#define CBarcodeApi_API extern "C" __declspec(dllexport)

#define MAX_STR		512

#define WM_BARCODE_ONSCANED		WM_USER + 1801

typedef void (CALLBACK* BARCODECALLBACK)();

// 2D Barcode Only
//Capture image size definitions
#define CAP_IMG_VGA			1		//640*480
#define CAP_IMG_QVGA		2		//320*240
#define CAP_IMG_PREVIEW		3		//240*180

#define IMG_FORMAT_JPEG		1
#define IMG_FORMAT_BMP		2

typedef struct
{
	char ImgFormat;
	char ImgSize;
	TCHAR szFilePath[MAX_PATH];
}BARCODECAPTUREPARAMS;


typedef enum
{
	BARCODE_RESULT_SUCCESS = 0,
	BARCODE_RESULT_INVALID_ARGS,
	BARCODE_RESULT_OUTOFMEMORY,
	BARCODE_RESULT_UNSUPPORTED,
	BARCODE_RESULT_ALREADY_OPENED,
	BARCODE_RESULT_FAILURE,
	BARCODE_RESULT_BARCODE_INIT_FAILURE,
	BARCODE_RESULT_NOT_OPENED,
	BARCODE_RESULT_CAM_SELECTED,
	BARCODE_RESULT_ALREADY_BARCODE_SELECTED,
	BARCODE_RESULT_INVALID_DEVICE
}BARCODE_RESULT;

// Common Functions
CBarcodeApi_API BARCODE_RESULT BarcodeOpen(void);
CBarcodeApi_API BARCODE_RESULT BarcodeClose(void);
CBarcodeApi_API BOOL	BarcodeIsOpened(void);

// Barcode Functions
CBarcodeApi_API void	BarcodeSetHwnd(HWND hWnd);
CBarcodeApi_API BARCODE_RESULT BarcodeStart(void);
CBarcodeApi_API BARCODE_RESULT BarcodeStop(void);
CBarcodeApi_API BARCODE_RESULT BarcodeGetScannedData(LPWSTR BarcodeValue, LPWSTR BarcodeTypeName, LPWSTR BarcodeTypeId);
CBarcodeApi_API BARCODE_RESULT BarcodeGetVersionInfo(void);
CBarcodeApi_API BARCODE_RESULT BarcodeSetCallback(BARCODECALLBACK pFunc);

CBarcodeApi_API BARCODE_RESULT BarcodeStartScanRawData(void);
CBarcodeApi_API BARCODE_RESULT BarcodeStopScanRawData(void);
CBarcodeApi_API BARCODE_RESULT BarcodeGetScannedRawData(BYTE * BarcodeValue, WORD * pLength, LPWSTR BarcodeTypeName, LPWSTR BarcodeTypeId);

CBarcodeApi_API BARCODE_RESULT BarcodeStartScanMultiple(DWORD dwTimeout);
CBarcodeApi_API BARCODE_RESULT BarcodeStopScanMultiple();
CBarcodeApi_API BARCODE_RESULT BarcodeStartScanRawDataMultiple(DWORD dwTimeout);
CBarcodeApi_API BARCODE_RESULT BarcodeStopScanRawDataMultiple();

// Cam Functions
CBarcodeApi_API BARCODE_RESULT BarcodeSetPreviewHwnd(HWND hWnd);
CBarcodeApi_API BARCODE_RESULT BarcodeInitCapture();
CBarcodeApi_API BARCODE_RESULT BarcodeDeinitCapture(void);
CBarcodeApi_API BARCODE_RESULT BarcodeStartPreview(void);
CBarcodeApi_API BARCODE_RESULT BarcodeStopPreview(void);
CBarcodeApi_API BARCODE_RESULT BarcodeDoCapture(BARCODECAPTUREPARAMS* pCaptureParams);

// 2D engine's scanning lights mode
CBarcodeApi_API BARCODE_RESULT BarcodeSetScanningLightsMode(int nMode);
CBarcodeApi_API BARCODE_RESULT BarcodeSetLeaveLightsOn(BOOL bEnable);
CBarcodeApi_API BARCODE_RESULT BarcodeGetLeaveLightsOn(BOOL * pbEnable);

// Motorola SE4500
CBarcodeApi_API BARCODE_RESULT BarcodeGetIlluminationPowerLevel(BYTE* pPwrLevel);
CBarcodeApi_API BARCODE_RESULT BarcodeSetIlluminationPowerLevel(BYTE PwrLevel);

typedef struct{
	int Symbol;
	void* pConfigData;
}BARCODE_SYMBOL_CONFIG;

// 1D Only
typedef struct{
	int nLinearCodeSecurityLevel;
	BOOL bBiDirectionalRedundancy;
	int nAngle;
}BARCODE_GENERAL_CONFIG;

// Symbology Config Functions
CBarcodeApi_API BARCODE_RESULT BarcodeEnableSymbologies(BOOL* pSymbolTable);
CBarcodeApi_API BARCODE_RESULT BarcodeDisableSymbologies(BOOL* pSymbolTable);
CBarcodeApi_API BARCODE_RESULT BarcodeEnableSymbologiesAll(BOOL bEnable);
CBarcodeApi_API BARCODE_RESULT BarcodeGetEnableStateAll(BOOL* pSymbolTable);
CBarcodeApi_API BARCODE_RESULT BarcodeSetDefaultSymbol();
CBarcodeApi_API BARCODE_RESULT BarcodeGetSymbologyConfig(BARCODE_SYMBOL_CONFIG* pSymbolConfig);
CBarcodeApi_API BARCODE_RESULT BarcodeSetSymbologyConfig(BARCODE_SYMBOL_CONFIG* pSymbolConfig);
CBarcodeApi_API BARCODE_RESULT BarcodeGetGeneralConfig(BARCODE_GENERAL_CONFIG* pGeneralConfig);
CBarcodeApi_API BARCODE_RESULT BarcodeSetGeneralConfig(BARCODE_GENERAL_CONFIG* pGeneralConfig);

#endif

















