#ifndef __CCAM__API__H__
#define __CCAM__API__H__

#define CCamApi_API extern "C" __declspec(dllexport)


//color space enumerations
typedef enum {
	Raw 			= 0x1, 
	RGB 			= 0x2,
	YCbCr 			= 0x4
} CAM_COL_SP;

//bpp enumerations
typedef enum {
	RawRGB8Bits		= 0x1,
	RGB565 			= 0x2,
	YCbCr422		= 0x4
} CAM_BPP;

//resolution enumerations
typedef enum {
	QQVGA=0, 		//{160, 120}
	QVGA, 		//{320, 240}
	CIF,			//{352, 288}
	VGA,		//{640, 480}
	SVGA, 		//{800, 600}
	XGA, 		//{1024, 768}
	SXGA, 		//{1280, 960}
	UXGA, 		//{1600, 1200}
	QXGA,		//{2048, 1536}
} CAM_RESOL;

//flip and mirror enumerations
typedef enum {
	NO_MIRROR_FLIP = 0x0,
	MIRR 			= 0x1,
	FLIP			= 0x2,
	FLIP_AND_MIRROR	= 0x4,
	MIRR_FLIP_UNSUPPORTED = 0xFF
} FLIP_MIRROR;

//white balance enumerations
typedef enum {
	WB_AUTO		= 0x0,
	WB_CLOUDY		= 0x1,
	WB_SUNNY		= 0x2,
	WB_FLUORESCENT	= 0x4,
	WB_INCANDESCENT	= 0x8,
	WB_UNSUPPORTED	= 0xFF
} WHITE_BAL;

//Saturation enumerations
typedef enum {
	SAT_0X = -4,
	SAT_025X,
	SAT_05X,
	SAT_075X,
	SAT_1X,
	SAT_125X,
	SAT_15X,
	SAT_175X,
	SAT_2X,
} SAT_OPTIONS;

typedef enum {
	PM_NORMAL = 0,
	PM_SPOT,
} PM_OPTIONS;

//Exposure enumerations
typedef enum {
	EV_N1D7 = -5,
	EV_N1D3,
	EV_N1D0,
	EV_N0D7,
	EV_N0D3,
	EV_0,
	EV_P0D3,
	EV_P0D7,
	EV_P1D0,
	EV_P1D3,
	EV_P1D7
} EV_OPTIONS;

//Contrast enumerations
typedef enum {
	CONT_N5 = -5,
	CONT_N4,
	CONT_N3,
	CONT_N2,
	CONT_N1,
	CONT_0,
	CONT_P1,
	CONT_P2,
	CONT_P3,
	CONT_P4,
	CONT_P5
} CONT_OPTIONS;

//effect enumerations
typedef enum {
	NO_EFFECT	= 0,
	EF_SEPIA,
	EF_MONO,
	EF_NEGATIVE,
	EF_RED,
	EF_BLUE,
	EF_GREEN,
	EF_VIOLET
} EFFECT;

//preview mode sensor capability
typedef struct {
	CAM_COL_SP		ColorSpace;		//color space
	CAM_BPP			bpp;			//
	CAM_RESOL		Resol;
	int				Zoom;
	WHITE_BAL		WhiteBalance;
	EV_OPTIONS		Exposure;
	EFFECT			Effect;
	PM_OPTIONS		PhotoMetry;
	FLIP_MIRROR		FlipMirror;
	int				Brightness;
	CONT_OPTIONS	Contrast;
	SAT_OPTIONS		Saturation;
	BOOL			bAutoFocus;
	BOOL			bEnableAF;
	BOOL 			bSupportFlash;
} SENSOR_SETUP_INFO;

typedef enum {
	SEN_CTRL_WHITE_BALANCE,
	SEN_CTRL_EXPOSURE,
	SEN_CTRL_BRIGHTNESS,
	SEN_CTRL_EFFECT,
	SEN_CTRL_PHOTOMETRY,
	SEN_CTRL_FLIP_MIRROR,
	SEN_CTRL_CONTRAST,
	SEN_CTRL_SATURATION,
	SEN_CTRL_AUTOFOCUS
} SEN_CTRL_CODE;	

//image encoding format enumerations
typedef enum {
	IMG_ENCODE_BMP = 1,
	IMG_ENCODE_JPG,
	IMG_ENCODE_GIF,
	IMG_ENCODE_TIFF,
	IMG_ENCODE_EXIF,
	IMG_ENCODE_ICON
} IMG_ENCODE_FORMAT;

//cpature mode setup information
typedef struct {
	SENSOR_SETUP_INFO 	SensorSetup;	
	IMG_ENCODE_FORMAT 	ImgFormat;
	TCHAR pSavePath[MAX_PATH];
	int					nJpegQuality;
} CAPTURE_SETUP_INFO;

//zoom data structure
typedef struct {
	BOOL 		bZoomSupport;
	unsigned int  	MaxZoom;
	int			ZoomStep;
} stZOOM_CAPA;

//Exposure set
typedef struct {
	int			ExposureDefault;
	signed int		ExposureMin;
	signed int 	ExposureMax;
	char			ExposureSteps;
} stEXPOSURE;

//Contrast set
typedef struct {
	int			ContrastDefault;
	signed int		ContrastMin;
	signed int 	ContrastMax;
	char			ContrastSteps;
} stCONTRAST;

//brightness set
typedef struct {
	int			SaturationDefault;
	signed int		SaturationMin;
	signed int 	SaturationMax;
	char			SaturationSteps;
} stSATURATION;

//brightness enumerations set
typedef struct {
	int			BrightDefault;
	signed int		BrightMin;
	signed int 	BrightMax;
	char			BrightSteps;
} stBRIGHTNESS;

//sensor capability data structure
typedef struct {
	DWORD			ColorSpace;	
	DWORD			bpp;	
	DWORD			MaxResol;
	stZOOM_CAPA	ZoomCapa;
	DWORD			WhiteBalance;
	stEXPOSURE		Exposure;
	DWORD			Effect;
	DWORD			PhotoMetry		;	
	DWORD			FlipMirror;
	stBRIGHTNESS	Brightness;
	stCONTRAST		Contrast;
	stSATURATION	Saturation;
	BOOL			bSupportAF;
	BOOL 			bSupportFlash;
} SENSOR_CAPA;

#define CAM_MSG_OFFSET			1000
#define WM_MSG_CAMAPI			WM_USER+CAM_MSG_OFFSET

#define WM_CAM_DRV_CAMERA_INITIALIZED		WM_MSG_CAMAPI+1
#define WM_CAM_DRV_CAPTURE_DONE				WM_MSG_CAMAPI+2
#define WM_CAM_DRV_CAPTURE_START_TO_SAVE  	WM_MSG_CAMAPI+3
#define WM_CAM_DRV_CAPTURE_COMPLETE_SAVE  	WM_MSG_CAMAPI+4
#define WM_CAM_DRV_PREVIEW_DONE				WM_MSG_CAMAPI+5

typedef enum
{
	CAM_RESULT_SUCCESS = 0,
	CAM_RESULT_INVALID_ARGS,
	CAM_RESULT_OUTOFMEMORY,
	CAM_RESULT_UNSUPPORTED,
	CAM_RESULT_ALREADY_OPENED,
	CAM_RESULT_NOT_OPENED,
	CAM_RESULT_FAILURE,
	CAM_RESULT_SCAN_SELECTED,
	CAM_RESULT_ALREADY_CAM_SELECTED,
	CAM_RESULT_INVALID_DEVICE
}CAM_RESULT;

CCamApi_API CAM_RESULT	CamOpen();
CCamApi_API CAM_RESULT	CamClose();
CCamApi_API BOOL		CamIsOpen();
CCamApi_API CAM_RESULT	CamStartPreview();
CCamApi_API CAM_RESULT	CamStopPreview();
CCamApi_API CAM_RESULT	CamPausePreview();
CCamApi_API CAM_RESULT	CamResumePreview();
CCamApi_API CAM_RESULT	CamCapture(CAPTURE_SETUP_INFO *CamCaptureParas);
CCamApi_API CAM_RESULT	CamGetInfo(SENSOR_SETUP_INFO * CamInfo);
CCamApi_API CAM_RESULT	CamGetSensorCapa(SENSOR_CAPA * CamCapa);
CCamApi_API CAM_RESULT	CamSetHandle(HANDLE hwnd, HANDLE picboxHwnd);

//CCamApi_API CAM_RESULT CamZoomPreview(SENSOR_SETUP_INFO *CamParas);
CCamApi_API CAM_RESULT	CamSetWhiteBalance(SENSOR_SETUP_INFO *CamParas);
CCamApi_API CAM_RESULT	CamSetContrast(SENSOR_SETUP_INFO *CamParas);
CCamApi_API CAM_RESULT	CamSetSaturation(SENSOR_SETUP_INFO *CamParas);
CCamApi_API CAM_RESULT	CamSetBrightness(SENSOR_SETUP_INFO *CamParas);
CCamApi_API CAM_RESULT	CamSetEffect(SENSOR_SETUP_INFO *CamParas);
CCamApi_API CAM_RESULT	CamSetFlipMirror(SENSOR_SETUP_INFO *CamParas);
CCamApi_API CAM_RESULT	CamSetAutoFocus(SENSOR_SETUP_INFO *CamParas);
CCamApi_API CAM_RESULT	CamSetAutoFocusEx(int nMode);

CCamApi_API CAM_RESULT	CamEnableFlash(BOOL bEnable);


#endif