#ifndef __CMODEM__API__H__
#define __CMODEM__API__H__

#define CModemApi_API extern "C" __declspec(dllexport)

#include <ras.h>
#include <Raserror.h>

#define MODEM_SYS_SMS_NUM_LEN			20
#define MODEM_SYS_SMS_MSG_LEN			80

typedef struct {
	int			nID;
	SYSTEMTIME	Time;
	int			nStatus;
	WCHAR		ToNum[MODEM_SYS_SMS_NUM_LEN];
	WCHAR		FromNum[MODEM_SYS_SMS_NUM_LEN];
	WCHAR		szMSG[MODEM_SYS_SMS_MSG_LEN];
} MODEM_SYS_SMS_CONTEXT;

/************************************************************************/
/*  Define  RSSI Error Code                                                  */
/************************************************************************/
#define		HSDPA_ERROR_INVALID_MODEM		-1
#define		HSDPA_ERROR_MODEM_TIMEOUT		-2
#define		HSDPA_ERROR_ETC					-3


/************************************************************************/
/*						       FAIL CODE                                */
/************************************************************************/
#define			MODEM_SYS_FAIL				-1
#define			MODEM_SYS_ERROR_RECV		 0
#define			MODEM_SYS_OK_RECV			 1


/************************************************************************/
/*						     MODEM STATUS                               */
/************************************************************************/
typedef	enum{
		MODEM_SYS_STATUS_ON_DEINITIALIZE		= MODEM_SYS_FAIL-1,
		MODEM_SYS_STATUS_FAIL_INITIALIZE		= MODEM_SYS_FAIL,
		MODEM_SYS_STATUS_NOT_READY				= 0x0000,
		MODEM_SYS_STATUS_ON_INITIALIZE			= 0x0001,	// Initialize   . Initialize Into.
		MODEM_SYS_STATUS_ON_POWER_UP			= 0x0002,	// Modem Power On.
		MODEM_SYS_STATUS_COMPLETE_POWER_UP		= 0x0003,
		MODEM_SYS_STATUS_READY					= 0x0004,	// Received "#000"  After Power On 	
		MODEM_SYS_STATUS_COMPLET_INITIALIZE		= 0x0005,	// Initialize Success.
		MODEM_SYS_STATUS_CAN_USE_EXIT			= 0x0006,	// (+)SBIN Close Timeout Alert Start [4/1/2010]
}	MODEM_STATUS;

typedef enum{
		MODEM_POWER_STATUS_OFF	= 0,
		MODEM_POWER_STATUS_ON	= 1,
		MODEM_POWER_STATUS_INITIALIZING,
		MODEM_POWER_STATUS_DEINITIALIZING,
		MODEM_POWER_STATUS_REINITIALIZING
}	MODEM_POWER_STATUS;

typedef enum{
		MODEM_SOCKET_NOT_READY = 0,
		MODEM_SOCKET_READY,
		MODEM_SOCKET_DISCONNECTING,
		MODEM_SOCKET_DISCONNECTED,
		MODEM_SOCKET_CONNECTING,
		MODEM_SOCKET_CONNECTED
}	MODEM_SOCKET_STATUS;

typedef enum{
		NOT_REGISTERED = 0,
		NOT_REGISTERED_BUT_SEARCHING_OPERATOR,
		REGISTRATION_DENIED,
		REGISTERED_TO_GSM,
		REGISTERED_TO_UTRAN,
		UNKNOWN,
		COMMAND_FAILED
}	MODEM_NETWORK_REGISTRATION_STATUS;

//typedef MODEM_SYS_CLASS_MODEM_STATUS MODEM_STATUS;

/************************************************************************/
/*							  MODEM TYPE                                */
/************************************************************************/
typedef	enum{
	MODEM_SYS_TYPE_NONE							= 0x00,
	MODEM_SYS_TYPE_AM200						= 0x01,
	MODEM_SYS_TYPE_AME200						= 0x02,
	MODEM_SYS_TYPE_UC864						= 0x03,
	MODEM_SYS_TYPE_HC25							= 0x04,
	MODEM_SYS_TYPE_HC28							= 0x05,
	MODEM_SYS_TYPE_MC55I						= 0x06,
}	MODEM_SYS_CLASS_MODEM_TYPE;


/************************************************************************/
/*						   SIM PIN STATUS                               */
/************************************************************************/
typedef enum{
	SIM_SUCCESS = 0,
	SIM_READY = 1,
	SIM_INSERTED,
	SIM_PIN,
	SIM_PUK,
	PH_SIM_PIN,
	FSIM_PIN,
	FSIM_PUK,
	SIM_PIN2,
	SIM_PUK2,
	PH_NET_PIN,
	PH_NET_PUK,
	PH_NETSUB_PIN,
	PH_NETSUB_PUK,
	PH_SP_PIN,
	PH_SP_PUK,
	PH_CORP_PIN,
	PH_CORP_PUK,
	SIM_BLOCKED,
	SIM_BUSY,
	SIM_WRONG,
	SIM_FAILURE,
	INVALID_INPUT_VALUE,
	INCORRECT_PASSWORD,
	CARD_HOLDER_TRAY_REMOVED
}MODEM_SIM_PIN_AUTH_STATUS;
#define			MODEM_SYS_SIM_STATUS_READY				1
#define			MODEM_SYS_SIM_STATUS_PIN				MODEM_SYS_SIM_STATUS_READY+1
#define			MODEM_SYS_SIM_STATUS_PUK				MODEM_SYS_SIM_STATUS_READY+2
#define			MODEM_SYS_SIM_STATUS_PH_SIM_PIN			MODEM_SYS_SIM_STATUS_READY+3

#define			MODEM_SYS_SIM_STATUS_FSIM_PIN			MODEM_SYS_SIM_STATUS_READY+4
#define			MODEM_SYS_SIM_STATUS_FSIM_PUK			MODEM_SYS_SIM_STATUS_READY+5

#define			MODEM_SYS_SIM_STATUS_PIN2				MODEM_SYS_SIM_STATUS_READY+6
#define			MODEM_SYS_SIM_STATUS_PUK2				MODEM_SYS_SIM_STATUS_READY+7

#define			MODEM_SYS_SIM_STATUS_PH_NET_PIN			MODEM_SYS_SIM_STATUS_READY+8
#define			MODEM_SYS_SIM_STATUS_PH_NET_PUK			MODEM_SYS_SIM_STATUS_READY+9

#define			MODEM_SYS_SIM_STATUS_PH_NETSUB_PIN		MODEM_SYS_SIM_STATUS_READY+10
#define			MODEM_SYS_SIM_STATUS_PH_NETSUB_PUK		MODEM_SYS_SIM_STATUS_READY+11

#define			MODEM_SYS_SIM_STATUS_PH_SP_PIN			MODEM_SYS_SIM_STATUS_READY+12
#define			MODEM_SYS_SIM_STATUS_PH_SP_PUK			MODEM_SYS_SIM_STATUS_READY+13

#define			MODEM_SYS_SIM_STATUS_PH_CORP_PIN		MODEM_SYS_SIM_STATUS_READY+14
#define			MODEM_SYS_SIM_STATUS_PH_CORP_PUK		MODEM_SYS_SIM_STATUS_READY+15


/************************************************************************/
/*                             Define MAX                               */
/************************************************************************/
#define			MODEM_SYS_MAX_PHONENUMBER				0x20


/************************************************************************/
/*                      User Window Index Define                        */
/************************************************************************/
typedef	enum{
		MODEM_SYS_USER_WND_BASIC,
		MODEM_SYS_USER_WND_CALL,
		MODEM_SYS_USER_WND_SMS,
		MODEM_SYS_USER_WND_SOCKET,
		MODEM_SYS_USER_WND_ALL,
}MODEM_SYS_INDEX_USER_WND;


/************************************************************************/
/*					  DEFINE USER WINDOW MESSAGE						*/
/************************************************************************/
#define			MODEM_SYS_NOTI_PWR_UP_INITIALIZE			WM_USER+0x1000
#define			MODEM_SYS_NOTI_PWR_UP_SUCCESS				WM_USER+0x1001
#define			MODEM_SYS_NOTI_PWR_UP_FAIL					WM_USER+0x1002

#define			MODEM_SYS_NOTI_PORT_OPEN_SUCCESS			WM_USER+0x1003
#define			MODEM_SYS_NOTI_PORT_OPEN_FAIL				WM_USER+0x1004

#define			MODEM_SYS_NOTI_PORT_CLOSE_SUCCESS			WM_USER+0x1005
#define			MODEM_SYS_NOTI_PORT_CLOSE_FAIL				WM_USER+0x1006

#define			MODEM_SYS_NOTI_PWR_DOWN_SUCCESS				WM_USER+0x1007
#define			MODEM_SYS_NOTI_PWR_DOWN_FAIL				WM_USER+0x1008

#define 		MODEM_SYS_NOTI_BATT_DETECT					WM_USER+0x1009
#define 		MODEM_SYS_NOTI_BATT_NOT_DETECT				WM_USER+0x100A
#define			MODEM_SYS_NOTI_BATT_LOW						WM_USER+0x100B

#define			MODEM_SYS_NOTI_PWR_UP_REINITIALIZE			WM_USER+0x100C

#define			MODEM_SYS_NOTI_NETWORK_REGISTERED			WM_USER+0x100D
#define			MODEM_SYS_NOTI_NETWORK_NOT_REGISTERED		WM_USER+0x100E

#define			MODEM_SYS_NOTIFICATION_CALL_TIMEOUT			WM_USER+0x1100
#define			MODEM_SYS_NOTIFICATION_CALL_DIALING			MODEM_SYS_NOTIFICATION_CALL_TIMEOUT+1	// OUT
#define			MODEM_SYS_NOTIFICATION_CALL_RINGING			MODEM_SYS_NOTIFICATION_CALL_TIMEOUT+2	// OUT
#define			MODEM_SYS_NOTIFICATION_CALL_RING			MODEM_SYS_NOTIFICATION_CALL_TIMEOUT+3	// In
#define			MODEM_SYS_NOTIFICATION_CALL_CONNECTED		MODEM_SYS_NOTIFICATION_CALL_TIMEOUT+4
#define			MODEM_SYS_NOTIFICATION_CALL_DISCONNECTED	MODEM_SYS_NOTIFICATION_CALL_TIMEOUT+5
#define			MODEM_SYS_NOTIFICATION_CALL_RELEASED		MODEM_SYS_NOTIFICATION_CALL_DISCONNECTED	
#define			MODEM_SYS_NOTIFICATION_CALL_NOCARRIER		MODEM_SYS_NOTIFICATION_CALL_TIMEOUT+6

#define			MODEM_SYS_NOTIFICATION_SIGNAL				WM_USER+0x1200

#define			MODEM_SYS_NOTIFICATION_SOCKET							WM_USER+0x1300
#define			MODEM_SYS_NOTIFICATION_SOCKET_INIT_SUCCESS				MODEM_SYS_NOTIFICATION_SOCKET+1
#define			MODEM_SYS_NOTIFICATION_SOCKET_INIT_FAIL					MODEM_SYS_NOTIFICATION_SOCKET+2
#define			MODEM_SYS_NOTIFICATION_SOCKET_DEINIT_SUCCESS			MODEM_SYS_NOTIFICATION_SOCKET+3
#define			MODEM_SYS_NOTIFICATION_SOCKET_DEINIT_FAIL				MODEM_SYS_NOTIFICATION_SOCKET+4
#define			MODEM_SYS_NOTIFICATION_SOCKET_CONNECT_SUCCESS			MODEM_SYS_NOTIFICATION_SOCKET+5
#define			MODEM_SYS_NOTIFICATION_SOCKET_CONNECT_FAIL				MODEM_SYS_NOTIFICATION_SOCKET+6
#define			MODEM_SYS_NOTIFICATION_SOCKET_DISCONNECT_SUCCESS		MODEM_SYS_NOTIFICATION_SOCKET+7
#define			MODEM_SYS_NOTIFICATION_SOCKET_DISCONNECT_FAIL			MODEM_SYS_NOTIFICATION_SOCKET+8
#define			MODEM_SYS_NOTIFICATION_SOCKET_RECEIVED_DATA				MODEM_SYS_NOTIFICATION_SOCKET+9
#define			MODEM_SYS_NOTIFICATION_SOCKET_NOCARRIER					MODEM_SYS_NOTIFICATION_CALL_NOCARRIER

typedef enum
{
	RCS_OpenPort = 0,
	RCS_Connecting,
	RCS_Connected,
	RCS_Disconnected
}RAS_CONNECTION_STATUS;

typedef struct{
	MODEM_SYS_INDEX_USER_WND nWndIndex;
	int m_UserWnMsg;
	BOOL bBatteryDetect;
	WPARAM wParam;
	LPARAM lParam;
}MODEM_CALLBACK_DATA;

typedef void (CALLBACK* MODEMCALLBACK)(MODEM_SYS_INDEX_USER_WND nWndIndex, int m_UserWnMsg, BOOL bBatteryDetect, WPARAM wParam, LPARAM lParam);

typedef enum
{
	MODEM_RESULT_SUCCESS			= 0,
	MODEM_RESULT_INVALID_ARGS,
	MODEM_RESULT_OUTOFMEMORY,
	MODEM_RESULT_UNSUPPORTED,
	MODEM_RESULT_ALREADY_OPENED,
	MODEM_RESULT_NOT_OPENED,
	MODEM_RESULT_FAILURE,
	MODEM_RESULT_NO_BATTERY,
	MODEM_RESULT_LOW_BATTERY,
	MODEM_RESULT_TIMEOUT,
	MODEM_RESULT_ALREADY_ALLOCATED,
	MODEM_RESULT_NOT_POWER_ON,
	MODEM_RESULT_ALREADY_POWER_ON
}MODEM_RESULT;

#define WM_MODEM_RESPONSE		WM_USER + 1806

// General
CModemApi_API MODEM_RESULT AllocModemContext();
CModemApi_API MODEM_RESULT DeallocModemContext();
CModemApi_API MODEM_RESULT ModemPowerUp();
CModemApi_API MODEM_RESULT ModemPowerDown();
CModemApi_API MODEM_RESULT ModemPowerEnable(BOOL bEnable);
CModemApi_API MODEM_RESULT ModemOpen();
CModemApi_API MODEM_RESULT ModemClose();
CModemApi_API BOOL	 ModemIsOpened();
CModemApi_API BOOL	 ModemIsOpen();
CModemApi_API MODEM_RESULT ModemSetHwnd(HWND hWnd);
CModemApi_API MODEM_RESULT ModemSetCallback(MODEMCALLBACK pFunc);
CModemApi_API MODEM_STATUS ModemGetStatus();
CModemApi_API MODEM_POWER_STATUS ModemGetPowerStatus();
CModemApi_API MODEM_RESULT ModemReset();
CModemApi_API BOOL	ModemIsRegistered();
CModemApi_API MODEM_NETWORK_REGISTRATION_STATUS ModemGetNetworkRegistrationStatus();
CModemApi_API MODEM_RESULT ModemGetFwVersion(LPWSTR version);
CModemApi_API MODEM_RESULT ModemGetTime(LPWSTR time);
 
CModemApi_API MODEM_RESULT ModemGetManufacturerId(LPWSTR szManufacturerID);
CModemApi_API MODEM_RESULT ModemGetModelId(LPWSTR szModelID);
CModemApi_API MODEM_RESULT ModemGetRevisionId(LPWSTR szRevisionID);
CModemApi_API MODEM_RESULT ModemGetCapabilitiesList(LPWSTR szCapabilities);
CModemApi_API MODEM_RESULT ModemGetSerialNumber(LPWSTR szSerialNumber);
CModemApi_API MODEM_RESULT ModemGetPhoneNumber(LPWSTR szPhoneNumber);
CModemApi_API MODEM_RESULT ModemGetRSSI(int* pnRSSI);
CModemApi_API MODEM_RESULT ModemCertificateSimPin(LPWSTR szPin, LPWSTR szNewPin);

// Sim
CModemApi_API MODEM_SIM_PIN_AUTH_STATUS	ModemSimQueryCardHolderStatus();
CModemApi_API MODEM_SIM_PIN_AUTH_STATUS	ModemSimQueryAuthStatus();
CModemApi_API MODEM_SIM_PIN_AUTH_STATUS	ModemSimEnterPin(LPWSTR szPin);
CModemApi_API MODEM_SIM_PIN_AUTH_STATUS ModemSimEnterPuk(LPWSTR szPuk, LPWSTR szNewPin);
CModemApi_API MODEM_SIM_PIN_AUTH_STATUS	ModemSimChangePin(LPWSTR szOldPin, LPWSTR szNewPin);
CModemApi_API MODEM_SIM_PIN_AUTH_STATUS	ModemSimLockUnlock(LPWSTR szPinNum, BOOL bLock);
CModemApi_API MODEM_SIM_PIN_AUTH_STATUS	ModemSimGetLockStatus(BOOL* pbActive);
CModemApi_API MODEM_SIM_PIN_AUTH_STATUS	ModemSimGetPinCounter(int* pnPinCounter);

// Sms
CModemApi_API MODEM_RESULT ModemSmsSendMessage(LPWSTR szToNum, LPWSTR szFromNum, LPWSTR szMessage);
CModemApi_API MODEM_RESULT ModemSmsReadMessage(int nID, MODEM_SYS_SMS_CONTEXT* pSMSContext);
CModemApi_API MODEM_RESULT ModemSmsReadSentMessage(int nID, MODEM_SYS_SMS_CONTEXT* pSMSContext);
CModemApi_API MODEM_RESULT ModemSmsReadNewMessage(int nID, MODEM_SYS_SMS_CONTEXT* pSMSContext);
CModemApi_API MODEM_RESULT ModemSmsReadOldMessage(int nID, MODEM_SYS_SMS_CONTEXT* pSMSContext);
CModemApi_API MODEM_RESULT ModemSmsSetMark(BOOL bRead);
CModemApi_API MODEM_RESULT ModemSmsGetCountOfMessage(int* pnSmsAll);
CModemApi_API MODEM_RESULT ModemSmsGetCountOfNewMessage(int* pnSmsNew);

// Call
CModemApi_API MODEM_RESULT ModemCallDialUp(LPWSTR szToNum);
CModemApi_API MODEM_RESULT ModemCallHangUp();
CModemApi_API MODEM_RESULT ModemCallPickUp();
CModemApi_API MODEM_RESULT ModemCallGetConnectedNumber(LPWSTR szConnectedNumber);

// Port
CModemApi_API MODEM_RESULT ModemPortRequest();
CModemApi_API MODEM_RESULT ModemPortReleased();

// Single Socket
CModemApi_API MODEM_RESULT ModemSocketInit();
CModemApi_API MODEM_RESULT ModemSocketDeinit();
CModemApi_API MODEM_RESULT ModemSocketConnect(LPWSTR lpwszIPAddress, int nPort);
CModemApi_API MODEM_RESULT ModemSocketDisconnect();
CModemApi_API MODEM_RESULT ModemSocketRead(LPSTR pData, int nLen);
CModemApi_API MODEM_RESULT ModemSocketWrite(LPSTR pData, int nLen);
CModemApi_API MODEM_RESULT ModemSocketSetEndChar(UCHAR Char);
CModemApi_API MODEM_SOCKET_STATUS ModemGetSocketStatus();

// RAS
CModemApi_API DWORD ModemRasDial(LPRASDIALEXTENSIONS dialExtensions, LPTSTR phoneBookPath, LPRASDIALPARAMS rasDialParam, DWORD NotifierType, LPVOID notifier, LPHRASCONN pRasConn);
CModemApi_API DWORD ModemRasHangUp(HRASCONN Session);
#endif