#ifndef __CPRINTER__API__H__
#define __CPRINTER__API__H__

#define CPrinterApi_API extern "C" __declspec(dllexport)


// Paper Types

//Nippon Paper
#define TF50KS_E2D_E			0.2489
#define TF50KS_E2D_T			0.002885
#define TP50KJ_R_E				0.2745
#define TP50KJ_R_T				0.00295
#define TL69KS_LH_E				0.3286
#define TL69KS_LH_T				0.002341
//Oji Paper
#define PD160R_63_E				0.2393
#define PD160R_63_T				0.001678
#define PD160R_N_E				0.3068
#define PD160R_N_T				0.003668
//Mitsubishi Paper mills limited
#define P220VBB_1_E				0.2864
#define P220VBB_1_T				0.003028
//Jujo Thermal
#define AP50KS_D_25C_HI_E		0.2796
#define AP50KS_D_25C_HI_T		0.003093
#define AP50KS_D_25C_LO_E		0.2798
#define AP50KS_D_25C_LO_T		0.002085
#define AP50KS_E_E				0.2651
#define AP50KS_E_T				0.002981
#define AP50KS_FZ_E				0.2934
#define AP50KS_FZ_T				0.002707
//Mitsubishi Hi-Tech Paper
#define F5041_E					0.2576
#define F5041_T					0.002847
#define P5045_E					0.3262
#define P5045_T					0.003601
//Papierfabrix August Koehler AG
#define KT55F20_E				0.3068
#define KT55F20_T				0.003668
//KSP
#define P300_E					0.2703
#define P300_T					0.002797
#define P350_E					0.2462
#define P350_T					0.002723
#define P350_20_E				0.2347
#define P350_20_T				0.002778
#define KIP370_E					0.3206
#define KIP370_T					0.002328
#define KIP470_E					0.2759
#define KIP470_T					0.003447
//KANZAN
#define KF50_E					0.2774
#define KF50_T					0.002183
#define KPR440_E				0.2966
#define KPR440_T				0.001951
//Appleton
#define ALPHA900_34_E			0.2476
#define ALPHA900_34_T			0.003539

#define PRT_ALIGN_LEFT		TA_LEFT		// 0
#define PRT_ALIGN_CENTER	TA_CENTER	// 6
#define PRT_ALIGN_RIGHT		TA_RIGHT		// 2

#define PRT_SIZE_GREAT_LARGE	10
#define PRT_SIZE_LARGE			9
#define PRT_SIZE_NORMAL		8
#define PRT_SIZE_SMALL			7
#define PRT_SIZE_GREAT_SMALL	5

/*
#define PRT_BOTTOM_LARGE		56
#define PRT_BOTTOM_NORMAL		39
#define PRT_BOTTOM_SMALL		28
*/

/* Font Height */
#define PRT_HEIGHT_NORMAL		2
#define PRT_HEIGHT_DOUBLE		4


#define PRT_LOW					1		
#define PRT_NORMAL				2		
#define PRT_FINE 				4		


/* Font Weights */
#define PRT_WEIGHT_NORMAL		FW_NORMAL
#define PRT_WEIGHT_BOLD			FW_BOLD

//(2014-01-06)2D barcode size add MunHG
#define PRT_IMG_SIZE_LARGE			200
#define PRT_IMG_SIZE_NORMAL			150
#define PRT_IMG_SIZE_SMALL			100

typedef struct {
	double	Energy;		// Standard Printing energy ( mJ )
	double	TempCoeffi;	// Temperature coefficeient
} PRT_PAPER_INFO, *PPRT_PAPER_INFO;

typedef enum {
	PRT_PAPER_ENERGY,	
	PRT_PAPER_TEMPCOEFF,	
	PRT_DENSITY_INFO,		
	PRT_LINE_SPACE	,
	PRT_RECT_BOTTOM,
	PRT_LINE_QUALITY,
	PRT_LINE_HEIGHT
} SET_PRT_INFO;

typedef struct {
	SET_PRT_INFO	SetPrtInfo;
	double			PaperInfo;
} PRT_PAGE_INFO, *PPRT_PAGE_INFO;

typedef struct {
	BOOL	PlatenDetect;
	BOOL	PaperDetect;
} PRT_STATUS, *PPRT_STATUS;

typedef enum {
	M_DIR_BACKWARD,
	M_DIR_FORWARD
} MOTOR_DIRECT;

typedef struct {
	MOTOR_DIRECT FeedDirect;		//Motor accelator step table index, Max value index
	DWORD FeedCount;
} PAPER_FEED;

typedef struct
{
	int Alignment;
	int Size;
	int Weight;
	int Height;
	int WordSpace;
	int WordWidth;
	BOOL bUnderline;
}PRINTOPT, *PPRINTOPT;

typedef struct
{
	TCHAR szString[MAX_PATH];
	PRINTOPT Option;
}PRINTER_TEXT, *PPRINTER_TEXT;

typedef enum
{
	BCODE_TYPE_UNSPECIFIED = 0,
	BCODE_TYPE_UPCA,
	BCODE_TYPE_UPCE,
	BCODE_TYPE_UPC_SUPPLEMENTAL_2DIGIT,
	BCODE_TYPE_UPC_SUPPLEMENTAL_5DIGIT,
	BCODE_TYPE_EAN13,
	BCODE_TYPE_EAN8,
	BCODE_TYPE_INTERLEAVED2OF5,
	BCODE_TYPE_STANDARD2Of5,
	BCODE_TYPE_CODE39 = 10,
	BCODE_TYPE_CODE39EXTENDED,
	BCODE_TYPE_CODABAR,
	BCODE_TYPE_POSTNET,
	BCODE_TYPE_BOOKLAND,
	BCODE_TYPE_ISBN,
	BCODE_TYPE_JAN13,
	BCODE_TYPE_MSI_MOD10,
	BCODE_TYPE_MSI_2MOD10,
	BCODE_TYPE_MSI_MOD11,
	BCODE_TYPE_MSI_MOD11_MOD10,
	BCODE_TYPE_PLESSEY,
	BCODE_TYPE_CODE11,
	BCODE_TYPE_CODE128 = 27,
	BCODE_TYPE_CODE128A,
	BCODE_TYPE_CODE128B,
	BCODE_TYPE_CODE128C,
	BCODE_TYPE_CODE93 = 32,
	BCODE_TYPE_TELEPEN,

	//(2014-01-03)2D barcode add Munhg
	BCODE_TYPE_AZTEC = 64,
	BCODE_TYPE_DATA_MATRIX = 512,
	BCODE_TYPE_ITF = 1024,
	BCODE_TYPE_MAXICODE = 2048,
	BCODE_TYPE_PDF_417 = 4096,
	BCODE_TYPE_QR_CODE = 8192,
	BCODE_TYPE_RSS_14 = 16384,
	BCODE_TYPE_RSS_EXPANDED = 32768,
	BCODE_TYPE_UPC_EAN_EXTENSION = 65536,
}BARCODE_TYPE;

#define CODE39_MAX_LEN				12
#define CODE93_MAX_LEN				17
#define CODE128_MAX_LEN				13

#define INTERLEAVED2OF5_MAX_LEN		22
#define CODABAR_MAX_LEN				19
#define MSI_MAX_LEN					14
#define EAN8_MAX_LEN				7
#define EAN13_MAX_LEN				12


#define QRCODE_MAX_LEN				500
typedef enum
{
	PRT_RESULT_SUCCESS = 0,
	PRT_RESULT_INVALID_ARGS,
	PRT_RESULT_OUTOFMEMORY,
	PRT_RESULT_UNSUPPORTED,
	PRT_RESULT_ALREADY_OPENED,
	PRT_RESULT_NOT_OPENED,
	PRT_RESULT_FAILURE,
	PRT_RESULT_INSUFFICIENT_BATTERY,
	PRT_RESULT_INVALID_DEVICE,
	PRT_RESULT_ALREADY_PRINTERSTART,
	PRT_RESULT_NOT_PRINTERSTART
}PRT_RESULT;


#define WM_PRT_STATUS		WM_USER + 1808
typedef void (CALLBACK* PRINTERCALLBACK)(PRT_STATUS PrtStatus);


CPrinterApi_API PRT_RESULT	PrinterOpen();
CPrinterApi_API PRT_RESULT	PrinterClose();
CPrinterApi_API PRT_RESULT	PrinterStart();
CPrinterApi_API PRT_RESULT	PrinterStop();
CPrinterApi_API BOOL	PrinterIsOpen();
CPrinterApi_API PRT_RESULT	PrinterFeedPaper(DWORD nPagerFeed);
CPrinterApi_API PRT_RESULT	PrinterPrintText(LPWSTR szString, int Alignment, int Size, int Weight, int Height, int WordSpace, int WordWidth, BOOL bUnderline);
CPrinterApi_API PRT_RESULT	PrinterPrintBitmap(UCHAR Alignment, LPWSTR szFileName);
CPrinterApi_API PRT_RESULT	PrinterPrintBarcode(BARCODE_TYPE Type, LPWSTR szBarcodeValue, UCHAR Algnment, BOOL bIncludeLabel);
CPrinterApi_API PRT_RESULT	PrinterPrintBarcodeEx(BARCODE_TYPE Type, LPWSTR szBarcodeValue, UCHAR Algnment, BOOL bIncludeLabel, UINT imgSize);
CPrinterApi_API PRT_RESULT	PrinterGetLineSpace(UINT* pnLineSpace);
CPrinterApi_API PRT_RESULT	PrinterSetLineSpace(UINT nLineSpace);
CPrinterApi_API PRT_RESULT	PrinterGetDensity(UINT* pnDensity);
CPrinterApi_API PRT_RESULT	PrinterSetDensity(UINT nDensity);
CPrinterApi_API PRT_RESULT	PrinterGetWordHeight(UINT* pnHeight);//PrinterGetQuality(UINT* pnQuality);
CPrinterApi_API PRT_RESULT	PrinterSetWordHeight(UINT nHeight);	
CPrinterApi_API PRT_RESULT	PrinterGetQuality(UINT* pnQuality);
CPrinterApi_API PRT_RESULT	PrinterSetQuality(UINT nQuality);	
CPrinterApi_API PRT_RESULT	PrinterGetPaperType(PRT_PAPER_INFO* pPaperInfo);
CPrinterApi_API PRT_RESULT	PrinterSetPaperType(PRT_PAPER_INFO PaperInfo);
CPrinterApi_API PRT_RESULT	PrinterGetStatus(PRT_STATUS *pPrtStatus);
CPrinterApi_API PRT_RESULT	PrinterSetCallback(PRINTERCALLBACK pfnCallback);
CPrinterApi_API PRT_RESULT	PrinterSetHwnd(HWND hWnd);
//CPrinterApi_API PRT_RESULT	PrinterEndPage();

#endif