#ifndef __CSMARTCARD__API__H__
#define __CSMARTCARD__API__H__

#define CSmartCardApi_API extern "C" __declspec(dllexport)

#define RATE_600	0x00
#define RATE_1200	0x01
#define RATE_2400	0x02
#define RATE_4800	0x03
#define RATE_9600	0x04
#define RATE_14400	0x05
#define RATE_19200	0x06
#define RATE_28800	0x07
#define RATE_38400	0x08
#define RATE_57600	0x09
#define RATE_115200	0x0A //default

// Icc Power On.
#define EMV_MODE	0
#define ISO_MODE	1

#define AUTO_VOLTAGE_SELECTION  0x00
#define VOLTS_5_0               0x01
#define VOLTS_3_0               0x02
#define VOLTS_1_8               0x03

// Set Parameters.
#define PROTOCOL_T0 0
#define PROTOCOL_T1 1

// the following #defines are used on the bStatus byte
#define BSTATUS_ERROR_MASK          0xe0 // bits 5,6 & 7
#define BSTATUS_ICC_STATUS_MASK     0x0f // bits 0,1 & 2
#define BSTATUS_FAILED              0x40
#define BSTATUS_TIME_EXT            0x80
#define ICC_PRESENT_AND_ACTIVE      0
#define ICC_PRESENT_AND_INACTIVE    1
#define ICC_NOT_PRESENT             2
#define ICC_FAILED					8

// the following #defines are used on the bError byte
#define CMD_ABORTED			0xff
#define ICC_MUTE			0xfe
#define HW_ERROR			0xfb
#define BAD_ATR_TS			0xf8
#define BAD_ATR_TCK			0xf7
#define COMM_TIMEOUT		0xf6
#define RECEIVED_NAK		0xf5
#define ERROR_NONE			0xf4
#define CMD_FAILED			0xf3


typedef enum
{
	SCARD_RESULT_SUCCESS		= 0,
	SCARD_RESULT_INVALID_ARGS,
	SCARD_RESULT_OUTOFMEMORY,
	SCARD_RESULT_UNSUPPORTED,
	SCARD_RESULT_ALREADY_OPENED,
	SCARD_RESULT_NOT_OPENED,
	SCARD_RESULT_FAILURE,
	SCARD_RESULT_INVALID_DEVICE
}SCARD_RESULT;



CSmartCardApi_API SCARD_RESULT SCardProtocolInit();
CSmartCardApi_API SCARD_RESULT SCardProtocolDeinit();
CSmartCardApi_API SCARD_RESULT SCardSetParameters(UCHAR Slot, UCHAR T_Protocol, UCHAR FiDi, UCHAR* pStatus, UCHAR* pError);
CSmartCardApi_API SCARD_RESULT SCardGetParameters(UCHAR Slot, UCHAR* pT_Protocol, UCHAR* pFiDi, UCHAR* pStatus, UCHAR* pError);
CSmartCardApi_API SCARD_RESULT SCardSetDefaultParameter(UCHAR Slot, UCHAR* pStatus, UCHAR* pError);
CSmartCardApi_API SCARD_RESULT SCardPowerOn(UCHAR Slot, UCHAR Mode, UCHAR Powersel, UCHAR* pStatus, UCHAR* pError, UCHAR* pAtr, UINT* pLengthOfAtr);
CSmartCardApi_API SCARD_RESULT SCardPowerOff(UCHAR Slot, UCHAR* pStatus, UCHAR* pError);
CSmartCardApi_API SCARD_RESULT SCardGetSlotStatus(UCHAR Slot, UCHAR* pStatus, UCHAR* pError);
CSmartCardApi_API SCARD_RESULT SCardTransferBlock(UCHAR Slot, UINT LengthOfTxData, UCHAR* pTxData, UINT* pLengthOfRxData, UCHAR* pRxData, UCHAR* pStatus, UCHAR* pError);
CSmartCardApi_API SCARD_RESULT SCardGetFWVersion(UCHAR Slot, UCHAR* pVersion, UCHAR* pStatus, UCHAR* pError);

#endif